// **************************************************************************************
// *
// * © 2012 Dell, Inc.
// * ALL RIGHTS RESERVED.
// *
// * Redistribution and use in source and binary forms, with or without modification, 
// * are permitted provided that the following conditions are met:
// * 
// *  *  Redistributions of source code must retain the above copyright notice, 
// *     this list of conditions and the following disclaimer.  
// * 
// *  *  Redistributions in binary form must reproduce the above copyright notice, 
// *     this list of conditions and the following disclaimer in the documentation 
// *     and/or other materials provided with the distribution.  
// * 
// *  *  Neither the name of Dell, Inc. nor the names of its contributors 
// *     may be used to endorse or promote products derived from this software 
// *     without specific prior written permission. 
// * 
// * 
// * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
// * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
// * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
// * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
// * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
// * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
// * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
// * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
// * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// * 
// **************************************************************************************

#include "mainhelpdialog.h"
#include "ui_mainhelpdialog.h"

MainHelpDialog::MainHelpDialog(QWidget *parent) :
    CommonDialog(parent),
    ui(new Ui::MainHelpDialog)
{
    ui->setupUi(this);

    loadHelpText();

    setWindowFlags(Qt::FramelessWindowHint | Qt::Dialog);
    setWindowMask( "", width() -11 , height() -11 );
}

MainHelpDialog::~MainHelpDialog()
{
    delete ui;
}

void MainHelpDialog::changeEvent(QEvent *e)
{
    QDialog::changeEvent(e);
    switch (e->type()) {
    case QEvent::LanguageChange:
        ui->retranslateUi(this);
        break;
    default:
        break;
    }
}

void MainHelpDialog::on_helpViewerToolbox_currentChanged(int index)
{

}

void MainHelpDialog::resizeEvent( QResizeEvent * event )
{
    CommonDialog::resizeEvent( event );
    setWindowMask( "", event->size().width() -11 , event->size().height() -11 );
}

void MainHelpDialog::loadHelpText( )
{
    // Create the list of files to be loaded

    loadRefreshButtonPage();
    loadActiveConnectionsPage();
    loadAddFarmPage();
    loadAddLocationPage();
    loadApplicationsPage();
    loadConnectToFarmPage();
    loadDeleteFarmPage();
    loadDeleteLocationPage();
    loadEditFarmPage();
    loadEditLocationPage();
    loadOverviewPage();

}


void MainHelpDialog::loadRefreshButtonPage()
{
    loadHelpPage( QString("refreshbutton.htm"), ui->refreshButtonTextEdit );
}

void MainHelpDialog::loadActiveConnectionsPage()
{
    loadHelpPage( QString("activeconnections.htm"), ui->activeConnectionsTextEdit );
}

void MainHelpDialog::loadAddFarmPage()
{
    loadHelpPage( QString("addfarm.htm"), ui->addFarmTextEdit );
}

void MainHelpDialog::loadAddLocationPage()
{
    loadHelpPage( QString("addlocation.htm"), ui->addLocationTextEdit );
}

void MainHelpDialog::loadApplicationsPage()
{
    loadHelpPage( QString("applications.htm"), ui->applicationsTextEdit );
}

void MainHelpDialog::loadConnectToFarmPage()
{
    loadHelpPage( QString("connectingtoafarm.htm"), ui->connectingToFarmTextEdit );
}
void MainHelpDialog::loadDeleteFarmPage()
{
    loadHelpPage( QString("deletefarm.htm"), ui->deleteFarmTextEdit );
}
void MainHelpDialog::loadDeleteLocationPage()
{
    loadHelpPage( QString("deletelocation.htm"), ui->deleteLocationTextEdit );
}
void MainHelpDialog::loadEditFarmPage()
{
    loadHelpPage( QString("editfarm.htm"), ui->editFarmTextEdit );
}
void MainHelpDialog::loadEditLocationPage()
{
    loadHelpPage( QString("editlocation.htm"), ui->editLocationTextEdit );
}
void MainHelpDialog::loadOverviewPage()
{
    loadHelpPage( QString("overview.htm"), ui->overviewTextEdit );
}








void MainHelpDialog::loadHelpPage( QString fileName, QTextEdit * displayWidget )
{
    QString fullpath;

    fullpath.append( "/usr/local/share/qrdesktop/help/");
    fullpath.append( fileName );
    QFile file( fullpath );

    if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
        return;
    QByteArray bytes = file.readAll();

    displayWidget->setHtml(QString( bytes));

}
