// **************************************************************************************
// *
// * © 2012 Dell, Inc.
// * ALL RIGHTS RESERVED.
// *
// * Redistribution and use in source and binary forms, with or without modification, 
// * are permitted provided that the following conditions are met:
// * 
// *  *  Redistributions of source code must retain the above copyright notice, 
// *     this list of conditions and the following disclaimer.  
// * 
// *  *  Redistributions in binary form must reproduce the above copyright notice, 
// *     this list of conditions and the following disclaimer in the documentation 
// *     and/or other materials provided with the distribution.  
// * 
// *  *  Neither the name of Dell, Inc. nor the names of its contributors 
// *     may be used to endorse or promote products derived from this software 
// *     without specific prior written permission. 
// * 
// * 
// * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
// * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
// * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
// * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
// * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
// * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
// * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
// * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
// * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// * 
// **************************************************************************************

#include <stdexcept>
#include "farmSettingsNames.h"
#include "credentialsPage.h"


CredentialsPage::CredentialsPage(Farm* farm, ManageConnectionDialog* parent)
    : ManageConnectionPage(farm, parent)
{
    setupUi(this);

    bool connected = connect(donePushButton, SIGNAL(clicked()), parent, SLOT(accept()));
    if (!connected)
    {
        throw std::runtime_error("QT's signal and slot connection error in CredentialsPage constructor");
    }

    loadFarmSettings();
}

void CredentialsPage::loadFarmSettings()
{
    // Farm doesn`t exist
    if (m_pCurrentFarm == NULL)
    {
        return;
    }

    userNameLineEdit->setText(m_pCurrentFarm->getUsername());
    passwordLineEdit->setText(m_pCurrentFarm->getPassword());
    domainLineEdit->setText(m_pCurrentFarm->getCurrentDomain());
    saveCheckBox->setChecked(m_pCurrentFarm->isCredentialsSaved());

    QString tmpStr;
    bool tmpBool;
    userNameLineEdit->setEnabled(!m_appSettings->getValue(USERNAME, &tmpStr));
    passwordLineEdit->setEnabled(!m_appSettings->getValue(PASSWORD, &tmpStr));
    domainLineEdit->setEnabled(!m_appSettings->getValue(DOMAIN, &tmpStr));
    saveCheckBox->setEnabled(!m_appSettings->getValue(ALLOW_SAVE_CREDENTIALS, &tmpBool));
}

void CredentialsPage::saveFarmSettings()
{
    // Farm doesn`t exist
    if (m_pCurrentFarm == NULL)
    {
        return;
    }

    m_pCurrentFarm->setUsername(userNameLineEdit->text());
    m_pCurrentFarm->setPassword(passwordLineEdit->text());
    m_pCurrentFarm->setCurrentDomain(domainLineEdit->text());
    m_pCurrentFarm->setCredentialsSave(saveCheckBox->isChecked());
}
