// **************************************************************************************
// *
// * © 2012 Dell, Inc.
// * ALL RIGHTS RESERVED.
// *
// * Redistribution and use in source and binary forms, with or without modification, 
// * are permitted provided that the following conditions are met:
// * 
// *  *  Redistributions of source code must retain the above copyright notice, 
// *     this list of conditions and the following disclaimer.  
// * 
// *  *  Redistributions in binary form must reproduce the above copyright notice, 
// *     this list of conditions and the following disclaimer in the documentation 
// *     and/or other materials provided with the distribution.  
// * 
// *  *  Neither the name of Dell, Inc. nor the names of its contributors 
// *     may be used to endorse or promote products derived from this software 
// *     without specific prior written permission. 
// * 
// * 
// * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
// * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
// * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
// * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
// * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
// * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
// * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
// * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
// * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// * 
// **************************************************************************************

#include <stdexcept>
#include "titlePage.h"


TitlePage::TitlePage(Farm* farm, ManageConnectionDialog* parent)
    : ManageConnectionPage(farm, parent)
{
    setupUi(this);

    bool connected = connect(donePushButton, SIGNAL(clicked()), parent, SLOT(accept()));
    if (!connected)
    {
        throw std::runtime_error("QT's signal and slot connection error in TitlePage constructor");
    }

    loadFarmSettings();
}

void TitlePage::loadFarmSettings()
{
    // Farm doesn`t exist
    if (m_pCurrentFarm == NULL)
    {
        return;
    }

    farmNameLineEdit->setText(m_pCurrentFarm->getName());
    autoConnectCheckBox->setChecked(m_pCurrentFarm->autoConnected());
}

void TitlePage::saveFarmSettings()
{
    // Farm doesn`t exist
    if (m_pCurrentFarm == NULL)
    {
        return;
    }

    // Verifing new name
    QString newName = farmNameLineEdit->text();
    bool isFarmNameCorrect = m_pCurrentFarm->verifyName(newName);
    if (isFarmNameCorrect)
    {
        m_pCurrentFarm->setName(newName);
    }

    bool isFarmAutoConnected = autoConnectCheckBox->isChecked();
    if (isFarmAutoConnected)
    {
        m_pParentDialog->setAllFarmsNotAutoConnected();
    }
    m_pCurrentFarm->setAutoConnect(isFarmAutoConnected);
}

void TitlePage::setTitle(const QString& title)
{
    titleLabel->setText(title);
}
