// **************************************************************************************
// *
// * © 2012 Dell, Inc.
// * ALL RIGHTS RESERVED.
// *
// * Redistribution and use in source and binary forms, with or without modification, 
// * are permitted provided that the following conditions are met:
// * 
// *  *  Redistributions of source code must retain the above copyright notice, 
// *     this list of conditions and the following disclaimer.  
// * 
// *  *  Redistributions in binary form must reproduce the above copyright notice, 
// *     this list of conditions and the following disclaimer in the documentation 
// *     and/or other materials provided with the distribution.  
// * 
// *  *  Neither the name of Dell, Inc. nor the names of its contributors 
// *     may be used to endorse or promote products derived from this software 
// *     without specific prior written permission. 
// * 
// * 
// * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
// * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
// * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
// * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
// * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
// * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
// * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
// * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
// * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// * 
// **************************************************************************************

#include "aboutDialog.h"

#include <QDir>
#include <QMessageBox>

AboutDialog::AboutDialog(const QString & version, QWidget *pwgt/*=0*/) : QDialog(pwgt)
{
        QVBoxLayout *mainLayout = new QVBoxLayout;
        mainLayout->setSizeConstraint(QLayout::SetFixedSize);

	QHBoxLayout *captionLayout = new QHBoxLayout;

	QVBoxLayout *nameLayout = new QVBoxLayout;
	QLabel *lblName = new QLabel(tr("vWorkspace AppPortal"));
	QFont *nameFont = new QFont(lblName->font().family(), 16, QFont::Bold);
	lblName->setFont(*nameFont);
        // QLabel *lblWhat = new QLabel(tr("Enhanced Remote Desktop Services Client"));
        // lblWhat->setWordWrap(true);
	QLabel *lblVersion = new QLabel(tr("Version: ") + version);
	nameLayout->addWidget(lblName, 0, Qt::AlignTop);
        // nameLayout->addWidget(lblWhat, 0, Qt::AlignBottom);
	nameLayout->addWidget(lblVersion, 0, Qt::AlignBottom);
	nameLayout->insertSpacing(1, 25);
	nameLayout->setSpacing(5);

	QLabel *lblLogo = new QLabel;
    lblLogo->setPixmap(QPixmap(":/img/QST_logo_dvg_250.png"));
	lblLogo->setFrameShadow(QFrame::Sunken);
	lblLogo->setFrameShape(QFrame::Panel);

	captionLayout->addItem(nameLayout);
	captionLayout->addWidget(lblLogo, 0, Qt::AlignTop | Qt::AlignRight);
	captionLayout->setSpacing(20);

	mainLayout->addItem(captionLayout);

	QFrame *bottomLine = new QFrame;
	bottomLine->setFrameStyle(QFrame::HLine | QFrame::Sunken);
	bottomLine->setFixedHeight(2);
	mainLayout->addWidget(bottomLine);

        QLabel *lblWarning = new QLabel(tr("Warning: This program is protected by copyright laws and international\n"
                                           "treaties. Unauthorized reproductions or distribution of this program, or "
                                           "any\nportion of it, may result in severe civil and criminal penalties, "
                                           "and will be\nprosecuted to the maximum extent possible under the law."));
	lblWarning->setWordWrap(true);

	QPushButton *btnOk = new QPushButton(tr("&OK"));
	connect(btnOk, SIGNAL(clicked()), SLOT(reject()));

        QPushButton *btnFileInfo = new QPushButton(tr("File Information..."));
        connect(btnFileInfo, SIGNAL(clicked()), SLOT(showFileInfo()));

	QHBoxLayout *bottomLayout = new QHBoxLayout;
	bottomLayout->addWidget(lblWarning);

    QHBoxLayout *buttonsLayout = new QHBoxLayout;
    buttonsLayout->setAlignment(Qt::AlignRight);
    buttonsLayout->addWidget(btnFileInfo);
    buttonsLayout->setSpacing(5);
	buttonsLayout->addWidget(btnOk);

	mainLayout->addItem(bottomLayout);
	QFrame *bottomLine2 = new QFrame;
	bottomLine2->setFrameStyle(QFrame::HLine | QFrame::Sunken);
	bottomLine2->setFixedHeight(2);
	mainLayout->addWidget(bottomLine2);
	mainLayout->addItem(buttonsLayout);

	this->setLayout(mainLayout);

    this->setWindowTitle(tr("About vWorkspace AppPortal"));
}

QStringList AboutDialog::getFileList()
{
    QStringList list;

    QFile file(":/common/appportalfiles.txt");
    file.open(QIODevice::ReadOnly | QIODevice::Text);

    while (!file.atEnd())
    {
        QString line(file.readLine());
        line.remove("\n");
        list << line;
    }

    return list;
}

void AboutDialog::showFileInfo()
{
    QStringList headers;
    headers << tr("File Name") << tr("Location") << tr("Version");

    QDialog* fileListDlg = new QDialog(this);
    fileListDlg->setWindowTitle(tr("Quest vWorkspace"));
    fileListDlg->setMinimumWidth( 600 );
    fileListDlg->setMinimumHeight( 400 );

    QVBoxLayout* mainLayout = new QVBoxLayout();
    QHBoxLayout* frstLine = new QHBoxLayout();
    QHBoxLayout* scndLine = new QHBoxLayout();
    QHBoxLayout* thrdLine = new QHBoxLayout();
    thrdLine->setAlignment(Qt::AlignRight);

    mainLayout->addLayout(frstLine);
    mainLayout->addLayout(scndLine);
    mainLayout->addLayout(thrdLine);

    QTableWidget* tableWidget = new QTableWidget(0, 3, fileListDlg);
    tableWidget->setHorizontalHeaderLabels(headers);
    tableWidget->setColumnWidth(0, 100);
    tableWidget->setColumnWidth(1, 300);
    tableWidget->setColumnWidth(2, 120);

    QLabel* text = new QLabel(tr("vWorkspace File Information"));

    QPushButton* okButton = new QPushButton(tr("OK"));
    connect(okButton, SIGNAL(clicked()), fileListDlg, SLOT(accept()));

    frstLine->addWidget(text);
    scndLine->addWidget(tableWidget);
    thrdLine->addWidget(okButton);
    fileListDlg->setLayout(mainLayout);

    QStringList files = getFileList();

    tableWidget->setRowCount(files.count());

    for (int i = 0; i < files.count(); ++i)
    {
        QString file = files.at(i).split('/').last();
        QString path = files.at(i);
        path.remove(file);

        QTableWidgetItem* fileNameItem = new QTableWidgetItem(file);
        fileNameItem->setToolTip(file);
        tableWidget->setItem(i, 0, fileNameItem);

        QTableWidgetItem* pathItem = new QTableWidgetItem(path);
        pathItem->setToolTip(path);
        tableWidget->setItem(i, 1, pathItem);

        tableWidget->setItem(i, 2, new QTableWidgetItem(QFile::exists(files.at(i)) ? "" : tr("<File Not Found>")));
    }

    tableWidget->setSortingEnabled(true);
    tableWidget->sortByColumn(0, Qt::AscendingOrder);

    fileListDlg->exec();
}


