// **************************************************************************************
// *
// * © 2012 Dell, Inc.
// * ALL RIGHTS RESERVED.
// *
// * Redistribution and use in source and binary forms, with or without modification, 
// * are permitted provided that the following conditions are met:
// * 
// *  *  Redistributions of source code must retain the above copyright notice, 
// *     this list of conditions and the following disclaimer.  
// * 
// *  *  Redistributions in binary form must reproduce the above copyright notice, 
// *     this list of conditions and the following disclaimer in the documentation 
// *     and/or other materials provided with the distribution.  
// * 
// *  *  Neither the name of Dell, Inc. nor the names of its contributors 
// *     may be used to endorse or promote products derived from this software 
// *     without specific prior written permission. 
// * 
// * 
// * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
// * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
// * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
// * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
// * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
// * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
// * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
// * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
// * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// * 
// **************************************************************************************

#include "applicationSettings.h"

ApplicationSettings::ApplicationSettings(const QString fileName)
    : m_settings(fileName, QSettings::IniFormat)
{
}

ApplicationSettings::~ApplicationSettings()
{
}

bool ApplicationSettings::getQvariantValue(const QString& key, QVariant* value)
{
    if (m_settings.contains(key))
    {
        *value = m_settings.value(key);
        return true;
    }
    return false;
}

int ApplicationSettings::getQvariantValueList(const QString& key, QList<QVariant>* valueList)
{
    valueList->clear();
    // The usage of QSettings arrays in this function is atypical:
    //setArrayIndex() does not used, instead of it here manually passes index of array as a key
    int size = m_settings.beginReadArray(key);
    for (int i = 0; i < size; ++i)
    {
        QVariant value;
        if (getQvariantValue(QVariant(i+1).toString(), &value))
        {
            valueList->append(value);
        }
    }
    m_settings.endArray();
    return valueList->size();
}

void ApplicationSettings::beginGroup(const QString& key)
{
    m_settings.beginGroup(key);
}

void ApplicationSettings::endGroup()
{
    m_settings.endGroup();
}
