// **************************************************************************************
// *
// * © 2012 Dell, Inc.
// * ALL RIGHTS RESERVED.
// *
// * Redistribution and use in source and binary forms, with or without modification, 
// * are permitted provided that the following conditions are met:
// * 
// *  *  Redistributions of source code must retain the above copyright notice, 
// *     this list of conditions and the following disclaimer.  
// * 
// *  *  Redistributions in binary form must reproduce the above copyright notice, 
// *     this list of conditions and the following disclaimer in the documentation 
// *     and/or other materials provided with the distribution.  
// * 
// *  *  Neither the name of Dell, Inc. nor the names of its contributors 
// *     may be used to endorse or promote products derived from this software 
// *     without specific prior written permission. 
// * 
// * 
// * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
// * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
// * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
// * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
// * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
// * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
// * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
// * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
// * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// * 
// **************************************************************************************

#ifndef _FARM_H_
#define _FARM_H_

#include <QSettings>
#include <QTemporaryFile>
#include <QTextStream>
#include <QList>
#include <QMessageBox>
#include <QFile>
#include <QDomNode>
#include <QByteArray>
#include <QSharedPointer>

#include "location.h"
#include "FarmClient/applicationList.h"
#include "common/applicationSettings.h"

typedef QSharedPointer<ApplicationSettings> SharedApplicationSettings;
typedef QSharedPointer<Location> LocationSharedPointer;
typedef QSharedPointer<QFile> FileSharedPointer;


class Farm
{
public:
    Farm(QSettings* s, QString key = QString(), const QString& name = QString());
    virtual ~Farm();

    FileSharedPointer createPitFile();

    bool loadFarm();
    void saveFarm();

    QString getKey() const { return key; }

    // Location management
    LocationSharedPointer createNewLocation();
    int addLocation(LocationSharedPointer location);
    void removeLocation(int number);
    void removeAllLocations();

    void setCurrentLocation(int number) { m_currentLocation = m_locations.at(number); } // Not affected by override values mechanism
    LocationSharedPointer getCurrentLocation() { return m_currentLocation; }
    LocationSharedPointer getLocation(int number)
    {
        if(m_locations.size() == 1)
        {
            return m_locations.at(0);
        }
        return m_locations.at(number);
    }
    QStringList getLocationNamesList();

    void setLastLocation(const int number);
    int getLastLocation() { return iLast; }
    void setDefaultLocation(const int number);
    int getDefaultLocation() { return iDefault; }
    bool isPromptLocation() { return promptLocation; }
    void setPromptLocation(const bool prompt);

    QString getName() const;
    void setName(const QString& name); // Not affected by override values mechanism
    bool verifyName(const QString& name);

    QString  getUsername() const { return username; }
    void setUsername(const QString& name);
    QString  getPassword() const { return password; }
    void setPassword(const QString& pass);
    QStringList  getDomainList() const { return domainList; }
    void setDomainList(const QStringList& dmns);
    QString getCurrentDomain() const { return currentDomain; }
    void setCurrentDomain(const QString& dmn);
    bool isCredentialsSaved() const { return saveCredentials; }
    void setCredentialsSave(const bool save);
    bool isKerberosMode() const { return kerberosMode; }
    void setKerberosMode(const bool km);
    bool isKerberosEnabled() const { return kerberosEnabled; }
    void setKerberosEnable(const bool mode);
    bool isSsoEnabled() const { return ssoEnabled; }
    void setSsoEnable(const bool mode);
    int getWaitTime() const { return waitTime; }
    void setWaitTime(const int wait);
    int getNumberOfRetries() const { return numberOfRetries; }
    void setNumberOfRetries(const int retry);
    bool waitForDesktop() const { return waitForDesktopIsEnabled; }
    void setWaitForDesktop(const bool enabled);

    // User experience
    int  getExperienceLevel() const { return experienceLevel; }
    void setExperienceLevel( const int level );
    bool isWallpaperEnabled() const { return wallpaperEnabled; }
    void setWallpaperEnable(const bool enabled);
    bool isDraggingEnabled() const { return draggingEnabled; }
    void setDraggingEnable(const bool enabled);
    bool isAnimationEnabled() const { return animationEnabled; }
    void setAnimationEnable(const bool enabled);
    bool isThemesEnabled() const { return themesEnabled; }
    void setThemesEnable(const bool enabled);
    bool isBitmapCachingEnabled() const { return bitmapCachingEnabled; }
    void setBitmapCachingEnable(const bool enabled);

    bool isMmrEnabled() const { return mmrEnabled; }
    bool isFmmrEnabled() const { return fmmrEnabled; }
    bool isGaEnabled() const { return gaEnabled; }
    bool isDnaEnabled() const { return dnaEnabled; }
    void setMmrEnable(const bool enabled);
    void setFmmrEnable(const bool enabled);
    void setGaEnable(const bool enabled);
    void setDnaEnabled(const bool enabled);

    bool isReconnection() const { return reconnect; }
    void setReconnection(const bool b);

    // Desktop options
    int getDesktopWidth() const { return desktopWidth; }
    void setDesktopWidth(const int width);
    int getDesktopHeight() const { return desktopHeight; }
    void setDesktopHeight(const int height);
    int getColorDepth() const { return colorDepth; }
    void setColorDepth(const int depth);

    bool isConnectionBar() const { return connectionBar; }
    void setConnectionBar(const bool mode);
    bool isPinConnectionBar() const { return pinConnectionBar; }
    void setPinConnectionBar(const bool mode);
    bool isSpanMonitors() const { return spanMonitors; }
    void setSpanMonitors(const bool mode);
    bool isSmartSizing() const { return smartSizing; }
    void setSmartSizing(const bool mode);
    bool isSeamlessMode() const { return seamlessMode; }
    void setSeamlessMode(const bool smode);

    // Redirection options
    bool isRedirectClipboard() const { return redirectClipboard; }
    void setRedirectClipboard(const bool rc);
    bool isRedirectDrives() const { return redirectDrives; }
    void setRedirectDrives(const bool rd);
    bool isRedirectPrinters() const { return redirectPrinters; }
    void setRedirectPrinters(const bool rp);
    bool isRedirectUPrinters() const { return redirectUPrinters; }
    void setRedirectUPrinters(const bool rp);
    bool isRedirectComPorts() const { return redirectComPorts; }
    void setRedirectComPorts(const bool rc);
    bool isRedirectHandhelds() const { return redirectHandhelds; }
    void setRedirectHandhelds(const bool rh);
    bool isRedirectMicrophone() const { return redirectMicrophone; }
    void setRedirectMicrophone(const bool rm);
    bool isRedirectSmartCards() const { return redirectSmartCards; }
    void setRedirectSmartCards(const bool rs);

    // Sound mode
    QString getSoundMode() const { return soundMode; }
    void setSoundMode(const QString& mode);

    // Downstream audio
    bool isDownstreamAudioEnabled() const { return downstreamAudioEnabled; }
    void setDownstreamAudioEnabled(const bool daEnabled);
    QString getDAMode() const { return daMode; }
    void setDAMode(const QString& mode);
    QString getDAQuality() const { return daQuality; }
    void setDAQuality(const QString& quality);
    int getDABuffering() const { return daBuffering; }
    void setDABuffering(const int buf);
    int getDACustomQuality() const { return daCustomQuality; }
    void setDACustomQuality(const int cq);

    // Upstream audio
    bool isUpstreamAudioEnabled() const { return upstreamAudioEnabled; }
    void setUpstreamAudioEnabled(const bool uaEnabled);
    QString getUAMode() const { return uaMode; }
    void setUAMode(const QString& mode);
    int getUAQuality() const { return uaQuality; }
    void setUAQuality(const int quality);
    int getUAComplexity() const { return uaComplexity; }
    void setUAComplexity(const int complexity);
    bool getUAEnhancedCompression() const { return uaEnhancedCompression; }
    void setUAEnhancedCompression(const bool ec);

    // Settings hide
    bool isHideSettings() const { return hideSettings; }
    void setHideSettings(const bool hide);

    // Password management
    QString getPasswordManagementServer() const { return pmServer; }
    void setPasswordManagementServer(const QString& server);
    int getPasswordManagementPort() const { return pmPort; }
    void setPasswordManagementPort(const int port);
    bool isPasswordManagementAllowed() const { return allowPasswordManagement; }
    void setPasswordManagementAllow(const bool allow);

    // Load farm from server parameters
    QString getSourceProtocol() const { return sourceProtocol; }
    void setSourceProtocol(const QString& protocol);
    QString getSourceServer() const { return sourceServer; }
    void setSourceServer(const QString& server);
    QString getSourceFile() const { return sourceFile; }
    void setSourceFile(const QString& file);
    QString getSourceProxy() const { return sourceProxy; }
    void setSourceProxy(const QString& proxy);

    // Applications management
    ApplicationList applications() const;
    void setApplications(const ApplicationList apps); // Not affected by override values mechanism
    ApplicationList autolaunch() const;
    void setAutolaunch(const ApplicationList apps); // Not affected by override values mechanism
    //QStringList getAutoLaunchList() const { return autoLaunch; }
    //void setAutoLaunchList(QStringList apps) { autoLaunch = apps; }

    void setNeedsReconnecting(const bool need);
    bool needsReconnecting() const { return needReconnect; }

    void setAutoConnect(const bool val);
    bool autoConnected() const { return autoConnect; }

    void setShowSteps(const bool val);
    bool showSteps() const { return showHelpSteps; }
    bool getShowClickHere() const { return showClickHere; }
    void setShowClickHere( const bool val );

private:
    QString name;
    QString username;
    QString password;
    QStringList domainList;
    QString currentDomain;
    bool kerberosMode;
    bool kerberosEnabled;
    bool ssoEnabled;
    bool saveCredentials;

    int  experienceLevel;
    bool wallpaperEnabled;
    bool draggingEnabled;
    bool animationEnabled;
    bool themesEnabled;
    bool bitmapCachingEnabled;
    bool mmrEnabled;
    bool fmmrEnabled;
    bool dnaEnabled;
    bool gaEnabled;
    bool reconnect;

    int desktopWidth;
    int desktopHeight;
    int colorDepth;
    bool connectionBar;
    bool pinConnectionBar;
    bool spanMonitors;
    bool smartSizing;
    bool seamlessMode;

    QString soundMode;

    bool downstreamAudioEnabled;
    QString daMode;
    QString daQuality;
    int daBuffering;
    int daCustomQuality;

    bool upstreamAudioEnabled;
    QString uaMode;
    int uaQuality;
    int uaComplexity;
    bool uaEnhancedCompression;

    bool redirectClipboard;
    bool redirectDrives;
    bool redirectPrinters;
    bool redirectUPrinters;
    bool redirectComPorts;
    bool redirectHandhelds;
    bool redirectMicrophone;
    bool redirectSmartCards;

    bool hideSettings;
    QString pmServer;
    int pmPort;
    bool allowPasswordManagement;

    QString sourceServer;
    QString sourceProtocol;
    QString sourceFile;
    QString sourceProxy;

    int iLast;
    int iDefault;
    bool promptLocation;

    bool needReconnect;
    bool autoConnect;
    bool showHelpSteps;
    bool showClickHere;

    int waitTime;
    int numberOfRetries;
    bool waitForDesktopIsEnabled;

    QString key;

    QSettings *settings;
    QList<LocationSharedPointer> m_locations;
    LocationSharedPointer m_currentLocation;

    ApplicationList p_applications;
    ApplicationList p_autolaunch;
    QStringList autoLaunch;
    SharedApplicationSettings m_appSettings;

    void setAllDefaultValues();
    void setAllOverrideValues();

    template <typename Type>
    void setDefaultValue(const QString& key, Type* variable, const Type value);

    template <typename Type>
    void setValue(const QString& key, Type* variable, const Type value);

    template <typename Type>
    void setValueList(const QString& key, QList<Type>* list, const QList<Type> valueList);
};

#endif
