// **************************************************************************************
// *
// * © 2012 Dell, Inc.
// * ALL RIGHTS RESERVED.
// *
// * Redistribution and use in source and binary forms, with or without modification, 
// * are permitted provided that the following conditions are met:
// * 
// *  *  Redistributions of source code must retain the above copyright notice, 
// *     this list of conditions and the following disclaimer.  
// * 
// *  *  Redistributions in binary form must reproduce the above copyright notice, 
// *     this list of conditions and the following disclaimer in the documentation 
// *     and/or other materials provided with the distribution.  
// * 
// *  *  Neither the name of Dell, Inc. nor the names of its contributors 
// *     may be used to endorse or promote products derived from this software 
// *     without specific prior written permission. 
// * 
// * 
// * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
// * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
// * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
// * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
// * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
// * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
// * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
// * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
// * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// * 
// **************************************************************************************

#ifndef _FARMSETTINGS_H_
#define _FARMSETTINGS_H_

#include <QSettings>
#include <QStringList>
#include <QMessageBox>
#include <QTemporaryFile>
#include <QTextStream>
#include <QList>

#include "farm.h"


class FarmSettings : public QSettings
{
    Q_OBJECT

public:
    FarmSettings(const QString & organization = QString(),
    const QString & application = QString(), QObject * parent = 0);
    void setCryptedValue(const QString & key, const QVariant & value);
    QVariant cryptedValue(const QString & key,
    const QVariant & defaultValue = QVariant()) const;

    void removeFarm(int number);
    void removeFarm(Farm *farm);
    void addFarm(Farm *farm);
    Farm * createNewFarm();

    void loadFarms();
    void saveFarms();

    void setDefaultFarm(int number) { iDefault = number; }
    int getDefaultFarm() { return iDefault; }

    void setCurrentFarm(int number) { currentFarm = farms.at(number); }
    Farm * getCurrentFarm() { return currentFarm; }
    Farm * getFarm(const QString & name);
    Farm * getFarm(int number) { return farms.at(number); }
    QStringList getFarmNamesList();

    bool isPassThroughEnabled() { return passThroughEnabled; }
    void setPassThroughEnable(bool mode) { passThroughEnabled = mode; }

private:
    QList<Farm *> farms;
    Farm *currentFarm;

    int iDefault;
    bool passThroughEnabled;
};

#endif // _FARMSETTINGS_H_


// List of register keys and strings in pit-file
/*    Username                    username:s:            *
 *     Password                   password:s:            *
 *     Domain                     domain:s:            *
 *     KerberosMode
 *     CredentialsType
 *     AllowSaveCredentials
 *
 *    EnableWallpaper             enableBackground:i:        *
 *    EnableFullWindowDrag        showWhileDragging:i:    *
 *    EnableAnimation             enableAnimation:i:        *
 *    EnableThemes                enableThemes:i:            *
 *    EnableBitmapCaching         enableBitmapCaching:i:    *
 *    AutoReconnect
 *
 *    DesktopHeight               windowSize:s:            *
 *    DesktopWidth                windowSize:s:            *
 *    ColorDepth                  colorQuality:s:            *
 *    DisplayConnectionBar
 *    PinConnectionBar
 *    SpanMonitors
 *    EnableSmartSizing
 *    SeamlessMode                seamlessMode:i:            *
 *
 *    PasswordManagementPort
 *    PasswordManagementServer
 *
 *    KeyboardHook
 *    RedirectAudio               sound:s:[local|off|remote]    *
 *    RedirectDrives              drives:s:"
 *    RedirectPrinters            printers:s:
 *    RedirectHandhelds
 *    RedirectClipboard           clipboard:i:                *
 *    RedirectComPorts            comports:s:
 *    RedirectSmartCards          smartcards:s:
 *    RedirectUniversalPrinters
 *    RedirectMicrophone
 *    EOP Xtream                  dnaproxy:i:
 *
 *    NAT                         llmenablenat:i:    *
 *    SSLGateway                  sslgateway:s:    *
 *    RDPonSSL                    enablessl        *                            *
 *    UseProxy
 *    ProxyBypassList
 *    ProxyServer
 *    Name
 *    Servers                     llmservers:s:    *
 *    TCPPort                     llmtcpport:s:    *
 *    Protocol                    llmprotocol:i:    *
 *
 *    DefaultLocation                                *
 *
 *
 *
 *
 *
 * */

