// **************************************************************************************
// *
// * © 2012 Dell, Inc.
// * ALL RIGHTS RESERVED.
// *
// * Redistribution and use in source and binary forms, with or without modification, 
// * are permitted provided that the following conditions are met:
// * 
// *  *  Redistributions of source code must retain the above copyright notice, 
// *     this list of conditions and the following disclaimer.  
// * 
// *  *  Redistributions in binary form must reproduce the above copyright notice, 
// *     this list of conditions and the following disclaimer in the documentation 
// *     and/or other materials provided with the distribution.  
// * 
// *  *  Neither the name of Dell, Inc. nor the names of its contributors 
// *     may be used to endorse or promote products derived from this software 
// *     without specific prior written permission. 
// * 
// * 
// * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
// * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
// * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
// * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
// * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
// * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
// * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
// * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
// * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// * 
// **************************************************************************************

#include "location.h"
#include "common/common.h"
#include "farmSettingsNames.h"

Location::Location()
{
    m_appSettings = SharedApplicationSettings(new ApplicationSettings(APPLICATION_CONFIG_FILE_NAME));

    // Default paramters
    m_appSettings->beginGroup(FARM_DEFAULT_VALUES);

    setDefaultValue(LOCATION_RDP_ON_SSL, &ssl, false);
    setDefaultValue(LOCATION_NAT, &nat, false);
    setDefaultValue(LOCATION_TCP_PORT, &port, QString("8080"));
    setDefaultValue(LOCATION_PROTOCOL, &protocol, QString("http"));
    setDefaultValue(LOCATION_SSL_GATEWAY, &sslGateway, QString());
    setDefaultValue(LOCATION_PROXY_SERVER, &proxy, QString());
    setDefaultValue(LOCATION_PROXY_BYPASS_LIST, &proxyBypassList, QString());

    m_appSettings->endGroup();

    // Override parameters if exist
    setPort(port);
    setProtocol(protocol);
    setServers(servers);
    setNat(nat);
    setSsl(ssl);
    setSslGateway(sslGateway);
    setProxyServer(proxy);
    setProxyBypassList(proxyBypassList);
}

Location::~Location()
{

}

void Location::setPort(const QString& p)
{
    setValue(LOCATION_TCP_PORT, &port, p);
}

void Location::setProtocol(const QString& p)
{
    setValue(LOCATION_PROTOCOL, &protocol, p);
}

void Location::setServers(QStringList srvs)
{
    setValueList(LOCATION_SERVERS, &servers, srvs);
}

void Location::setNat(bool isEnabled)
{
    setValue(LOCATION_NAT, &nat, isEnabled);
}

void Location::setSsl(bool isEnabled)
{
    setValue(LOCATION_RDP_ON_SSL, &ssl, isEnabled);
}

void Location::setSslGateway(const QString& gateway)
{
    setValue(LOCATION_SSL_GATEWAY, &sslGateway, gateway);
}

void Location::setProxyServer(const QString& pxy)
{
    setValue(LOCATION_PROXY_SERVER, &proxy, pxy);
}

void Location::setProxyBypassList(const QString& list)
{
    setValue(LOCATION_PROXY_BYPASS_LIST, &proxyBypassList, list);
}

template <typename Type>
void Location::setDefaultValue(const QString& key, Type* variable, const Type value)
{
    if (!m_appSettings->getValue(key, variable))
    {
        *variable = value;
    }
}

template <typename Type>
void Location::setValue(const QString& key, Type* variable, const Type value)
{
    m_appSettings->beginGroup(FARM_OVERRIDE_VALUES);
    if (!m_appSettings->getValue(key, variable))
    {
        *variable = value;
    }
    m_appSettings->endGroup();
}

template <typename Type>
void Location::setValueList(const QString& key, QList<Type>* list, const QList<Type> valueList)
{
    m_appSettings->beginGroup(FARM_OVERRIDE_VALUES);
    if (!m_appSettings->getValueList(key, list))
    {
        *list = valueList;
    }
    m_appSettings->endGroup();
}
