// **************************************************************************************
// *
// * © 2012 Dell, Inc.
// * ALL RIGHTS RESERVED.
// *
// * Redistribution and use in source and binary forms, with or without modification, 
// * are permitted provided that the following conditions are met:
// * 
// *  *  Redistributions of source code must retain the above copyright notice, 
// *     this list of conditions and the following disclaimer.  
// * 
// *  *  Redistributions in binary form must reproduce the above copyright notice, 
// *     this list of conditions and the following disclaimer in the documentation 
// *     and/or other materials provided with the distribution.  
// * 
// *  *  Neither the name of Dell, Inc. nor the names of its contributors 
// *     may be used to endorse or promote products derived from this software 
// *     without specific prior written permission. 
// * 
// * 
// * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
// * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
// * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
// * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
// * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
// * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
// * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
// * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
// * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// * 
// **************************************************************************************

#ifndef MAINWINDOW_H
#define MAINWINDOW_H

#include <QMainWindow>
#include <QListWidget>
#include <QProgressBar>
#include <QAction>
#include <QMenu>
#include <QMenuBar>
#include <QApplication>
#include <QSettings>
#include <QTemporaryFile>
#include <QToolButton>
#include <QMutex>
#include <QtGui>
#include <QStandardItemModel>
#include <QNetworkReply>

#include "treeMenuWidget.h"
#include "verticalMenu.h"
#include "aboutDialog.h"
#include "FarmClient/farmConnection.h"
#include "ProgressDialog/progressDialog.h"
#include "appPortalConfigCollector.h"
#include "appPortalConfigParser.h"
#include "applicationListModel.h"
#include "updateserver.h"
#include "SimpleAddFarm/simpleaddfarmdialog.h"
#include "FarmMenuDialog/farmmenudialog.h"
#include "maskfactory.h"
#include "minimalviewdialog.h"
#include "CredentialsDialog/credentialsDialog.h"
#include "AppSettingsDialog/appsettingsdialog.h"
#include "MainHelpDialog/mainhelpdialog.h"
#include "MainAboutDialog/mainaboutdialog.h"
#include "BasicMessageDialog/basicmessagedialog.h"
#include "mouseoverpushbutton.h"
#include "TransparentDialog/transparentdialog.h"

class QCheckBox;

namespace Ui {
    class qVwWindow;
}

class MainWindow : public QMainWindow {
    Q_OBJECT


public slots:
    // Slot for receiving replies on farms settings requests
    void networkReply(QNetworkReply::NetworkError, QString, QString );
    void minimalViewRemoveSessionClicked( QString );
    void runFarm(const QString & );
    void handleFarmMenuMouseEnter( QString );
    void handleFarmMenuMouseLeave( QString );
    void showStartupTooltips();
    void handleMinimalViewActivateSession( QString );

private slots:
    void on_cancelClickHereButton_clicked();
    void on_helpButton_clicked();
    void on_settingsButton_clicked();
    void on_changePasswordButton_clicked();
    void on_changeUserButton_clicked();
    void on_farmAccessPushbutton_clicked();
    void on_mainCloseButton_clicked();
    void on_mainShrinkButton_clicked();
    void on_refreshAppsButton_clicked();
    void on_applicationListView_doubleClicked(QModelIndex );
    void refreshApplicationSet();
    void handleAddFarm( bool );
    void handleFarmMenuClick( bool );
    void handleFarmMenuSettingsClick( bool );
    void handleLocationClick( bool );
    void handleLocationDefaultClick( bool );
    void handleMinimalRefreshRequest();
    void handleMinimalViewApplicationStart( QModelIndex );

    // Connection
    void connectionFinished(FarmConnection::FinishCode );
    void connectionFinishedAlt(FarmConnection::FinishCode , FarmConnection *);
    void connectionError();
    void runApplication(const QString &);
    void showStatusMessage(const QString &);

    void updateThumbnail( QPixmap, QString, struct pixel_update_header * );
    void handleSessionSelected( QModelIndex );

public:

    static MainWindow* instance()
    {
      static QMutex mutex;
      if (!m_instance)
      {
        mutex.lock();

        if (!m_instance)
          m_instance = new MainWindow();

        mutex.unlock();
      }

      return m_instance;
    }

    // Setting the flag that shows if we want to remove farms settings from GUI on exit
    void setRemoveFarms(bool isRemoveFarm);

    MainWindow(QWidget *parent = 0);
    ~MainWindow();

    void updateFarmsMenu();

    // Settings setters
    void setUsbAutoshare( bool );
    void setLogging( bool );
    void setShowStartupTooltips( bool );
    void setShowHelpSteps( bool );

    // Settings getters
    bool getUsbAutoshare(  );
    bool getLogging(  );
    bool getShowStartupTooltips( );
    bool getShowHelpSteps( );


protected:
    void changeEvent(QEvent *e);
    void mousePressEvent(QMouseEvent* event);
    void mouseMoveEvent(QMouseEvent* event);

private:

    void readSettings();
    void writeSettings();
    void loadFarmSettingsIfNeeded();
    void readFarmSettingsFromPredefinedLocationMasks();
    void makeFarmMenuButton( QString, QMenu *, int, int );
    void makeLocationMenuButton( QString, QString, QMenu *, int, int );
    void handleLocationSelectionButton( QString, bool );
    void readFarmSettingsFromDefaultLocations();
    void updateLocationLabel( QString );

    AppPortalConfigCollector *m_pConfigCollector;
    AppPortalConfigParser *m_pConfigParser;

    ApplicationListModel appListModel;
    QStandardItemModel sessionListModel;
    QMenu * farmMenu;
    QMenu * locationMenu;

    int m_requestsLeftCount;
    bool m_isRemoveFarms;
    bool m_isFarmLoaded;
    bool m_usbAutoshare;
    QPoint m_dragPosition;

    static MainWindow * m_instance;
    Ui::qVwWindow *ui;

    MinimalViewDialog * minimalView;
    ProgressDialog *progressDialog;

    FarmConnection *currentConnection;
    MaskFactory maskFactory;
    FarmSettings settings;
    UpdateServer m_updateServer;

};

#endif // MAINWINDOW_H
