// **************************************************************************************
// *
// * © 2012 Dell, Inc.
// * ALL RIGHTS RESERVED.
// *
// * Redistribution and use in source and binary forms, with or without modification, 
// * are permitted provided that the following conditions are met:
// * 
// *  *  Redistributions of source code must retain the above copyright notice, 
// *     this list of conditions and the following disclaimer.  
// * 
// *  *  Redistributions in binary form must reproduce the above copyright notice, 
// *     this list of conditions and the following disclaimer in the documentation 
// *     and/or other materials provided with the distribution.  
// * 
// *  *  Neither the name of Dell, Inc. nor the names of its contributors 
// *     may be used to endorse or promote products derived from this software 
// *     without specific prior written permission. 
// * 
// * 
// * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
// * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
// * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
// * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
// * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
// * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
// * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
// * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
// * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// * 
// **************************************************************************************

#include "maskfactory.h"

MaskFactory::MaskFactory(QObject *parent) :
    QObject(parent)
{
    // This should be a singleton

}


QPixmap * MaskFactory::createMask( int width, int height, int corner_radius )
{

    QPixmap * mask;
    // Create the pixmap of the correct size
    mask = new QPixmap( width + 10, height + 10 );

    // Fill with transparent
    mask->fill( Qt::white );

    // Set up the painter
    QPainter painter( mask );
    painter.setBrush( Qt::SolidPattern );

    painter.drawRoundedRect( 0,0, width -1  , height -1   , corner_radius, corner_radius, Qt::AbsoluteSize );

    // mask->save( "/tmp/mask.png", 0, 85 );
    return mask;
}

QPixmap * MaskFactory::createLeftPointerMask( int width, int height, int corner_radius )
{

    QPixmap * mask;
    // Create the pixmap of the correct size
    mask = new QPixmap( width + 30, height + 10 );

    // Fill with transparent
    mask->fill( Qt::white );

    // Set up the painter
    QPainter painter( mask );
    painter.setBrush( Qt::SolidPattern );

    painter.drawRoundedRect( 20,20, width -41  , height -40   , corner_radius, corner_radius, Qt::AbsoluteSize );
    static const QPointF points[3] = {
        QPointF(00.0, 40.0),
        QPointF(20.0, 50.0),
        QPointF(20.0, 30.0)
    };
    painter.drawPolygon(points, 3, Qt::WindingFill );

    // mask->save( "/tmp/mask.png", 0, 85 );
    return mask;
}

QPixmap * MaskFactory::createTopPointerMask( int width, int height, int corner_radius )
{

    QPixmap * mask;
    // Create the pixmap of the correct size
    mask = new QPixmap( width + 30, height + 10 );

    // Fill with transparent
    mask->fill( Qt::white );

    // Set up the painter
    QPainter painter( mask );
    painter.setBrush( Qt::SolidPattern );

    painter.drawRoundedRect( 20, 20, width -41  , height -40   , corner_radius, corner_radius, Qt::AbsoluteSize );
    static const QPointF points[3] = {
        QPointF(40.0, 00.0),
        QPointF(50.0, 20.0),
        QPointF(30.0, 20.0)
    };
    painter.drawPolygon(points, 3, Qt::WindingFill );

    mask->save( "/tmp/mask.png", 0, 85 );
    return mask;
}
