// **************************************************************************************
// *
// * © 2012 Dell, Inc.
// * ALL RIGHTS RESERVED.
// *
// * Redistribution and use in source and binary forms, with or without modification, 
// * are permitted provided that the following conditions are met:
// * 
// *  *  Redistributions of source code must retain the above copyright notice, 
// *     this list of conditions and the following disclaimer.  
// * 
// *  *  Redistributions in binary form must reproduce the above copyright notice, 
// *     this list of conditions and the following disclaimer in the documentation 
// *     and/or other materials provided with the distribution.  
// * 
// *  *  Neither the name of Dell, Inc. nor the names of its contributors 
// *     may be used to endorse or promote products derived from this software 
// *     without specific prior written permission. 
// * 
// * 
// * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
// * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
// * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
// * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
// * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
// * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
// * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
// * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
// * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// * 
// **************************************************************************************

#include "verticalMenu.h"

VerticalMenu::VerticalMenu(const QString & name, QWidget *pwgt/*=0*/)
	: QWidget(pwgt)
{
	menuCaption = new QLabel;
	menuCaption->setText("<b>" + name + "</b>");

	vmLayout = new QVBoxLayout;
	QFrame *line = new QFrame;
	line->setFrameStyle(QFrame::HLine | QFrame::Sunken);
	line->setFixedHeight(2);

	vmLayout->addWidget(menuCaption);
        vmLayout->addWidget(line);
        vmLayout->setAlignment(Qt::AlignTop | Qt::AlignLeft);
        vmLayout->setContentsMargins(0,0,0,0);
        vmLayout->setSpacing(0);
	this->setLayout(vmLayout);
}

void VerticalMenu::addMenuAction(QAction * action)
{
	VerticalMenuItem *vmItem = new VerticalMenuItem(action);
	vmLayout->addWidget(vmItem);
}

void VerticalMenu::addMenuLayout(QLayout *layout)
{
    vmLayout->addLayout(layout);
}

void VerticalMenu::addWidget(QWidget *widget)
{
    vmLayout->addWidget(widget);
}

void VerticalMenu::addSpacer()
{
    const int spacerSize = 5;
    QSpacerItem* spacer = new QSpacerItem(spacerSize, spacerSize, QSizePolicy::Expanding, QSizePolicy::Expanding);
    vmLayout->addSpacerItem(spacer);
}

void VerticalMenu::setCaption(const QString & name)
{
	menuCaption->setText(name);
}

void VerticalMenu::clear()
{
	QLayoutItem *child;
        while ((child = vmLayout->takeAt(2)) != 0)
             delete child;


}
