// **************************************************************************************
// *
// * © 2012 Dell, Inc.
// * ALL RIGHTS RESERVED.
// *
// * Redistribution and use in source and binary forms, with or without modification, 
// * are permitted provided that the following conditions are met:
// * 
// *  *  Redistributions of source code must retain the above copyright notice, 
// *     this list of conditions and the following disclaimer.  
// * 
// *  *  Redistributions in binary form must reproduce the above copyright notice, 
// *     this list of conditions and the following disclaimer in the documentation 
// *     and/or other materials provided with the distribution.  
// * 
// *  *  Neither the name of Dell, Inc. nor the names of its contributors 
// *     may be used to endorse or promote products derived from this software 
// *     without specific prior written permission. 
// * 
// * 
// * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS ON AN "AS IS" 
// * AND UNSUPPORTED BASIS, WITHOUT ANY EXPRESS OR IMPLIED WARRANTIES OF ANY KIND, 
// * INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY, NON-
// * INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE. IN NO EVENT SHALL THE COPYRIGHT 
// * HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, 
// * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT 
// * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
// * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, 
// * STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT 
// * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
// * 
// **************************************************************************************

#include <syslog.h>
#include "crypto.h"
#include "rc4.h"

// Get these references from libqcommondata.a library
extern unsigned char  * get_scatter256();
extern unsigned short * get_scatter1024();
extern unsigned char  * get_SPN();

extern unsigned int get_SPN_size();
extern unsigned int get_scatter256_size();
extern unsigned int get_scatter1024_size();


#ifdef __cplusplus
extern "C"
#endif
char*
create_tgt_response(char* filename)
{
    FILE* f = fopen(filename, "rb");
    if (f == NULL)
        return 0;

    fseek(f, 0, SEEK_END);
    int l = ftell(f);
    fseek(f, 0, SEEK_SET);

    unsigned char* buf = (unsigned char*)malloc(16 + l);
    *(unsigned int*)buf = 1;
    *(unsigned int*)(buf + 4) = 0;
    *(unsigned int*)(buf + 8) = get_SPN_size();
    *(unsigned int*)(buf + 12) = l;

    memcpy(buf + 16, get_SPN(), get_SPN_size());
    fread(buf + 16 + get_SPN_size(), 1, l, f);

    fclose(f);

    l += 16 + get_SPN_size();
    char* res = (char*)malloc(l * 2 + 1);
    BinToHex(buf, l, res, l * 2 + 1);
    free(buf);
    return res;
}

static unsigned int Checksum(unsigned char* pBuffer, int cbBuffer)
{
    unsigned int dwChecksum = 0;

    for (int i = 0; i < cbBuffer; i++)
    {
        dwChecksum += pBuffer[i];
    }

    return dwChecksum % 256;
}

void Encode256(unsigned char* rgbBuffer, char* szString)
{
    int i;

    // Randomize the entire buffer.
    srand(time(0));
    for (i = 0; i < 256; i++)
    {
        rgbBuffer[i] = rand() % 256;
    }

    // Scatter the password in the buffer.
    for (i = 0; i < 256; i++)
    {
        rgbBuffer[get_scatter256()[i]] = (unsigned char)szString[i];
        if (szString[i] == '\0') break;
    }

    // Force the checksum for the entire buffer equal to zero.
    rgbBuffer[255] = (unsigned char)(256 - Checksum(rgbBuffer, 255));
}

void Decode256(unsigned char *rgbBuffer, char *szString)
{
    for (int i = 0; i < 256; i++)
    {
    	szString[i] = (unsigned char) rgbBuffer[get_scatter256()[i]];
    	if (szString[i] == '\0')
    	{
    		break;
    	}
    }
}

void Encode1024(unsigned char* rgbBuffer, char* szString)
{
    int i;

    // Randomize the entire buffer.
    srand(time(0));
    for (i = 0; i < 1024; i++)
    {
        rgbBuffer[i] = rand() % 256;
    }

    // Scatter the password in the buffer.
    for (i = 0; i < 1024; i++)
    {
        rgbBuffer[get_scatter1024()[i]] = (unsigned char)szString[i];
        if (szString[i] == '\0') break;
    }

    // Force the checksum for the entire buffer equal to zero.
    rgbBuffer[1023] = (unsigned char)(256 - Checksum(rgbBuffer, 1023));
}

void Decode1024(unsigned char *rgbBuffer, char *szString)
{
    for (int i = 0; i < 1024; i++)
    {
    	szString[i] = (unsigned char) rgbBuffer[get_scatter1024()[i]];
    	if (szString[i] == '\0')
    	{
    		break;
    	}
    }
}

int BinToHex(unsigned char* pBin, int cbBin, char* pHex, int cbHex)
{
    // Initialize the output buffer.
    memset(pHex, 0, cbHex);

    // Check the size of the output buffer.
    if (cbHex < ((cbBin * 2) + 1))
        return 0;

    // Convert each byte to its hexadecimal equivalent.
    for (int i = 0; i < cbBin; i++)
    {
        pHex += sprintf(pHex, "%02X", pBin[i]);
    }
    *pHex = '\0';

    return cbBin;
}

int HexToBin(unsigned char* pBin, int cbBin, char* pHex, int cbHex)
{
    int i;

    // Initialize the output buffer.
    memset(pBin, 0, cbBin);

    // Check the size of the output buffer.
    if (cbBin < ((cbHex / 2) + 1))
    {
        return 0;
    }

	for (i = 0; i < cbHex/2; ++i)
	{
		int tens = pHex[i*2] > 64 ? pHex[i*2] - 55 : pHex[i*2] - 48;
		int ones = pHex[i*2+1] > 64 ? pHex[i*2+1] - 55 : pHex[i*2+1] - 48;
		pBin[i] = tens*16 + ones;
	}
	pBin[i] = '\0';
    return cbHex;
}

void Encrypt(unsigned char* pBuffer, int cbBuffer)
{
    unsigned char key[] = { 0x84, 0x31, 0xA7, 0xF3, 0x4E, 0x2B, 0x90, 0x59 };
    RC4StreamCipher cipher(key, sizeof(key));
    cipher.Encrypt(pBuffer, pBuffer, cbBuffer);
}

void Decrypt(unsigned char* pBuffer, int cbBuffer)
{
    unsigned char key[] = { 0x84, 0x31, 0xA7, 0xF3, 0x4E, 0x2B, 0x90, 0x59 };
    RC4StreamCipher cipher(key, sizeof(key));
    cipher.Decrypt(pBuffer, pBuffer, cbBuffer);
}


////////////////////////////////////////////////////////////////////////
//
// Decrypt1024
//
// Decrypts a 1024 byte string.
//
void Decrypt1024( unsigned char * encrypted, unsigned char * decrypted, int cbDecrypted)
{
	unsigned char rgbBuffer[2048];
	int cbBuffer;

	memset(decrypted, 0x0, cbDecrypted);

	// Convert from hex to binary.
	memset(rgbBuffer, 0x0, sizeof(rgbBuffer));
	// cbBuffer = HexToBin( rgbBuffer, sizeof(rgbBuffer), (char *)encrypted, sizeof( encrypted ));
    cbBuffer = HexToBin( rgbBuffer, sizeof(rgbBuffer), (char *)encrypted, strlen( (char *)encrypted ));
	if (cbBuffer != sizeof(rgbBuffer))
	{
#ifdef _DEBUG
	    printf ( "Decrypt1024() failed in HexToBin call. %i != %i.\n", cbBuffer, sizeof( rgbBuffer));
	    syslog (LOG_NOTICE, "Decrypt1024() failed in HexToBin call. %i != %i.\n", cbBuffer, sizeof( rgbBuffer));
#endif
	    return;
	}

	// Decrypt the buffer.
	Decrypt(( unsigned char *)rgbBuffer, sizeof(rgbBuffer));
    
#ifdef _DEBUG
    syslog( LOG_NOTICE, "Decrypted1024 buffer: '%s'", rgbBuffer );
#endif

	// Decode the buffer.
	Decode1024(( unsigned char *)rgbBuffer, (char * ) decrypted );

#ifdef _DEBUG
    syslog( LOG_NOTICE, "Decoded1024 buffer: '%s'", decrypted );
#endif

	return;
}


////////////////////////////////////////////////////////////////////////
//
// Decrypt256
//
// Decrypts a 256 byte string.
//
void Decrypt256( unsigned char * encrypted, unsigned char * decrypted, int cbDecrypted)
{
	unsigned char rgbBuffer[257];
	int cbBuffer;

	memset(decrypted, 0x0, cbDecrypted);

	// Convert from hex to binary.
	memset(rgbBuffer, 0x0, sizeof(rgbBuffer));
    cbBuffer = HexToBin( rgbBuffer, sizeof(rgbBuffer), (char *)encrypted, strlen( (char *)encrypted ));

#if 0
	if (cbBuffer != sizeof(rgbBuffer))
	{
	    syslog( LOG_NOTICE,  "Decrypt256() failed in HexToBin call. %i != %i.\n", cbBuffer, sizeof( rgbBuffer));
	    return;
	}

#endif

	// Decrypt the buffer.
	Decrypt(( unsigned char *)rgbBuffer, sizeof(rgbBuffer));

	// Decode the buffer.
	Decode256(( unsigned char *)rgbBuffer, (char * ) decrypted );

    // syslog( LOG_NOTICE, "Decrypt256() '%s' => '%s'", encrypted, decrypted );

	return;
}




//
// DecryptAFile - Does what it says,  decrypts the named file.
//
int qdc_crypto_decrypt_file( const char * filename )
{
    unsigned char * pContents = NULL;
    int   cbContents = 0;
    FILE *fp;

    try
    {
        // Ensure the file is writable.
        // chmod( filename, _S_IREAD | _S_IWRITE);

        // Open the file.
        fp = fopen( filename, "rb");
        if (fp == NULL) throw 0;

        // Read the entire contents of the file.
        if (fseek(fp, 0, SEEK_END) != 0) throw 0;
        cbContents = ftell(fp);
        pContents = ( unsigned char * )malloc( cbContents );
        if (pContents == NULL) throw 0;
        fseek(fp, 0, SEEK_SET);
        fread(pContents, cbContents, 1, fp);
        if (ftell(fp) != cbContents) throw 0;

        // Close the file.
        fclose(fp);
        fp = NULL;

        // Decrypt the contents of the file using our static key.
        RC4StreamCipher cipher1(get_scatter256(), get_scatter256_size());
        cipher1.Decrypt(pContents, pContents, cbContents);

        // Decrypt the contents of the file using the session key (last 16 bytes of file).
        RC4StreamCipher cipher2(&pContents[cbContents - 16], 16);
        cipher2.Decrypt(pContents, pContents, cbContents);

        // Verify the first 3 bytes are "PIT".
        if ((pContents[0] != 'P') || (pContents[1] != 'I') || (pContents[2] != 'T')) throw 0;

        // Open the file.
        fp = fopen( filename, "wb");
        if (fp == NULL) throw 0;

        // Write the decrypted contents to the file.
        cbContents -= (3 + 16);
        fwrite(&pContents[3], cbContents, 1, fp);
        if (ftell(fp) != cbContents) throw 0;

        // Close the file.
        fclose(fp);
        fp = NULL;

        return 1;
    }
    catch (...) {}

    if (fp != NULL) fclose(fp);
    free(pContents);

    return 0;
}



