#!/bin/bash  

PROJECTDIR=$PWD
BUILDCMD=$1
DEVICE=
CUSTOMER=
CLEAN=

QTDIR=/nfs/tools/qt/generic/bin-static
PATH=/nfs/tools/qt/generic/bin-static/bin:$PATH
DEVICE="generic_src"

for arg in "$@"
    do
    case $arg in
        help)
            echo " all extensions commonlib proxying[_config] usbrdr vWorkspace pntsc pncm qrdesktop[_config]"
            exit 1
            ;;
            
        clean)
            CLEAN=" clean"
            ;;
    esac
done


case "$BUILDCMD" in
    prerequisites)
        PACKAGE_INSTALL_CMD='sudo apt-get -y install '
        sudo apt-get -y update
        $PACKAGE_INSTALL_CMD build-essential
        $PACKAGE_INSTALL_CMD pkg-config
        $PACKAGE_INSTALL_CMD autoconf
        $PACKAGE_INSTALL_CMD automake
        $PACKAGE_INSTALL_CMD libtools
        $PACKAGE_INSTALL_CMD libpng12-dev
        $PACKAGE_INSTALL_CMD libmpg123-dev
        $PACKAGE_INSTALL_CMD libjpeg62-dev
        $PACKAGE_INSTALL_CMD libXext-dev
        $PACKAGE_INSTALL_CMD libao-dev
        $PACKAGE_INSTALL_CMD libasound2-dev
        $PACKAGE_INSTALL_CMD libboost-all-dev
        $PACKAGE_INSTALL_CMD libasio-dev
        $PACKAGE_INSTALL_CMD libx11-dev
        $PACKAGE_INSTALL_CMD libxt-dev
        $PACKAGE_INSTALL_CMD libxcursor-dev
        $PACKAGE_INSTALL_CMD xdg-utils
        $PACKAGE_INSTALL_CMD libpcsclite1
        $PACKAGE_INSTALL_CMD libpcsclite-dev
        $PACKAGE_INSTALL_CMD libxerces-c-dev
        $PACKAGE_INSTALL_CMD libpopt-dev
        $PACKAGE_INSTALL_CMD libspeex-dev
        $PACKAGE_INSTALL_CMD libgstreamer0.10-dev
        $PACKAGE_INSTALL_CMD libgstreamer-plugins-base0.10-dev
        $PACKAGE_INSTALL_CMD gstreamer0.10-tools
        $PACKAGE_INSTALL_CMD gstreamer0.10-plugins-base
        $PACKAGE_INSTALL_CMD gstreamer0.10-plugins-good
        $PACKAGE_INSTALL_CMD gstreamer0.10-plugins-bad
        $PACKAGE_INSTALL_CMD gstreamer0.10-plugins-ugly
        $PACKAGE_INSTALL_CMD gstreamer0.10-ffmpeg
        $PACKAGE_INSTALL_CMD gstreamer0.10-pitfdll
        $PACKAGE_INSTALL_CMD libxinerama-dev
        $PACKAGE_INSTALL_CMD gdebi
        $PACKAGE_INSTALL_CMD wmctrl
        $PACKAGE_INSTALL_CMD libqt4-dev
        $PACKAGE_INSTALL_CMD libssl-dev
        $PACKAGE_INSTALL_CMD libsocksd0-dev
        $PACKAGE_INSTALL_CMD ffMpeg
        $PACKAGE_INSTALL_CMD libgtk2.0-dev
        $PACKAGE_INSTALL_CMD libnspr4-dev
        $PACKAGE_INSTALL_CMD libnss3-dev
        $PACKAGE_INSTALL_CMD libxv-dev
        #needed for qrdesktop 
        $PACKAGE_INSTALL_CMD libao2
        $PACKAGE_INSTALL_CMD libmpeg123

        ;;


    all)
            ./$0 commonlib         $CLEAN
            ./$0 extensions        $CLEAN
            ./$0 vWorkspace        $CLEAN
            ./$0 qrdesktop_config  $CLEAN
            ./$0 qrdesktop         $CLEAN
            ./$0 pntsc             $CLEAN
            ./$0 pncm              $CLEAN
        ;;

    extensions)
            ./$0 proxying_config  $CLEAN 
            ./$0 proxying         $CLEAN
            ./$0 usbrdr           $CLEAN
        ;;

    commonlib)
        cd $PROJECTDIR/lib
        make $CLEAN
        ;;

    proxying_config)
        if test -z $CLEAN
        then
           cd $PROJECTDIR/qrdesktop/dante_proxy
           rm -rf $PROJECTDIR/qrdesktop/dante_proxy/dante-1.3.2
           tar zxvf dante-1.3.2.tar.gz
           cd dante-1.3.2
           /bin/sh configure --enable-static --disable-shared --enable-debug --without-glibc-secure  --without-gssapi --without-krb5 --without-sasl --without-ldap
        fi
        ;;

    proxying)
        cd $PROJECTDIR/qrdesktop/dante_proxy/dante-1.3.2

        if test -z $CLEAN
        then
            make
            cp $PROJECTDIR/qrdesktop/dante_proxy/dante-1.3.2/lib/.libs/libsocks.a   $PROJECTDIR/qrdesktop/dante_proxy/libsocksd.a
        else
            rm qrdesktop/dante_proxy/dante-1.3.2/lib/.libs/libsocks.a
            rm $PROJECTDIR/qrdesktop/dante_proxy/libsocksd.a
            make clean;
        fi
        ;;

    usbrdr)      
        cd $PROJECTDIR/usbrdr/tusbd
        make $CLEAN;
        ;;

    vWorkspace)
        cd $PROJECTDIR/appportal2
        qmake
        make $CLEAN
        ;;

    pntsc)
        cd $PROJECTDIR/qrdesktop/pntsc
        make $CLEAN
        ;;

    pncm)
        #depends on proxying libsocksd.a
        cd $PROJECTDIR/qrdesktop/server
        make $CLEAN
        ;;

    qrdesktop_config)
        if test -z $CLEAN
        then
          cd $PROJECTDIR/qrdesktop/rdesktop
          autoconf
          cp configure.good configure
          /bin/sh configure --enable-smartcard --with-mm-support
        fi
        ;;

    qrdesktop)
        #depends on proxying libsocksd.a
        cd $PROJECTDIR/qrdesktop/rdesktop
        make $CLEAN
        ;;

    output)
        cd $PROJECTDIR
        mkdir -p output/
        mkdir -p output/$DEVICE
        mkdir -p output/$DEVICE/bin
        mkdir -p output/$DEVICE/lib
        mkdir -p output/$DEVICE/etc
        mkdir -p output/$DEVICE/etc/init.d
        mkdir -p output/$DEVICE/man
        mkdir -p output/$DEVICE/man/man1
        mkdir -p output/$DEVICE/share
        mkdir -p output/$DEVICE/share/qrdesktop

        cp -f extensions/usbrdr/tusbd/usbd.ko                          output/$DEVICE/bin/
        
        cp -f appportal2/vWorkspace                                    output/$DEVICE/bin
        cp -f qrdesktop/pntsc/pntsc                                    output/$DEVICE/bin
        cp -f qrdesktop/server/pncm                                    output/$DEVICE/bin
        cp -f qrdesktop/rdesktop/qrdesktop                             output/$DEVICE/bin
        
        cp -f qrdesktop/pntsc/launcher                                 output/$DEVICE/bin
        chmod 755 output/$DEVICE/bin/launcher
        strip output/$DEVICE/bin/launcher
      
        cp -rf qrdesktop/rdesktop/keymaps                              output/$DEVICE/share/qrdesktop
        cp -f  appportal2/AppPortal_en.qm                              output/$DEVICE/share/qrdesktop
        cp -f  qrdesktop/rdesktop/doc/rdesktop.1                       output/$DEVICE/man/man1

        cp -f appportal2/common/appPortalSettings.ini                  output/$DEVICE/etc
        cp -f extensions/usbrdr/tusbd/rc.usbsrvd                       output/$DEVICE/etc/init.d
        
        chmod 644 output/$DEVICE/man/man1/rdesktop.1
        chmod 755 output/$DEVICE/bin/pntsc
        chmod 755 output/$DEVICE/bin/pncm
        chmod 755 output/$DEVICE/bin/qrdesktop
        chmod 755 output/$DEVICE/bin/vWorkspace
        chmod 755 output/$DEVICE/etc/init.d/rc.usbsrvd

        strip output/$DEVICE/bin/pntsc
        strip output/$DEVICE/bin/pncm
        strip output/$DEVICE/bin/vWorkspace

        cd output
        tar -cz $DEVICE > $DEVICE.tgz        
        ;;

esac


exit 1
