#!/bin/sh

BINARY_COMPONENTS="qrdesktop pntsc"
GST_COMPONENTS="pntssrc Qxvimagesink"
test_component=
SUCCESS=0

GST_INSPECT=gst-inspect

if [ -f /usr/bin/gst-inspect-0.10 ]
then
  GST_INSPECT=/usr/bin/gst-inspect-0.10
else
  GST_INSPECT=/usr/bin/gst-inspect
fi

test_gst_component()
{
  $GST_INSPECT $@ >/dev/null 2>&1
  return $?
}

test_binary_component()
{
  $@ -? >/dev/null 2>&1
  return $?
}

test_components()
{
  absent_components=""
  for component_name in $@
  do
    /bin/echo -n " checking for $component_name... "
    $test_component $component_name
    if [ $? -ne $SUCCESS ]
    then
      echo "  the component was not installed or doesn't work properly"
    else
      /bin/echo -e "\tOK"
      continue
    fi
    absent_components="$absent_components ${component_name}"
  done
  if [ "$absent_components" != "" ]
  then
    echo
    echo "  Absent components: $absent_components"
    echo
  fi
}

echo '======================================'
echo 'Performing post-installation checks...'
echo '======================================'

echo ''
echo 'Checking for binaries:'
test_component=test_binary_component
test_components $BINARY_COMPONENTS

if [ "$absent_components" != "" ]
then
  echo 
  echo "Some of binary components that are required were not properly installed."
  echo
else
  echo ' OK'
  echo "All binary components are available."
  echo
fi


echo ''
echo 'Checking for gstreamer packages:'
test_component=test_gst_component
test_components $GST_COMPONENTS

if [ "$absent_components" != "" ]
then
  echo 
  echo "Some of gstreamer components were not properly installed - MMR will unlikely work"
  echo
else
  echo ' OK'
  echo "All gstreamer components are available."
  echo
fi

echo
echo '============================='
echo 'Post-installation checks done'
echo '============================='
echo

