#include "rdesktop.h"
#include "rdpsnd.h"
#include "channel_transport.h"

#define MAX_SESSIONS 256

typedef struct _cmd_event
{
    session_id owner_id;
    int cmd;
    union
    {
        uint32 data;
        session_id id;
    };
} cmd_event;

extern void session_process_event ( channel_session * _session , TRANSPORT_EVENT _event );
channel_session * sessions [ MAX_SESSIONS ];
/*
static int find_empty_session_slot( )
{
    int i;
    for ( i = 0; i < MAX_SESSIONS; ++ i )
    {
        if ( sessions[i] == 0 )
          return i;
    }
    return -1;
}*/

static channel_session * channel_transport_find_session ( session_id id )
{
    int i;
    for ( i = 0; i < MAX_SESSIONS; ++i )
    {
    	if ( sessions[i] == NULL )
    		continue;
        if ( sessions[i]->id == id )
            return sessions[i];
    }
    
    return 0;
}


static void transport_process( STREAM s );

/*------- -------- --------- ---------- ----------- ----------- --------- --------- --------*/

static VCHANNEL * transport_vchannel = NULL;


vpg_result channel_transport_send_message(  STREAM _msg );
vpg_result channel_transport_send_event  ( int _cmd , uint32 _data );


vpg_result channel_session_create_message ( channel_session * _session, STREAM * _msg , DWORD _size )
{
    *_msg = channel_init( transport_vchannel , _size + sizeof ( session_id ) + sizeof(DWORD) );
    out_uint32_le (*_msg, _size + sizeof( session_id ) );
    out_uint32_le (*_msg, MAKE_SESSION_ID(_session->id) );
    return VPG_OK;
}

vpg_result channel_session_free_message   ( channel_session * _session, STREAM _msg )
{
    return VPG_OK;
}

vpg_result channel_session_recv_message   ( channel_session * _session, STREAM * _msg )
{
    *_msg = _session->recieved;
    return VPG_OK;
}

vpg_result channel_session_send_message   ( channel_session * _session, STREAM _msg )
{
    return channel_transport_send_message ( _msg );
}

vpg_result channel_transport_init ( char * _vchannelname , transport_event_handler _event_handler)
{
	int i;

    transport_vchannel = 
        channel_register( _vchannelname, 
                           CHANNEL_OPTION_INITIALIZED,
                           transport_process);

    for ( i = 0; i < MAX_SESSIONS; ++i )
    {
    	sessions[i] = NULL;
    }
    return VPG_OK;
}

void qaudio_transport_init()
{
    channel_transport_send_event ( TRANSPORT_EVENT_CONNECTED , MAKE_SESSION_ID( 0 ) );
}

vpg_result channel_transport_send_event  ( int _cmd , uint32 _data )
{
    STREAM s;
    
    s = channel_init( transport_vchannel , 64 );

    out_uint32_le ( s , 3*sizeof(uint32) ); /* size */
    out_uint32_le ( s , 0 ); /* session id, for transport - is zero */
    out_uint32_le ( s , _cmd ); /* command  */
    out_uint32_le ( s , _data );/* data s*/

    return channel_transport_send_message( s );
}

vpg_result channel_transport_send_message(  STREAM _msg )
{
	s_mark_end(_msg);

    channel_send( _msg , transport_vchannel );    

	return VPG_OK;
}


vpg_result channel_transport_process_system_messages ( STREAM s )
{
    cmd_event * event;
    uint8 *p;
    uint32 size;
    
    in_uint32_le( s , size );

    in_uint8p ( s, p, sizeof ( cmd_event ) );
    event = (cmd_event*)p;
    
    switch ( event->cmd )
    {
        case TRANSPORT_EVENT_CONNECTED:
        {
            channel_session * session;
            
            if (!(event->id & 0x7FFFFFFF))
            {
                if ( ISREMOTE_ID( event->id ) )
                {
                    channel_transport_send_event ( TRANSPORT_EVENT_CONNECTED , MAKE_SESSION_ID( event->id) );
                }
                break;
            }
            
            session = channel_transport_find_session ( event->id );
            if (session == NULL )
            {
                if ( VPG_OK != channel_transport_create_session (&session) )
                    return VPG_FAIL;
               
                session->id = event->id;                     
                channel_transport_send_event ( TRANSPORT_EVENT_CONNECTED , MAKE_SESSION_ID( event->id ) );
            }
            
        }break;
        case TRANSPORT_EVENT_DISCONNECTED:
        {
        
        
            if ( (event->id & 0x7FFFFFFF ) == 0 )
            {
            	int i;
                for ( i = 0; i < MAX_SESSIONS; ++ i )
                {
                    if ( sessions[i] != NULL )
                    {
                        (*sessions[i]).channel_session_process( sessions[i], TRANSPORT_EVENT_DISCONNECTED );
                        channel_transport_destroy_session ( sessions[i] );                        
                    }
                }
           }
           else
           {
                channel_session * session = channel_transport_find_session( event->id );
                if ( session != NULL )
                {
                    session->channel_session_process ( session , TRANSPORT_EVENT_DISCONNECTED);

                    channel_transport_destroy_session ( session );
                    
                }
                else
                {
                    /* log err */
                }
           }
           
        }break;
    }
    return VPG_OK;
}

vpg_result channel_transport_create_session  ( channel_session ** _session )
{
    int i;
    
    for ( i = 0; i < MAX_SESSIONS; ++ i )
    {
        if ( sessions[ i ] == NULL )
        {
            sessions[ i ] = (channel_session*) xmalloc ( sizeof (channel_session ) );
            *_session = sessions[i];
            
            sessions[i]->recieved = NULL;
            sessions[i]->sending = NULL;
            sessions[i]->id = i + 1;
            sessions[i]->channel_session_process = &session_process_event;
            return VPG_OK;
        }
    }
    
    return VPG_FAIL;
}

vpg_result channel_transport_destroy_session ( channel_session * _session )
{
    int i;
    for ( i = 0; i < MAX_SESSIONS; ++ i )
    {
        if ( sessions[i] == _session )
        {
            _session->recieved = NULL;
            _session->sending  = NULL;
            _session->id = 0;
            
            xfree( _session );
            sessions[i] = NULL;
            return VPG_OK;
        }
    
    }
    return VPG_FAIL;
}


/*----------------------------------------------------------------------------------------*/

static void transport_process ( STREAM s )
{
    uint32 session_id, size ;
    
    in_uint32_le( s , size );
    in_uint32_le( s , session_id );

    if ( session_id == 0 )
    {
    	s->p -= sizeof(uint32) + sizeof(uint32);
        channel_transport_process_system_messages ( s );
    }
    else
    {
        channel_session * session = NULL;
        
        session = channel_transport_find_session ( session_id );
        if ( session == NULL )
        {
            /* log error */        
        }
        else
        {
            session->recieved = s;
            session->channel_session_process( session, TRANSPORT_EVENT_MESSAGE_RECIEVED );
            session->recieved = NULL; 
        }
        
    }
   
}

