#ifndef DYNAMIC_CHANNEL_H
#define DYNAMIC_CHANNEL_H

#include <vector>
#include <string>
#include "rdesktop.h"

#ifdef __cplusplus
extern "C"
{
#endif

void dynamic_channel_init();

#ifdef __cplusplus
}
#endif


class DynamicChannel;


class DynamicChannelManager
{
public:
    DynamicChannelManager();
    void setTransportChannel(VCHANNEL* pTransportChannel);

    void addDynamicChannel(DynamicChannel* pDynamicChannel);

    void processCapabilitiesRequestPdu(STREAM s);
    void processCreateRequestPdu(STREAM s);
    void processDataPdu(STREAM s);
    void processFirstDataPdu(STREAM s);
    void processCloseRequestPdu(STREAM s);

    STREAM prepareDataPdu(uint32 channelId, int dataLength);
    void sendDataPdu(STREAM outStrem);

    enum dynamicChannelCommands { CMD_CREATE = 1,
                                  CMD_DATA_FIRST,
                                  CMD_DATA,
                                  CMD_CLOSE,
                                  CMD_CAPS };

private:
    void sendCapabilitiesResponsePdu();
    void sendCreateResponsePdu(uint32 channelId, uint32 creationStatus);

    int readChannelId(STREAM s, uint8 header, uint32* p_channelId);

private:
    VCHANNEL* m_pTransportChannel;
    std::vector<DynamicChannel*> m_dynamicChannels;

    static const int s_maxDynamicChannelDataLength = 1600; // bytes
};


class DynamicChannel
{
public:
    DynamicChannel(const std::string& name);
    void setDynamicChannelManager(DynamicChannelManager* pDynamicChannelManager);

    std::string& getName();
    uint32 getId();
    bool isOpened();

    void open(uint32 id);
    void close();

    virtual void processMessage(STREAM s, uint32 totalLength) = 0;
    virtual void onClose() = 0;

protected:
    DynamicChannelManager* m_pDynamicChannelManager;

private:
    std::string m_name;
    uint32 m_id;
    bool m_isOpened;
};

#endif
