#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <sys/types.h>
#include <sys/socket.h>
#include <sys/select.h>
#include <sys/ioctl.h>
#include <sys/inotify.h>
#include <sys/stat.h>

#include <netinet/in.h>
#include <net/if.h>
#include <unistd.h>
#include <arpa/inet.h>
#include <netdb.h>
#include <ifaddrs.h>

#include <errno.h>
#include <syslog.h>
#include <stdarg.h>
#include <signal.h>
#include <fcntl.h>

#include "appportal_updater.h"



// extern void * hexdump( unsigned char *, unsigned int );

// Header type information and declarations
int                g_send_socket;
struct sockaddr_in g_send_sockaddr;

//do a nonblocking connect
//  return -1 on a system call error, 0 on success
//  sa - host to connect to, filled by caller
//  sock - the socket to connect
//  timeout - how long to wait to connect
inline int
conn_nonb(struct sockaddr_in sa, int sock, int timeout)
{
    int flags = 0, error = 0, ret = 0;
    fd_set  rset, wset;
    socklen_t   len = sizeof(error);
    struct timeval  ts;

    ts.tv_usec = 0;
    ts.tv_sec  = timeout;

    //clear out descriptor sets for select
    //add socket to the descriptor sets
    FD_ZERO(&wset);
    FD_SET(sock, &wset);
    rset = wset;    //structure assignment ok

    //set socket nonblocking flag
    if ( (flags = fcntl(sock, F_GETFL, 0)) < 0)
        return -1;

    if (fcntl(sock, F_SETFL, flags | O_NONBLOCK) < 0)
        return -1;

    //initiate non-blocking connect
    if ( (ret = connect(sock, (struct sockaddr *)&sa, 16)) < 0 )
        if (errno != EINPROGRESS)
            return -1;

    if (ret == 0)   //then connect succeeded right away
        goto done;

    //we are waiting for connect to complete now
    // Specifically wait for write permission
    // DO NOT check for read permission or the select will trigger incorrectly
    //     if ( (ret = select(sock + 1, &rset, &wset, NULL, (timeout) ? &ts : NULL)) < 0)
    if ( (ret = select(sock + 1, NULL , &wset, NULL, (timeout) ? &ts : NULL)) < 0)
        return -1;
    if (ret == 0)   //we had a timeout
    {
        errno = ETIMEDOUT;
        return -1;
    }

    // we had a positive return so a descriptor is ready
    // Again, as above we are only interested in the write signal
    //    if (FD_ISSET(sock, &rset) || FD_ISSET(sock, &wset))
    if (FD_ISSET(sock, &wset))
    {
        if (getsockopt(sock, SOL_SOCKET, SO_ERROR, &error, &len) < 0) return -1;
    }
    else return -1;

    if (error)  //check if we had a socket error
    {
        errno = error;
        return -1;
    }

done:
    //put socket back in blocking mode
    if (fcntl(sock, F_SETFL, flags) < 0)
        return -1;

    return 0;
}


// Init the network
void au_network_initialize()
{
    syslog( LOG_NOTICE, "Initializing the network" );

    // Initialize the broker sender?
    syslog( LOG_NOTICE, "Network init complete" );
}

// Deinit network
void au_network_deinitialize()
{
    syslog( LOG_NOTICE, "Deinitializing network" );

}

// Need something to send the network message
int au_network_send_update_message( char * buffer, int length, char * broker)
{
    int send_socket = -1,
    status = 0;
    ssize_t rc = 0;
    int result = 0;
    int slaveSocket = -1;
    int retval;
    ssize_t bytes = 0;
    fd_set rfds;
    struct timeval tv;
    struct hostent *hostPtr = NULL;
    struct sockaddr_in serverName =
        {
            0
        };
    socklen_t  sockaddr_len;

    // Do this monolithically for now.


    // Create the send socket
    send_socket = socket( AF_INET, SOCK_STREAM, IPPROTO_TCP );
    if ( -1 == send_socket )
    {
        syslog(LOG_ERR, "Failed to create client socket");
        return -1;
    }

    hostPtr = gethostbyname( "localhost" );
    if (NULL == hostPtr)
    {
        hostPtr = gethostbyaddr( broker , strnlen(broker , 255), AF_INET );
        if (NULL == hostPtr)
        {
            syslog( LOG_ERR, "Error resolving server address" );
            close( send_socket );
            return -1;
        }
    }

    serverName.sin_family = AF_INET;
    serverName.sin_port   = htons( AU_NETWORK_PORT );

    // syslog( LOG_DEBUG,  "Server name of server = %s ",  hostPtr->h_name );

    memcpy(&serverName.sin_addr,
           hostPtr->h_addr,
           hostPtr->h_length);

    // Connect the send socket
    //    status = connect( send_socket,
    //                      (struct sockaddr*) &serverName,
    //                      sizeof(serverName) );

    status = conn_nonb( serverName, send_socket, 5 );

    if ( -1 == status )
    {
        syslog( LOG_ERR, "Failed on the connect() call: %s", strerror( errno ) );
        result = -1;
    }
    else
    {
        // Send the length header
        rc = write( send_socket, (char *) &length, sizeof(int) );

        // Send the message
        rc = write( send_socket, buffer, length );

        // handle any errors
        if ( rc == -1 )
        {
            syslog( LOG_ERR, "Error while writing to socket: %s", strerror( errno ) );
            result = -1;
        }
    }

    close ( send_socket );

    return (result);
}

// Shutdown and cleanup
