#include "pnsslcli.h"
#include <cstdlib>
#include <syslog.h>
#include <sys/wait.h>
#include <signal.h>
#include <cassert>
#include <semaphore.h>

extern char g_localaddr[64];
extern char g_ssladdr[64];
extern char g_remoteaddr[64];
extern char g_username[64];
// ------------------------------------------------------------------------------
//  main() arguments:
//  1) local loopback address <host[:port]>, default port is 3389
//  2) SSL VPN server address <host[:port]>, default port is 443
//  3) RDP hostname and 4) username, they're passed to VPN server
//      5) PNTSC pid, to say to that initialization is finished
// ------------------------------------------------------------------------------

extern "C"
void *
pnsslcli_thread ( sem_t * ssl_semaphore )
{
    int err_code = 0;
    session_ptr work_session;
    
    syslog (LOG_NOTICE, "pnsslcli_thread: start session (g_ssladdr = %s)", g_ssladdr);

    try
    {
        asio::io_service io_service;

        // connect to SSL VPN server
        char *p = strstr (g_ssladdr, ":");
        if (p)
        {
            *p++ = '\0';
        }
        tcp::resolver resolver (io_service);
        syslog (LOG_NOTICE, "pnsslcli_thread: g_ssladdr = %s", g_ssladdr );
        tcp::resolver::query query (tcp::v4 (), g_ssladdr, p ? p : "443");
        syslog (LOG_NOTICE, "pnsslcli_thread: Created query for %s:%s", g_ssladdr, p ? p : "443");
        tcp::resolver::iterator iterator = resolver.resolve (query);
        syslog (LOG_NOTICE, "pnsslcli_thread: Resolved query for %s:%s", g_ssladdr, p ? p : "443");

        // listen to local loopback
        short int port;
        p = strchr (g_localaddr, ':');
        if (p)
        {
            *p++ = '\0';
            port = std::atoi (p);
        }
        else
        {
            port = 3389;
        }

        tcp::endpoint endpoint(asio::ip::address::from_string (g_localaddr), port);
        syslog (LOG_NOTICE, "pnsslcli_thread: start listening %s:%d", g_localaddr, port);
        server_ptr server(new pnssl_server(io_service, endpoint, iterator, g_remoteaddr, g_username));


        // Allow main thread to work
        sem_post( ssl_semaphore );

        io_service.run ();
    }
    catch (const asio::error_code & e)
    {
        if (e != asio::error::eof)
        {
            syslog (LOG_NOTICE, "pnsslcli_thread: error: %s", e.message().c_str());
        }
        else
        {
            syslog (LOG_NOTICE, "pnsslcli_thread: session was closed");
        }
    }
    catch (std::exception & e)
    {
        syslog (LOG_NOTICE, "pnsslcli_thread: an exception was thrown: %s\n", e.what ());
        err_code = 1;
    }
    catch (...)
    {
        syslog (LOG_NOTICE, "pnsslcli_thread: an unknown exception was thrown\n");
    }

    syslog (LOG_NOTICE, "pnsslcli_thread: session was closed");
    closelog ();
    exit (1);
}
