/* -*- c-basic-offset: 8 -*-
   rdesktop: A Remote Desktop Protocol client.
   Seamless Windows support
   Copyright (C) Peter Astrand <astrand@cendio.se> 2005-2006

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include "../../common/common.h"

#include "rdesktop.h"
#include <stdarg.h>
#include <assert.h>
#include <iconv.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <unistd.h>
#include <arpa/inet.h>
#include <errno.h>
#include <ctype.h>
#include <sys/select.h>
#include <pthread.h>
#include <pwd.h>
#ifdef __cplusplus
extern "C" {
#endif
#include "../../common/crypto.h"
#ifdef __cplusplus
}
#endif

// #define WITH_DEBUG_PNVCHANNEL

#ifdef WITH_DEBUG_PNVCHANNEL
#define DEBUG_PNVCHANNEL(args) printf args;
#else
#define DEBUG_PNVCHANNEL(args)
#endif

#define LISTEN_UDP_PORT 42333
#define PNSERVER_PORT   42332

#define WITH_MM_SUPPORT
#ifdef WITH_MM_SUPPORT
BOUNDS ui_mm_get_screen_bounds_pntsc_process(int num);
int ui_mm_number_of_screens();
#endif

extern RD_BOOL g_seamless_rdp;
extern char g_sessionname[64];
extern char g_shell[128];
extern char g_tgtfilename[256];
extern char g_response_message[256];
extern char * create_tgt_response(char * filename);

extern RD_BOOL g_fullscreen;
extern RD_BOOL g_span_monitors;

static VCHANNEL *pntsc_channel;
static VCHANNEL *pntss_channel;
static VCHANNEL *pnkerb_channel;

static char message[256];

void * feedback_thread_function(void * param);

int BinToHex(unsigned char* pBin, int cbBin, char* pHex, int cbHex)
{
    int i;

    // Initialize the output buffer.
    memset(pHex, 0, cbHex);

    // Check the size of the output buffer.
    if (cbHex < ((cbBin * 2) + 1))
        return 0;

    // Convert each byte to its hexadecimal equivalent.
    for (i = 0; i < cbBin; i++)
    {
        pHex += sprintf(pHex, "%02X", pBin[i]);
    }
    *pHex = '\0';

    return cbBin;
}


static unsigned int
pnts_send(const char * data, VCHANNEL *channel)
{
    STREAM s;
    unsigned int i;
    size_t len8, len16, inlen, outlen;
    char buf8[1024], errstring[1024];
    char *buf16, *in, *out;
    static iconv_t iconv_sdescr;

    len8 = strlen (data) + 1;
    assert(len8 < (sizeof(buf8) - 1));
    for (i = 0; i < len8; i++)
    {
        buf8[i] = toupper(data[i]);
    }
    buf8[i] = 0;
    len16 = len8*2;
    buf16 = xmalloc(len16);
    in = buf8;
    out = buf16;
    inlen = len8;
    outlen = len16;

    iconv_sdescr = iconv_open ("UTF-16LE", "ASCII");
    if (iconv(iconv_sdescr, &in, (size_t*)&inlen, &out, (size_t*)&outlen) == (size_t)-1)
    {
        sprintf( errstring, "iconv failed (%i).", errno );
        syslog(LOG_NOTICE, "%s", errstring);
        perror ( errstring );
    }
    else
    {
        // syslog (LOG_NOTICE, "pnts send: %s", buf8);
    }

    s = channel_init(channel, len16);
    memcpy (s->p, buf16, len16);
    s->p += len16;
    s_mark_end(s);

    channel_send(s, channel);

    return 0;
}

unsigned int
pnts_send_message(const char *message)
{
    return pnts_send(message, pntss_channel);
}

RD_BOOL qseamless_process_line(const char *line);

static void
pntsc_process(STREAM s)
{
    size_t pkglen;
    char *buf16, *buf8;
    char *in, *out;
    static iconv_t iconv_rdescr;

    char errstring[1024];
    pkglen = s->end - s->p;
    buf16 = xmalloc(pkglen);
    memcpy(buf16, s->p, pkglen);
    in = buf16;
    buf8 = xmalloc(pkglen);
    memset(buf8, '\0', pkglen);
    out = buf8;
    iconv_rdescr = iconv_open("UTF-8", "UTF-16LE");

    if (iconv(iconv_rdescr, &in, &pkglen, &out, &pkglen) == (size_t)-1)
    {
        sprintf( errstring, "iconv failed (%i).", errno );
        perror ( errstring );
    }
    else
    {
        syslog (LOG_NOTICE, "pntsc recv: %s", buf8);
        if (!strncmp (buf8, "ID", 2) || !strncmp (buf8, "IS", 2))
        {
            pnts_send_message(g_response_message);

            if (!strncmp (buf8, "ID", 2))
            {
                if (g_seamless_rdp)
                {
                    pnts_send ("DM1", pntss_channel);
                    ui_seamless_begin(0); // 0=not hide
                }
            }

#ifdef WITH_MM_SUPPORT
    // multimonitor support
    if (g_fullscreen && g_span_monitors)
    {
        char message[128] = "";
        int len = sprintf(message, "DM%d", 0);

        message[len] = '\0';

        syslog (LOG_NOTICE, "%s", message);
        pnts_send_message(message);

        // get number of screens
        int scr_counter = ui_mm_number_of_screens();

        if (scr_counter > 1)
        {
            syslog (LOG_NOTICE, "Multi-monitor: %d screen(s) detected", scr_counter);
            int i;

            for (i = 0; i < scr_counter; ++i)
            {
                BOUNDS bounds = ui_mm_get_screen_bounds_pntsc_process(i);

                syslog (LOG_NOTICE, "Monitor #%i bounds: %i,%i,%i,%i", i, bounds.left, bounds.top, bounds.right, bounds.bottom);

                len = sprintf(message, "AM%d,%d,%d,%d", bounds.left, bounds.top, bounds.right, bounds.bottom);
                message[len] = '\0';
                syslog (LOG_NOTICE, "%s", message);
                pnts_send_message(message);
            }
        }
    }
#endif
        }
        else
        {
            qseamless_process_line(buf8);
        }
    }

    iconv_close( iconv_rdescr );
    xfree(buf16);
    xfree(buf8);
}

static void
pnkerb_process(STREAM s)
{
    size_t pkglen;
    char *buf16, *buf8;
    char *in, *out;
    static iconv_t iconv_rdescr;

    char errstring[1024];
    pkglen = s->end - s->p;
    buf16 = xmalloc(pkglen);
    memcpy(buf16, s->p, pkglen);
    in = buf16;
    buf8 = xmalloc(pkglen);
    memset(buf8, '\0', pkglen);
    out = buf8;
    iconv_rdescr = iconv_open("UTF-8", "UTF-16LE");

    if (iconv(iconv_rdescr, &in, &pkglen, &out, &pkglen) == (size_t)-1)
    {
        sprintf( errstring, "iconv failed (%i).", errno );
        perror ( errstring );
    }
    else
    {
        // syslog (LOG_NOTICE, "pnkerb recv: %s", buf8);
        char* message = create_tgt_response(g_tgtfilename);

        if (message != NULL)
        {
            pnts_send (message, pnkerb_channel);
            free(message);
        }
        else
        {
            // DCT secure ticket handling here
            char  u8_buf[1024];
            FILE * ifile;

            memset( u8_buf, 0x0, sizeof( u8_buf ) );

            syslog (LOG_NOTICE, "pnkerb attempting to send secure ticket.");

            // Use the file passed from the pntsc process.
            // Read in the bytes here

            char userSysName[128];
            struct passwd *pw;
            pw = getpwuid( getuid() );
            if ( pw == NULL || pw->pw_name == NULL )
            {
                syslog( LOG_ERR, "Unable to get system user name!" );
                assert( pw != NULL && pw->pw_name != NULL );
            }
            else
            {
                strcpy( userSysName, pw->pw_name );
            }
            char ticketFile[256];
            sprintf( ticketFile, Q_SECURE_TICKET_FILE_TEMPLATE, userSysName );

            ifile = fopen( ticketFile, "rb" );

            // If the file was found
            if ( ifile != NULL )
            {
                fread( u8_buf, 1, sizeof( u8_buf ), ifile );
                fclose ( ifile );
                pnts_send ( u8_buf, pnkerb_channel );
                remove( ticketFile );
            }
            else
            {
                syslog ( LOG_NOTICE, "Could not find the secure ticket file!" );
            }

        }
    }

    iconv_close( iconv_rdescr );
    xfree( buf16 );
    xfree( buf8 );
}

int
pnts_init (const char * remote_endp)
{
    int res;
    char error_string [256];

    pntsc_channel =
        channel_register("pntsc", CHANNEL_OPTION_INITIALIZED | CHANNEL_OPTION_ENCRYPT_RDP,
                 pntsc_process);
    pntss_channel =
        channel_register("pntss", CHANNEL_OPTION_INITIALIZED | CHANNEL_OPTION_ENCRYPT_RDP,
                 NULL);
    pnkerb_channel =
        channel_register("PNKERB", CHANNEL_OPTION_INITIALIZED | CHANNEL_OPTION_ENCRYPT_RDP,
                 pnkerb_process);
    if (pntsc_channel == NULL || pntss_channel == NULL || pnkerb_channel == NULL)
    {
        syslog (LOG_NOTICE, "pnctrl: unable to open vchannel");
        return -1;
    }

    pthread_t feedback_thread;
    res = pthread_create (&feedback_thread, NULL, feedback_thread_function, NULL);
    if (res)
    {
        strerror_r (errno, error_string, sizeof (error_string));
        syslog (LOG_NOTICE, "unable to create feedback thread: %s", error_string);
        return -1;
    }

    return 0;
}

void *
feedback_thread_function(void * param)
{
    int listen_socket = -1, feedback_socket = -1, res;
    unsigned int maxfd, n;
    socklen_t len = sizeof (struct sockaddr);
    struct sockaddr_in thisaddr, servaddr, recvaddr;
    struct ip_mreq mreq;
    const int on = 1;
    fd_set rfds, wfds;
    char error_string [256];

    syslog (LOG_NOTICE, "feedback thread started");

    // create multicast socket and listen to it for ping message from control server
    listen_socket = socket (AF_INET, SOCK_DGRAM, 0);
    if (listen_socket == -1)
    {
        strerror_r (errno, error_string, sizeof (error_string));
        syslog (LOG_NOTICE, "control socket error: %s", error_string);
        exit (1);
    }
    mreq.imr_multiaddr.s_addr = inet_addr ("225.1.1.1");
    mreq.imr_interface.s_addr = htonl(INADDR_LOOPBACK);
    thisaddr.sin_family = AF_INET;
    thisaddr.sin_addr.s_addr = htonl(INADDR_ANY);
    thisaddr.sin_port = htons(LISTEN_UDP_PORT);
    setsockopt (listen_socket, SOL_SOCKET, SO_REUSEADDR, &on, sizeof (on));
    if (bind (listen_socket, (struct sockaddr* )&thisaddr, sizeof (thisaddr)) == -1)
    {
        strerror_r (errno, error_string, sizeof (error_string));
        syslog (LOG_NOTICE, "pnctrl bind error: %s", error_string);
        exit (1);
    }

    if (setsockopt (listen_socket, IPPROTO_IP, IP_ADD_MEMBERSHIP,
                    (char *)&mreq, sizeof (mreq)) < 0)
    {
        strerror_r (errno, error_string, sizeof (error_string));
        syslog (LOG_NOTICE, "joining multicast group error: %s", error_string);
        exit (1);
    }

    FD_ZERO (&rfds);
    FD_ZERO (&wfds);

    if ((feedback_socket = socket (AF_INET, SOCK_STREAM, 0)) == -1)
    {
        strerror_r (errno, error_string, sizeof (error_string));
        syslog (LOG_NOTICE, "creating socket to control error: %s", error_string);
    }
    else
    {
        servaddr.sin_family = AF_INET;
        servaddr.sin_addr.s_addr = inet_addr ("127.0.0.1");
        servaddr.sin_port = htons(PNSERVER_PORT);
        if ((res = connect (feedback_socket, (struct sockaddr *)&servaddr, len)) < 0)
        {
            strerror_r (errno, error_string, sizeof (error_string));
            syslog (LOG_NOTICE, "unable to connect to a control server: %s", error_string);
            close (feedback_socket);
            feedback_socket = -1;
        }
        else
        {
            FD_SET (feedback_socket, &wfds);
        }
    }

    for (;;)
    {
        FD_SET (listen_socket, &rfds);
        if (feedback_socket != -1)
        {
            FD_SET (feedback_socket, &rfds);
        }
        maxfd = (listen_socket > feedback_socket) ? listen_socket + 1 : feedback_socket + 1;
        res = select (maxfd, &rfds, &wfds, NULL, NULL);
        if (res < 0)
        {
            if (errno == EINTR)
            {
                continue;
            }
            else
            {
                strerror_r (errno, error_string, sizeof (error_string));
                syslog (LOG_NOTICE, "pnctrl select error: %s", error_string);
                exit (1);
            }
        }
        else
        {
            syslog (LOG_NOTICE, "select");
            if (FD_ISSET(listen_socket, &rfds))
            {
                // control server have sent a ping message
                len = sizeof (recvaddr);
                n = recvfrom (listen_socket, message, 255, 0, (struct sockaddr *)&recvaddr, &len);
                message[n] = '\0';
                syslog (LOG_NOTICE, "udp recv: %s", message);
                if (atoi (message) != PNSERVER_PORT)
                {
                    continue;
                }

/* Why are we resetting the socket connection?  this is causing
 * the session server to shut down as this is sometimes the last
 * session connection avaiable
 */

/*
                // reopen connection to control server
                if (feedback_socket != -1)
                {
                    close (feedback_socket);
                }
                if ((feedback_socket = socket (AF_INET, SOCK_STREAM, 0)) == -1)
                {
                    strerror_r (errno, error_string, sizeof (error_string));
                    syslog (LOG_NOTICE, "creating socket to control error: %s", error_string);
                    maxfd = listen_socket + 1;
                    continue;
                }
                servaddr.sin_family = AF_INET;
                servaddr.sin_addr.s_addr = inet_addr("127.0.0.1");
                servaddr.sin_port = htons(PNSERVER_PORT);
                if ((res = connect (feedback_socket, (struct sockaddr *)&servaddr, len)) < 0)
                {
                    strerror_r (errno, error_string, sizeof (error_string));
                    syslog (LOG_NOTICE, "unable to connect to control: %s", error_string);
                    close (feedback_socket);
                    maxfd = listen_socket + 1;
                    continue;
                }
                FD_SET (feedback_socket, &wfds);
*/
                continue;
            }
            if (FD_ISSET(feedback_socket, &wfds))
            {
                if (g_sessionname[0])
                {
                    strcpy (message, g_sessionname);
                }
                else
                {
                    n = sprintf (message, "%u", getpid());
                    message[n] = '\0';
                }
                syslog (LOG_NOTICE, "write %s", message);
                write (feedback_socket, message, strlen (message));
                syslog (LOG_NOTICE, "written");
                FD_ZERO (&wfds);
            }
            if (FD_ISSET(feedback_socket, &rfds))
            {
                res = read (feedback_socket, message, 255);
                syslog (LOG_NOTICE, "Incoming message in feedback socket");
                if (res != 0)
                {
                    message[res] = '\0';
                    syslog (LOG_NOTICE, "read message: %s", message);
                    strcpy(g_response_message, message);
                    pnts_send (message, pntss_channel);
                    FD_SET (feedback_socket, &rfds);
                }
                else
                {
                    syslog (LOG_NOTICE, "Read error when reading incoming message in feedback socket (%i)", res );
                    close (feedback_socket);
                    feedback_socket = -1;
                }
            }
        }
    } // for (;;)
}

        // pntsc: client <-- server
        // pntss: client --> server
