#include "rdesktop.h"
#include "qaudio.h"
#include "channel_transport.h"
#include "qplayer.h"
#include "qrecorder.h"
#include "qaudio_driver.h"

#include "qaudio_alsa_drv.h"

#include "qaudio_codec.h"

#define PAUSED 1
#define PLAYING 2
#define RECORDING 3
#define STOPPED 4


#define SYSTEM 1
#define WODM 2
#define WIDM 3
#define WAVE 4

#define REQUEST 1
#define RESPONSE 2
#define DATA 3

#define SYSTEM_DISCONNECTED 0
#define SYSTEM_CONNECTED 1
#define SYSTEM_SETTINGS 2

#define DATA_FORMAT_WAVE  'WAVE'
#define DATA_FORMAT_SPEEX 'SPX '
#define DATA_FORMAT_MP3   'MP3 '

#define WODM_CONFIGURE 512
#define WIDM_CONFIGURE 1024

#define WODM_OPEN 5
#define WODM_CLOSE 6
#define WODM_RESET 12


#define WIDM_OPEN 52
#define WIDM_CLOSE 53

typedef struct
{
    channel_session * transport;

    audio_player * player;
    audio_recorder * recorder;        
    
    speex_encoder * speex_encoder;
    mp3decoder    * mp3_decoder;

} audio_session;

typedef struct
{
	uint32 low;
	uint32 high;
} uint64_s;

typedef union
{
	void * ptr;
	uint64_s value;
} ptr_uint64;

typedef struct _qds_settings
{
	uint32 use_plugin;
	uint32 remote_sound;
	uint32 do_not_play;
	uint32 channels;
	uint32 rate;
	uint32 buffering;
}qds_settings;

typedef struct _qus_settings
{
	uint32 use_compression;
	uint32 quality;
	uint32 complexity;
	uint32 use_enh;
} qus_settings;

qds_settings g_qds_settings;
qus_settings g_qus_settings;

#define out_uint64_le(s , v) { ptr_uint64 a; a.ptr = v; out_uint32_le( s, a.value.low); out_uint32_le( s, a.value.high); }

#define MAX_AUDIO_SESSIONS 16

static audio_session * audio_sessions [ MAX_AUDIO_SESSIONS ];

audio_session * create_audio_session  ( channel_session * _session );
void            destroy_audio_session ( audio_session * _session   );
audio_session * find_audio_session    ( channel_session * _session );

void session_process_message ( channel_session * _session , STREAM _msg );
void session_process_system  ( channel_session * _session , STREAM _msg );
void session_process_wodm    ( channel_session * _session , STREAM _msg );
void session_process_widm    ( channel_session * _session , STREAM _msg );
void session_process_data    ( channel_session * _session , STREAM _msg );

void wodm_open       ( channel_session * _session , STREAM _msg );
void wodm_close      ( channel_session * _session , STREAM _msg );
void wodm_configure  ( channel_session * _session , STREAM _msg );
void wodm_reset      ( channel_session * _session , STREAM _msg );

void widm_open       ( channel_session * _session , STREAM _msg );
void widm_close      ( channel_session * _session , STREAM _msg );
void widm_configure  ( channel_session * _session , STREAM _msg );

void process_wave    ( channel_session * _session , uint8* _data , uint32 _size );
void process_mp3     ( channel_session * _session , uint8* _data , uint32 _size );

void send_system_settings ( channel_session * _session , STREAM _msg );

void qaudio_process();

int qaudio_is_not_local()
{
    return ( g_qds_settings.remote_sound );
}

void session_process_event ( channel_session * _session , TRANSPORT_EVENT _event )
{
    switch ( _event )
    {
        case TRANSPORT_EVENT_WAIT_TIMEOUT:
            {
            }break;
        case TRANSPORT_EVENT_CONNECTED:
            {
            }break;
        case TRANSPORT_EVENT_DISCONNECTED:
            {
            }break;
        case TRANSPORT_EVENT_ACCEPTED:
            {        
            }break;        
        case TRANSPORT_EVENT_MESSAGE_RECIEVED:
            {
                session_process_message( _session, _session->recieved );
            }break;
        case TRANSPORT_EVENT_MESSAGE_SENT:
            {
            }break;

        case TRANSPORT_EVENT_SYSTEM_ERROR:
            {
            }break;
        case TRANSPORT_EVENT_EXIT:
            {
            }break;
        default:
            {
            }break;
    }

} 

void session_process_message ( channel_session * _session , STREAM _msg )
{
    uint32 content_type;
    in_uint32_le( _msg , content_type );
    
    switch ( content_type )
    {
        case SYSTEM:
            {
                session_process_system( _session, _msg );
            }break;
        case WODM:
            {
                session_process_wodm( _session, _msg );
            }break;
        case WIDM:
            {
                session_process_widm( _session , _msg );
            }break;
        case WAVE:
            {
                session_process_data( _session, _msg );
            }break;
        default:
            {
            }break;
    }    
}

void session_process_system ( channel_session * _session, STREAM _msg )
{
    uint32 message_type, message;
    
    in_uint32_le( _msg , message_type );
    in_uint32_le( _msg , message );
    
    if ( message_type != REQUEST )
    {
        return;
    }
    
    switch ( message )
    {
        case SYSTEM_SETTINGS:
            {
                send_system_settings ( _session, _msg );
            }break;
        default:
            {
            }break;
    }
}

void send_system_settings ( channel_session * _session , STREAM _msg )
{
   STREAM response;   
   channel_session_create_message ( _session, &response , sizeof(uint32) * 9 );
   
   out_uint32_le ( response , SYSTEM );
   out_uint32_le ( response , RESPONSE );
   out_uint32_le ( response , SYSTEM_SETTINGS );
   out_uint32_le ( response , VPG_OK );


   out_uint32_le ( response , g_qds_settings.use_plugin );
   out_uint32_le ( response , g_qds_settings.remote_sound );
   out_uint32_le ( response , DATA_FORMAT_WAVE );
   out_uint32_le ( response , g_qds_settings.channels );
   out_uint32_le ( response , g_qds_settings.rate );
   
   channel_session_send_message ( _session , response );
}

void session_process_wodm ( channel_session * _session, STREAM _msg )
{
    uint32 message_type, message;

    in_uint32_le( _msg , message_type );
    in_uint32_le( _msg , message );

    if ( message_type != REQUEST )
    {
        return;
    }
    
    switch ( message )
    {
        case WODM_OPEN:
            {
                wodm_open(_session, _msg );
            }break;
        case WODM_RESET:
            { 
                wodm_reset(_session , _msg );
            }break;
        case WODM_CLOSE:
            {
                wodm_close(_session , _msg );
            }break;
        case WODM_CONFIGURE:
            {
                wodm_configure(_session , _msg );
            }break;
        default:
            {
            }break;
    }    
}

void session_process_widm ( channel_session * _session , STREAM _msg )
{
    uint32 message_type, message;
    
    in_uint32_le(_msg , message_type );
    in_uint32_le(_msg , message );
    
    if ( message_type != REQUEST )
    {
        return;
    }
    
    switch ( message )
    {
        case WIDM_OPEN:
            {
                widm_open ( _session , _msg );
            }break;
        case WIDM_CLOSE:
            {
                widm_close( _session , _msg );                
            }break;
        case WIDM_CONFIGURE:
            {
                widm_configure( _session , _msg );
            }break;
    }
}

void session_process_data ( channel_session * _session , STREAM _msg )
{
    uint32 message_type, message;

    in_uint32_le ( _msg , message_type );
    in_uint32_le ( _msg , message );
    
    if ( ( message_type != REQUEST ) || ( message != DATA ) || (g_qds_settings.do_not_play ) )
    {
        return;
    }
    
    {/* data-message processing */
        uint64 remote_handle;
        uint32 wave_format;
        uint32 uncompressed_data_size;
        uint32 compressed_data_size;
        
        uint8 * data;
        
        in_uint64_le( _msg , remote_handle );
        in_uint32_le( _msg , wave_format );
        in_uint32_le( _msg , uncompressed_data_size );
        in_uint32_le( _msg , compressed_data_size );
        
        in_uint8p( _msg , data , compressed_data_size );
        
        switch ( wave_format )
        {
            case DATA_FORMAT_WAVE:
                {
                    process_wave (_session, data , uncompressed_data_size );
                }break;
            case DATA_FORMAT_MP3:
                {
                    process_mp3 (_session, data , compressed_data_size );
                }break;
            default:
                {
                }break;
        }
    }
}

void wodm_open ( channel_session * _session , STREAM _msg )
{
    audio_session * audio = NULL;
    STREAM response = NULL;
    
    audio = create_audio_session ( _session );


    if ( audio != NULL )
    {
        WAVEFORMATEX wfe;
        uint32 flags;
        uint32 low_handle , high_handle;

        in_uint32_le ( _msg , low_handle );
        in_uint32_le ( _msg , high_handle );
        
        in_uint16_le ( _msg , wfe.wFormatTag );
        in_uint16_le ( _msg , wfe.nChannels );
        in_uint32_le ( _msg , wfe.nSamplesPerSec );
        in_uint32_le ( _msg , wfe.nAvgBytesPerSec );
        in_uint16_le ( _msg , wfe.nBlockAlign );        
        in_uint32_le ( _msg , flags );

        wfe.nSamplesPerSec = 44100;
        wfe.nChannels = 2;
        wfe.nBlockAlign = 4;
        wfe.wBitsPerSample = 16;
        wfe.nAvgBytesPerSec = wfe.nBlockAlign * wfe.nSamplesPerSec;

        audio->player = create_player ( &wfe );

        audio->mp3_decoder = mp3codec_create_decoder(wfe.nSamplesPerSec, wfe.nChannels );
    }
    
    channel_session_create_message ( _session , &response , sizeof (uint32) * 4 + sizeof( uint64 ) );

    out_uint32_le ( response , WODM );
    out_uint32_le ( response , RESPONSE );
    out_uint32_le ( response , WODM_OPEN );

    if ( (audio == NULL) || (audio->player == NULL ) )
    {
        out_uint32_le ( response , 1 ); /* error*/
        out_uint32_le ( response , 0 ); out_uint32_le ( response , 0 ); /*uint64 handler is NULL*/
        
        if ( audio != NULL )
        {
            destroy_audio_session ( audio );            
        }
    }
    else
    {
        out_uint32_le ( response , 0  ); /*OK*/
        out_uint64_le ( response , audio->player->handle );
    }


    channel_session_send_message ( _session , response );         
}

void wodm_close ( channel_session * _session , STREAM _msg )
{
    audio_session * audio = NULL;
    
    
    audio = find_audio_session ( _session );
    if (audio == NULL )
    {
        /*log error*/
        return;
    }

    destroy_audio_session( audio );
}

void wodm_reset ( channel_session * _session , STREAM _msg )
{
    audio_session * audio = NULL;

    audio = find_audio_session ( _session );
    if (audio == NULL )
    {
        /*log error*/
        return;
    }
    
    reset_player ( audio->player );    
}

void wodm_configure ( channel_session * _session , STREAM _msg )
{
    audio_session * audio = NULL;

    audio = find_audio_session ( _session );
    if (audio == NULL )
    {
        /*log error*/
        return;
    }
    else
    {
        uint32 pitch , volume , rate;
        
        in_uint32_le ( _msg , pitch );
        in_uint32_le ( _msg , volume );
        in_uint32_le ( _msg , rate );
        
        configure_player ( audio->player , pitch , volume , rate );    
    }

}

void widm_open ( channel_session * _session , STREAM _msg )
{
	WAVEFORMATEX wfe;
	uint32 flags;

	if ( _session == NULL )
		return;

	in_uint32_le( _msg , flags );
	in_uint32_le( _msg , flags ); /* ignoring 64 bits device id */
	in_uint16_le( _msg , wfe.wFormatTag);
	in_uint16_le( _msg , wfe.nChannels);
	in_uint32_le( _msg , wfe.nSamplesPerSec);
	in_uint32_le( _msg , wfe.nAvgBytesPerSec );
	in_uint16_le( _msg , wfe.nBlockAlign);
	in_uint32_le( _msg , flags );

	wfe.wBitsPerSample = (wfe.nBlockAlign / wfe.nChannels) * 8;

	{
		STREAM response;
		audio_session * audio = create_audio_session( _session );

		audio->recorder = create_recorder(&wfe) ;

		audio->speex_encoder = codec_create(&wfe, 5 , 3);

		channel_session_create_message( _session , &response , 4* sizeof(uint32) + sizeof(uint64));

		out_uint32_le( response , WIDM );
		out_uint32_le( response , RESPONSE );
		out_uint32_le( response , WIDM_OPEN );

		if (audio->recorder != NULL)
		{
			out_uint32_le( response , 0 ); /* MMSYSERR_NOERROR */
			out_uint64_le( response , audio->recorder->handle);
		}
		else
		{
			out_uint32_le( response , 1 ); /* MMSYSERR_ERROR */
			out_uint32_le( response , 0 );
			out_uint32_le( response , 0 );

			destroy_audio_session( audio );
		}

		channel_session_send_message( _session , response );
	}

}

void widm_close ( channel_session * _session, STREAM _msg )
{
	audio_session * session = find_audio_session( _session );
	if ( session == NULL )
		return;

	destroy_audio_session( session );
}

void widm_configure ( channel_session * _session , STREAM _msg )
{
	uint32 status;

	audio_session * session = find_audio_session(_session);
	if ( session == NULL )
		return;

	in_uint32_le( _msg , status );

	configure_recorder( session->recorder, status );
}

void process_wave ( channel_session * _session , uint8 * _data, uint32 _size )
{
    audio_session * audio = NULL;

    audio = find_audio_session ( _session );
    if (audio == NULL )
    {
      /*log error*/
      return;
    }
    if ( audio->player == NULL )
    {
        /*log error*/
        return;
    }
    
    play_wave ( audio->player , _data , _size );
}

void process_mp3 ( channel_session * _session , uint8 * _data , uint32 _size )
{
    
    audio_session * audio = NULL;

    audio = find_audio_session ( _session );
    if (audio == NULL )
    {
      /*log error*/
      return;
    }
    if ( audio->player == NULL )
    {
        /*log error*/
        return;
    }

    if ( audio->mp3_decoder != NULL )
    { /* process mp3 data */
    	uint32  wave_size = 0;
    	int decoded = 0;
        unsigned short * wave = xmalloc( sizeof(unsigned short) * 5 * 1024 * 1024 );
        
        for (;_size;)
        {
	    int to_decode = (_size > 1024) ? 1024: _size;
	    wave_size = mp3codec_decode( audio->mp3_decoder, _data + decoded, to_decode, wave );
	    _size -= to_decode;
	    decoded += to_decode;

	    play_wave(audio->player, (uint8*)wave, wave_size * sizeof( unsigned short ) );
	    
        }
        xfree( wave );
    }
}

RD_BOOL qaudio_init ( char * _options )
{
	int i;
	for ( i = 0; i < MAX_AUDIO_SESSIONS; ++ i )
	{
		audio_sessions[i] = NULL;
	}

	qaudio_driver_initialize(QDRIVER_ALSA, "default");

	channel_transport_init("VPG_VC", NULL );

	mp3codec_initialize();

	return 0;
}

void qaudio_init_transport()
{
}

void qaudio_free()
{
	mp3codec_uninitialize();
	qaudio_driver_uninitialize();
}

audio_session * create_audio_session  ( channel_session * _session )
{
    int i;
    for ( i = 0; i < MAX_AUDIO_SESSIONS; ++ i )
    {
        if ( audio_sessions[i] == NULL )
        {
            audio_session * session = xmalloc( sizeof( audio_session ) );
            if ( session == NULL )
                break;
            
            session->player = NULL;
            session->recorder = NULL;
            session->transport = _session;
            
            audio_sessions[i] = session;
            
            session->mp3_decoder = NULL;
            session->speex_encoder = NULL;

            return session;
        }
    }
    
    return NULL;
}

void            destroy_audio_session ( audio_session * _session   )
{
    int i;
    for ( i = 0; i < MAX_AUDIO_SESSIONS; ++i )
    {
        if ( audio_sessions[i] == _session ) 
        {
            if ( _session->player != NULL )
            {
                destroy_player( _session->player );
            }
            if ( _session->recorder != NULL )
            {
                destroy_recorder ( _session->recorder );
            }
            
            if (_session->speex_encoder != NULL )
            	codec_destroy(_session->speex_encoder);

            if ( _session->mp3_decoder != NULL )
            	mp3codec_destroy_decoder( _session->mp3_decoder );

            xfree ( _session );
            
            audio_sessions[i] = NULL;            
        }
    }    
}

audio_session * find_audio_session    ( channel_session * _session )
{
    int i;
    for ( i = 0; i < MAX_AUDIO_SESSIONS; ++i )
    {
    	if ( audio_sessions[i] == NULL)
    		continue;
        if ( audio_sessions[i]->transport == _session )
            return audio_sessions[i];
    }
    
    return NULL;
}

void send_speex_data ( audio_session * _session , uint8 * _data, uint32 _size)
{
	uint8 out [ 16834 ]; uint32 compressed_size;
	STREAM response;
	channel_session * transport = _session->transport;

	compressed_size = codec_compress_buffer( _session->speex_encoder, (char*)_data, _size,  out );
	if ( compressed_size > 0)
	{
		channel_session_create_message( transport , &response , 6 * sizeof(uint32) + sizeof(uint64) + compressed_size );

		out_uint32_le( response , WAVE );
		out_uint32_le( response , REQUEST );
		out_uint32_le( response , DATA );
		out_uint64_le( response , _session->recorder->handle );
		out_uint32_le( response , DATA_FORMAT_SPEEX );
		out_uint32_le( response , _size ); /* uncompressed size*/
		out_uint32_le( response , compressed_size  ); /* compressed size*/
		out_uint8p	 ( response , out, compressed_size );

		channel_session_send_message( transport, response );
	}

}

void send_wave_data ( audio_session * _session , uint8 * _data, uint32 _size)
{
	STREAM response;
	channel_session * transport = _session->transport;

	channel_session_create_message( transport , &response , 6 * sizeof(uint32) + sizeof(uint64) + _size);

	out_uint32_le(response, WAVE );
	out_uint32_le(response, REQUEST );
	out_uint32_le(response, DATA);
	out_uint64_le(response, _session->recorder->handle );
	out_uint32_le(response, DATA_FORMAT_WAVE);
	out_uint32_le(response, _size ); /* compressed size*/
	out_uint32_le(response, _size ); /* uncompressed size*/
	out_uint8p(response , _data , _size );

	channel_session_send_message( transport, response );
}


/* entry point for process recording */
void qaudio_process()
{
	int i = 0;
	uint8 data [ 32768 ]; /* 32 kb buffer */

	for ( i = 0; i < MAX_AUDIO_SESSIONS; ++ i )
	{
		int recorded = 0;
		int buffer_size = 1024;

		audio_session * session = audio_sessions[i];
		if ( session == NULL )
			continue;

		if ( session->recorder == NULL )
			continue;

		if ( (g_qus_settings.use_compression) && ( session->speex_encoder != NULL ) )
		{
			buffer_size = codec_get_encoder_frame_bytes (  session->speex_encoder );

			while ( buffer_size < 4096 * 2  )
			{
				buffer_size *= 2;
			}
		}

		do
		{
			int recorded = record_wave( session->recorder, data , buffer_size );
			if ( recorded > 0 )
			{
				//printf("recorded %d bytes\n", recorded );

				if ( (g_qus_settings.use_compression) && (session->speex_encoder != NULL) )
				{
					send_speex_data ( session, data, recorded );
				}
				else
				{
					send_wave_data ( session , data , recorded );
				}
			}
		}
		while ( recorded > 0 );
	}
}

/*------------------------ command line parse routines --------------------------*/

extern RD_BOOL str_startswith( const char *s, const char *prefix);
extern char * next_arg (char *src, char needle);
extern void toupper_str (char *p);
extern char * next_arg(char *src, char needle);

extern int g_qplayer_buffering;

void qaudio_default_parameters()
{
	g_qds_settings.use_plugin = 1;
	g_qds_settings.remote_sound = 0;
	g_qds_settings.channels = 2;
	g_qds_settings.rate = 128;
	g_qds_settings.buffering = 250;
	g_qds_settings.do_not_play = 0;


	g_qus_settings.use_compression = 0;
	g_qus_settings.quality = 5;
	g_qus_settings.complexity = 3;
	g_qus_settings.use_enh = 0;
	
	g_qplayer_buffering = g_qds_settings.buffering;
}

void qaudio_qds_cmd_line_parse ( char * args )
{
	char * pos = args + 1;
	if ( !str_startswith(pos , "on") )
	{
		g_qds_settings.use_plugin = 0;

		printf("Quest downstream redirection disabled\n");
                syslog( LOG_NOTICE, "Quest downstream redirection disabled");
	}
	else
	{
		do
		{
			pos += 2;

			if ( *pos != ':' )
				break;

			++ pos;

			if ( str_startswith(pos,"remote") )
			{
				g_qds_settings.use_plugin = 0;
				g_qds_settings.remote_sound = 1;
			}
			else
			if ( str_startswith(pos, "donotplay"))
			{
				g_qds_settings.use_plugin = 1;
				g_qds_settings.remote_sound = 0;
				g_qds_settings.do_not_play = 1;
			}
			else
			if ( str_startswith(pos,"local") )
			{
				pos += 5;

				if ( *pos != ':')
					break;

				++pos;

				if (str_startswith(pos , "radio"))
				{
					pos += 5;
					g_qds_settings.channels = 2;
					g_qds_settings.rate = 64;
				}
				else
				if (str_startswith(pos, "telephony"))
				{
				    pos += 9;
				    g_qds_settings.channels = 1;
				    g_qds_settings.rate = 64;
				}
				else
				if ( str_startswith(pos,"tp"))
				{
					pos += 2;
					g_qds_settings.channels = 1;
					g_qds_settings.rate = 64;
				}
				else
				if (str_startswith(pos , "cd"))
				{
					pos += 2;
					g_qds_settings.channels = 2;
					g_qds_settings.rate = 256;
				}
				else
				if (str_startswith(pos,"custom"))
				{
					pos += 6;

					if( *pos != '=' )
						break;

					++pos;
					{
						char * rate_pos = pos;

						for (;( 0 <= *pos) && ( *pos<='9' );++pos);

						if (rate_pos == pos)
							break;

						g_qds_settings.rate = atoi(rate_pos);

						if (g_qds_settings.rate <= 64)
							g_qds_settings.rate = 64;
						else
						if (g_qds_settings.rate <= 128)
							g_qds_settings.rate = 128;
						else
						if (g_qds_settings.rate <= 192)
							g_qds_settings.rate = 192;
						else
							g_qds_settings.rate = 256;

						if (*pos == 'm')
							g_qds_settings.channels = 1;

						++pos;
					}
				}
				if ( *pos == ':')
				{
					++ pos;
					if (str_startswith(pos , "buffering"))
					{
						pos += 9;
						if (*pos != '=')
							break;

						++pos;

						g_qds_settings.buffering = atoi(pos) * 10;

						if ( 0 <= g_qds_settings.buffering && g_qds_settings.buffering <= 5000 )
						{
							g_qplayer_buffering =  g_qds_settings.buffering;
							break;
						}
						else
							g_qds_settings.buffering = 250;
					}
				}
			}
		}while (0);

		printf("Quest downstream redirection enabled\n");
		printf("Channels  : %d\n", g_qds_settings.channels );
		printf("Rate      : %d kbps\n", g_qds_settings.rate );
		printf("Buffering : %d in tenth of sec.\n", g_qds_settings.buffering / 10 );
		syslog(LOG_NOTICE, "Quest downstream redirection enabled");
		syslog(LOG_NOTICE, "Channels  : %d", g_qds_settings.channels );
		syslog(LOG_NOTICE, "Rate      : %d kbps", g_qds_settings.rate );
		syslog(LOG_NOTICE, "Buffering : %d in tenth of sec.", g_qds_settings.buffering / 10 );
	}
}
void qaudio_qus_cmd_line_parse ( char * args )
{
	char * pos = args + 1;

	printf("Quest audio upstream redirection enabled\n" );
	syslog( LOG_NOTICE, "Quest audio upstream redirection enabled" );

	if ( str_startswith(pos , "wave"))
	{
		printf("You've choose wave format, no compression\n");
		syslog(LOG_NOTICE, "Wave format, no compression");
	}
	else
	if ( str_startswith(pos, "speex") )
	{
		printf("You've choose speex format, speex-lib compression\n");
		syslog(LOG_NOTICE, "You've choose speex format, speex-lib compression");

		pos += 5;

		g_qus_settings.use_compression = 1;


		if ( *pos != ':')
		{
			printf("default parameters will be used, quality: 5, complexity: 3, no enhancement\n");
			syslog(LOG_NOTICE, "default parameters will be used, quality: 5, complexity: 3, no enhancement");
		}
		else
		{
			++ pos;
			if ( str_startswith(pos,"quality") )
			{
				int quality = 5;
				pos += 7;
				if ( *pos == '=')
				{
					++pos;
					quality = atoi( pos );
					if ( quality == 0 )
					{
					    if (*pos != '0' )
					    {
						printf("invalid quality value: defaulting to 5\n");
						quality = 5;
					    }
					}
				}
				if ( 0<= quality && quality <= 10 )
				{
					g_qus_settings.quality = quality;

					printf("quality: %d\n", quality);
				}
				else
				{
					printf("quality: 5 - default\n");
				}
				pos = next_arg(pos , ':');
				if ( str_startswith(pos , "complexity"))
				{
					int complexity = 3;
					pos += 10;
					if ( *pos == '=' )
					{
						++pos;
						complexity = atoi( pos );
					}

					if ( 1<= complexity && complexity <= 10)
					{
						g_qus_settings.complexity = complexity;
						printf( "complexity: %d\n",complexity);
					}
					else
					{
						printf( "complexity: 3 - default\n");
					}
					pos = next_arg( pos , ':' );
					if (str_startswith(pos, "enh"))
					{
						int enh = 0;
						pos += 3;
						if (*pos=='=')
						{
							++pos;
							enh = str_startswith(pos , "on");
							g_qus_settings.use_enh = enh;
						}

						if ( enh )
						{
							printf("enhancement: on\n");
						}
						else
						{
							printf("enhancement: off\n" );
						}
					}/*enh*/
					else
					{
						printf("enhancement: off - default\n");
					}
				}/*complexity*/
				else
				{
					printf("complexity  : 3   - default\n");
					printf("enhancement : off - default\n");
				}
			}/*quality*/
			else
			{
				printf("quality     : 5   - default\n");
				printf("complexity  : 3   - default\n");
				printf("enhancement : off - default\n");
			}
		}
	}
	else
	{
		printf("Invalid parameters specified\n");
	}
}


