#include "rdesktop.h"
#include "qaudio_libao_drv.h"
#include <ao/ao.h>

typedef struct
{
    ao_device * handle;
} libao_waveout_handle;


static int device_id = 0;

 
int libao_driver_init ( char * _options )
{
	ao_initialize();

	if ( _options == NULL )
	{
		device_id = ao_default_driver_id();
	}
	else
	{
		device_id = ao_driver_id (_options);
	}

	return 0;
}
int libao_driver_free ()
{
	ao_shutdown();
    return 0;
}

int libao_waveout_open  ( LPHWAVEOUT _handle, WAVEFORMATEX * _format )
{
	ao_sample_format format;
	libao_waveout_handle * handle = (libao_waveout_handle*)xmalloc(sizeof( libao_waveout_handle));

	format.bits 	   = _format->wBitsPerSample;
	format.channels    = _format->nChannels;
	format.rate 	   = _format->nSamplesPerSec;
	format.byte_format = AO_FMT_LITTLE;

	handle->handle = ao_open_live( device_id , &format , NULL );

	if ( handle->handle == NULL)
	{
		xfree ( handle );
		return -1;
	}

	*_handle = (HWAVEOUT)handle;

    return 0;
}

int libao_waveout_reset ( HWAVEOUT _handle )
{
    return 0;
}

int libao_waveout_close ( HWAVEOUT _handle )
{
	libao_waveout_handle * handle = (libao_waveout_handle*) _handle;
	if ( handle == NULL )
		return -1;

	ao_close( handle->handle );

	xfree ( handle );

    return 0;
}

int libao_waveout_write ( HWAVEOUT _handle , uint8 * _wave , uint32 _size )
{
	int ret = 0;
	libao_waveout_handle * handle = (libao_waveout_handle*) _handle;
	if ( handle == NULL )
		return 0;

	ret = ao_play( handle->handle ,(char*) _wave, (uint_32)_size );
	if ( ret < 0 )
		return 0;

    return ret;
}
