#include "rdesktop.h"
#include "qaudio_oss_drv.h"

#include <fcntl.h>
#include <sys/ioctl.h>
#include <sys/soundcard.h>

static char DEFAULTDEVICE[] = "/dev/dsp";


typedef struct
{
    int handle;
    int mode;    
} oss_waveout_handle , oss_wavein_handle;

static char * init_options = NULL;
 
int oss_driver_init ( char * _options )
{
    if ( _options == NULL)
    {
    	init_options = xstrdup( DEFAULTDEVICE );
    }
    else
    {
    	init_options = xstrdup( _options );
    }

    return 0;
}
int oss_driver_free ()
{
	xfree ( init_options );
	return 0;
}

int oss_waveout_open  ( LPHWAVEOUT _handle, WAVEFORMATEX * _format )
{
	int format = 0 ,stereo = 0, fragments = 0;
	oss_waveout_handle * handle = NULL;
	handle = (oss_waveout_handle*) xmalloc( sizeof ( oss_waveout_handle ));

	handle->handle = open(init_options ,O_WRONLY );
	if ( handle->handle == -1 )
	{
		xfree( handle );
		return -1;
	}

	ioctl(handle->handle, SNDCTL_DSP_RESET, NULL);
	ioctl(handle->handle, SNDCTL_DSP_SYNC, NULL);

	if (_format->wBitsPerSample == 8)
		format = AFMT_U8;
	else if (_format->wBitsPerSample == 16)
		format = AFMT_S16_LE;

	if (_format->nChannels == 2)
		stereo = 1;

	if ( (ioctl(handle->handle, SNDCTL_DSP_SETFMT, &format) == -1) ||
		 (ioctl(handle->handle, SNDCTL_DSP_STEREO, &stereo) == -1) ||
		 (ioctl(handle->handle, SNDCTL_DSP_SPEED, &_format->nSamplesPerSec) == -1))
	{
		close( handle->handle );
		xfree( handle );
		return -1;
	}

	fragments = (12 << 16) + 12;
	ioctl(handle->handle, SNDCTL_DSP_SETFRAGMENT, &fragments);

	*_handle = (HWAVEOUT)handle;

	return 0;
}

int oss_waveout_reset ( HWAVEOUT _handle )
{
    return 0;
}

int oss_waveout_close ( HWAVEOUT _handle )
{
	oss_waveout_handle * handle = (oss_waveout_handle*)_handle;
	if ( handle == NULL )
		return -1;

	close( handle->handle );

	xfree ( handle );

	return 0;
}

int oss_waveout_write ( HWAVEOUT _handle , uint8 * _wave , uint32 _size )
{
	oss_waveout_handle * handle = (oss_waveout_handle*)_handle;
	if ( handle == NULL )
		return 0;
	else
	{
		int ret = write(handle->handle , _wave, _size );
		if (ret < 0 )
			return 0;
		return ret;
	}
    return 0;
}

/*=----------------------------------------- recording --------------------------- */

int oss_wavein_open   ( LPHWAVEIN _handle, WAVEFORMATEX * _format )
{
	int format = 0 ,stereo = 0, fragments = 0;
	oss_wavein_handle * handle = NULL;

	handle = (oss_wavein_handle *)xmalloc( sizeof( oss_wavein_handle ) );

	handle->handle = open( init_options , O_RDONLY );
	if ( handle ->handle == -1)
	{
		xfree(handle);
		return -1;
	}

	ioctl(handle->handle, SNDCTL_DSP_RESET, NULL);
	ioctl(handle->handle, SNDCTL_DSP_SYNC, NULL);

	if (_format->wBitsPerSample == 8)
		format = AFMT_U8;
	else if (_format->wBitsPerSample == 16)
		format = AFMT_S16_LE;

	if (_format->nChannels == 2)
		stereo = 1;

	if ( (ioctl(handle->handle, SNDCTL_DSP_SETFMT, &format) == -1) ||
		 (ioctl(handle->handle, SNDCTL_DSP_STEREO, &stereo) == -1) ||
		 (ioctl(handle->handle, SNDCTL_DSP_SPEED, &_format->nSamplesPerSec) == -1))
	{
		close( handle->handle );
		xfree( handle );
		return -1;
	}

	fragments = (12 << 16) + 12;
	ioctl(handle->handle, SNDCTL_DSP_SETFRAGMENT, &fragments);

	*_handle = (HWAVEIN) handle;

	return 0;
}

int oss_wavein_close  ( HWAVEIN _handle )
{
	oss_wavein_handle * handle = (oss_wavein_handle*) _handle;
	if ( handle == NULL )
		return -1;

	close ( handle->handle );

	xfree( handle );

	return 0;
}

int oss_wavein_start  ( HWAVEIN _handle )
{
	return 0;
}

int oss_wavein_stop   ( HWAVEIN _handle )
{
	return 0;
}

int oss_wavein_record ( HWAVEIN _handle , uint8 * _data , uint32 _size )
{
	oss_wavein_handle * handle = (oss_wavein_handle*) _handle;
	if ( handle == NULL )
		return 0;
	else
	{
		int ret = read( handle->handle , _data , _size );
		if ( ret < 0 )
			return 0;
		return ret;
	}
	return 0;
}




