/*
 * qrecorder.c
 *
 *  Created on: 05.08.2009
 *      Author: beloborodov
 */

#include "rdesktop.h"
#include "qrecorder.h"
#include "wave_buffer.h"

#include <pthread.h>

#define WIDM_START 57
#define WIDM_STOP  58

typedef struct
{
	pthread_t handle;
	int		  terminate;
	audio_recorder * recorder;
} thread_context;


extern qaudio_wavein_driver wavein_driver;

static int is_correct_audio_driver ( )
{
    if (
       ( wavein_driver.waveInOpen   == NULL ) ||
       ( wavein_driver.waveInClose  == NULL ) ||
       ( wavein_driver.waveInStart  == NULL ) ||
       ( wavein_driver.waveInStop   == NULL ) ||
       ( wavein_driver.waveInRecord == NULL ) )
    return 0;

    if ( ( wavein_driver.driver < 0 ) || ( wavein_driver.driver > 6 ) )
        return 0;

    return 1;
}


void * recording_thread ( void * _context )
{
	thread_context * context = (thread_context*) _context;

	while (!context->terminate)
	{
		audio_recorder * recorder = context->recorder;

		if ( recorder->status == WIDM_STOP )
		{
			usleep( 33 * 1000); /* 33 ms audio chunk, reduce CPU utilization */
		}
		else
		{
			uint8 wave_data[16834]; // 8kb chunk
			int recorded = recorder->driver->waveInRecord(recorder->handle, wave_data, sizeof(wave_data));

			wave_buffer_write((sync_wave_buffer*)recorder->wave_buffer, wave_data, recorded );

		}
	}
	return NULL;
}


audio_recorder * create_recorder (  WAVEFORMATEX * _format )
{
	audio_recorder * recorder = NULL;
	if (_format == NULL)
		return NULL;

	if ( !is_correct_audio_driver())
		return NULL;

	recorder = (audio_recorder*) xmalloc (sizeof ( audio_recorder ) );

	if ( recorder == NULL )
		return NULL;

	recorder->driver = &wavein_driver;

	_format->wBitsPerSample = (_format->nBlockAlign / _format->nChannels) * 8;

	wavein_driver.waveInOpen( &recorder->handle, _format );
	if ( recorder->handle == NULL )
	{
		xfree ( recorder );
		return 0;
	}

	recorder->wave_buffer = wave_buffer_create(MAX_WAVE_BUFFER_SIZE);

	recorder->opened = 1;
	recorder->status = WIDM_STOP;

	thread_context * context = (thread_context*) xmalloc(sizeof(thread_context));

	context->recorder = recorder;
	context->terminate = 0;
	recorder->context = context;

	pthread_create(&context->handle, 0, &recording_thread,context);

	return recorder;
}

void destroy_recorder ( audio_recorder * _recorder )
{
	if ( _recorder == NULL )
		return;

	((thread_context*)_recorder->context)->terminate = 1;
	pthread_join(((thread_context*)_recorder->context)->handle,NULL);

	wave_buffer_clear( (sync_wave_buffer*)_recorder->wave_buffer );
	wave_buffer_destroy((sync_wave_buffer*)_recorder->wave_buffer);

	_recorder->driver->waveInStop( _recorder ->handle );
	_recorder->driver->waveInClose( _recorder->handle );

	xfree (_recorder);
}

void configure_recorder ( audio_recorder * _recorder , uint32 _status )
{
	if (_recorder == NULL )
		return;
	switch ( _status )
	{
	case WIDM_START:
		{
			_recorder->driver->waveInStart(_recorder->handle);
		}break;
	case WIDM_STOP:
		{
			_recorder->driver->waveInStop( _recorder->handle );
		}break;
	}
	_recorder->status = _status;
}

int record_wave ( audio_recorder * _recorder , uint8 * _data , uint32 _size )
{
	if (_recorder == NULL )
		return 0;

	if ( _data == NULL || _size == 0 )
		return 0;

	if ( ((thread_context*)_recorder->context)->terminate )
		return 0;

	if ( wave_buffer_get_writed_bytes( (sync_wave_buffer*) _recorder->wave_buffer) < _size )
	    return 0;

	return wave_buffer_read( (sync_wave_buffer*) _recorder->wave_buffer, _data , _size );
}
