/* -*- c-basic-offset: 8 -*-
   Copyright (C) Matthew Chapman 1999-2008
   Copyright 2007 Pierre Ossman <ossman@cendio.se> for Cendio AB

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/Xproto.h>
#include <X11/Xatom.h>
#include <X11/IntrinsicP.h>
#include <unistd.h>

// _CURSOR_FIX
#include <X11/Xcursor/Xcursor.h>
#include <sys/time.h>
#include <time.h>
#include <errno.h>
#include <strings.h>
#include "rdesktop.h"
#include "xproto.h"

#include "appportal_updater.h"

#define WITH_MM_SUPPORT
#define QTHUMBNAIL_SCREENSHOT_SECS 2

#ifdef WITH_MM_SUPPORT
#include <X11/extensions/Xinerama.h>
#endif

extern int g_width;
extern int g_height;
extern int g_xpos;
extern int g_ypos;
extern int g_pos;
extern RD_BOOL g_sendmotion;
extern RD_BOOL g_fullscreen;
extern RD_BOOL g_span_monitors;
extern RD_BOOL g_grab_keyboard;
extern RD_BOOL g_hide_decorations;
extern char g_title[];
/* Color depth of the RDP session.
   As of RDP 5.1, it may be 8, 15, 16 or 24. */
extern int g_server_depth;
extern int g_win_button_size;

Display *g_display;
Time g_last_gesturetime;
static int g_x_socket;
static Screen *g_screen;
Window g_wnd;

/* SeamlessRDP support */
typedef struct _seamless_group
{
    Window wnd;
    unsigned long id;
    unsigned int refcnt;
} seamless_group;
typedef struct _seamless_window
{
    Window wnd;
    unsigned long id;
    unsigned long behind;
    unsigned long tooltip;  //  variable to manage tooltip windows
    unsigned long parent;   //  parent window
    seamless_group *group;
    int xoffset, yoffset;
    int width, height;
    int state;        /* normal/minimized/maximized. */
    unsigned int desktop;
    struct timeval *position_timer;

    RD_BOOL outstanding_position;
    unsigned int outpos_serial;
    int outpos_xoffset, outpos_yoffset;
    int outpos_width, outpos_height;

    unsigned int icon_size;
    unsigned int icon_offset;
    char icon_buffer[32 * 32 * 4];

    struct _seamless_window *next;
} seamless_window;
static seamless_window *g_seamless_windows = NULL;
static unsigned long g_seamless_focused = 0;
static RD_BOOL g_seamless_started = False;    /* Server end is up and running */
static RD_BOOL g_seamless_active = False;    /* We are currently in seamless mode */
static RD_BOOL g_seamless_hidden = False;    /* Desktop is hidden on server */
static RD_BOOL g_seamless_broken_restack = False;    /* WM does not properly restack */
extern RD_BOOL g_seamless_rdp;

extern uint32 g_embed_wnd;
RD_BOOL g_enable_compose = False;
RD_BOOL g_Unobscured;        /* used for screenblt */
static GC g_gc = NULL;
static GC g_create_bitmap_gc = NULL;
static GC g_create_glyph_gc = NULL;
static XRectangle g_clip_rectangle;
static Visual *g_visual;
/* Color depth of the X11 visual of our window (e.g. 24 for True Color R8G8B visual).
   This may be 32 for R8G8B8 visuals, and then the rest of the bits are undefined
   as far as we're concerned. */
static int g_depth;
/* Bits-per-Pixel of the pixmaps we'll be using to draw on our window.
   This may be larger than g_depth, in which case some of the bits would
   be kept solely for alignment (e.g. 32bpp pixmaps on a 24bpp visual). */
static int g_bpp;
static XIM g_IM;
static XIC g_IC;
static XModifierKeymap *g_mod_map;
/* Maps logical (xmodmap -pp) pointing device buttons (0-based) back
   to physical (1-based) indices. */
static unsigned char g_pointer_log_to_phys_map[32];
static Cursor g_current_cursor;
static RD_HCURSOR g_null_cursor = NULL;
static Atom g_protocol_atom, g_kill_atom;
extern Atom g_net_wm_state_atom;
extern Atom g_net_wm_desktop_atom;
static RD_BOOL g_focused;
static RD_BOOL g_mouse_in_wnd;
/* Indicates that:
   1) visual has 15, 16 or 24 depth and the same color channel masks
      as its RDP equivalent (implies X server is LE),
   2) host is LE
   This will trigger an optimization whose real value is questionable.
*/
static RD_BOOL g_compatible_arch;
/* Indicates whether RDP's bitmaps and our XImages have the same
   binary format. If so, we can avoid an expensive translation.
   Note that this can be true when g_compatible_arch is false,
   e.g.:

     RDP(LE) <-> host(BE) <-> X-Server(LE)

   ('host' is the machine running rdesktop; the host simply memcpy's
    so its endianess doesn't matter)
 */
static RD_BOOL g_no_translate_image = False;

/* endianness */
static RD_BOOL g_host_be;
static RD_BOOL g_xserver_be;
static int g_red_shift_r, g_blue_shift_r, g_green_shift_r;
static int g_red_shift_l, g_blue_shift_l, g_green_shift_l;

/* software backing store */
extern RD_BOOL g_ownbackstore;
static Pixmap g_backstore = 0;
static Pixmap g_backstore2 = 0;


/* Moving in single app mode */
static RD_BOOL g_moving_wnd;
static int g_move_x_offset = 0;
static int g_move_y_offset = 0;
static RD_BOOL g_using_full_workarea = False;

#ifdef WITH_RDPSND
extern RD_BOOL g_rdpsnd;
#endif

// AppPortal updater
extern int au_dump_screenshot( XImage * );
void xwin_dump_screen( void );

#include <X11/extensions/shape.h>
#define QWIN_MIN_WIDTH 0x80
#define QWIN_MIN_HEIGHT 0x80
#define QWIN_SIDE_LEFT 1
#define QWIN_SIDE_RIGHT 2
#define QWIN_SIDE_BUTTOM 6
#define QWIN_SIDE_UP 3
#define QWIN_CORNER_UP_LEFT 4
#define QWIN_CORNER_UP_RIGHT 5
#define QWIN_CORNER_BUTTOM_LEFT 7
#define QWIN_CORNER_BUTTOM_RIGHT 8
#define Q_MESSAGE_LEN 0x80
#define Q_ALLOWED_ACTIONS_SIZE 6
Window gq_fake_wnd;
static int gq_window_manager_process;
extern unsigned long  gq_is_AS;
extern Atom g_net_wm_actions_atom;
extern Atom g_allowed_actions[Q_ALLOWED_ACTIONS_SIZE];
/* Moving in seamless app mode */
unsigned long  g_moving_wnd_seamless_start_move;
unsigned long g_moving_wnd_seamless_start_size;
unsigned long g_moving_wnd_seamless;
int g_move_x_offset_seamless = 0;
int g_move_y_offset_seamless = 0;
void qprocess_moving_wnd_seamless (XEvent xevent);
void qprocess_moving_wnd_seamless_size (XEvent xevent);
void qprocess_moving_wnd_seamless_move (XEvent xevent);
void qprocess_send_state (seamless_window * sw);
void qprocess_send_all_to_desktop(seamless_window *sw);
void qseamless_send_message_SY_CLOSE(unsigned long id);
void qseamless_send_message_WA_CLICKACTIVE(unsigned long id);
void qseamless_send_message_WA_INACTIVE(unsigned long id);
void qseamless_draw_fake( int width, int height);
unsigned int pnts_send_message(const char *message);
unsigned long gq_support_skype_id = 0;
extern char g_response_message[256];

/* MWM decorations */
#define MWM_HINTS_DECORATIONS   (1L << 1)
#define PROP_MOTIF_WM_HINTS_ELEMENTS    5
typedef struct
{
    unsigned long flags;
    unsigned long functions;
    unsigned long decorations;
    long inputMode;
    unsigned long status;
}
PropMotifWmHints;

typedef struct
{
    uint32 red;
    uint32 green;
    uint32 blue;
}
PixelColour;

#define ON_ALL_SEAMLESS_WINDOWS(func, args) \
        do { \
                seamless_window *sw; \
                XRectangle rect; \
        if (!g_seamless_windows) break; \
                for (sw = g_seamless_windows; sw; sw = sw->next) { \
                    rect.x = g_clip_rectangle.x - sw->xoffset; \
                    rect.y = g_clip_rectangle.y - sw->yoffset; \
                    rect.width = g_clip_rectangle.width; \
                    rect.height = g_clip_rectangle.height; \
                    XSetClipRectangles(g_display, g_gc, 0, 0, &rect, 1, YXBanded); \
                    func args; \
                } \
                XSetClipRectangles(g_display, g_gc, 0, 0, &g_clip_rectangle, 1, YXBanded); \
        } while (0)

static void
seamless_XFillPolygon(Drawable d, XPoint * points, int npoints, int xoffset, int yoffset)
{
    points[0].x -= xoffset;
    points[0].y -= yoffset;
    XFillPolygon(g_display, d, g_gc, points, npoints, Complex, CoordModePrevious);
    points[0].x += xoffset;
    points[0].y += yoffset;
}

static void
seamless_XDrawLines(Drawable d, XPoint * points, int npoints, int xoffset, int yoffset)
{
    points[0].x -= xoffset;
    points[0].y -= yoffset;
    XDrawLines(g_display, d, g_gc, points, npoints, CoordModePrevious);
    points[0].x += xoffset;
    points[0].y += yoffset;
}

#define FILL_RECTANGLE(x,y,cx,cy)\
{ \
    XFillRectangle(g_display, g_wnd, g_gc, x, y, cx, cy); \
        ON_ALL_SEAMLESS_WINDOWS(XFillRectangle, (g_display, sw->wnd, g_gc, x-sw->xoffset, y-sw->yoffset, cx, cy)); \
    if (g_ownbackstore) \
        XFillRectangle(g_display, g_backstore, g_gc, x, y, cx, cy); \
}

#define FILL_RECTANGLE_BACKSTORE(x,y,cx,cy)\
{ \
    XFillRectangle(g_display, g_ownbackstore ? g_backstore : g_wnd, g_gc, x, y, cx, cy); \
}

#define FILL_POLYGON(p,np)\
{ \
    XFillPolygon(g_display, g_wnd, g_gc, p, np, Complex, CoordModePrevious); \
    if (g_ownbackstore) \
        XFillPolygon(g_display, g_backstore, g_gc, p, np, Complex, CoordModePrevious); \
    ON_ALL_SEAMLESS_WINDOWS(seamless_XFillPolygon, (sw->wnd, p, np, sw->xoffset, sw->yoffset)); \
}

#define DRAW_ELLIPSE(x,y,cx,cy,m)\
{ \
    switch (m) \
    { \
        case 0:    /* Outline */ \
            XDrawArc(g_display, g_wnd, g_gc, x, y, cx, cy, 0, 360*64); \
                        ON_ALL_SEAMLESS_WINDOWS(XDrawArc, (g_display, sw->wnd, g_gc, x-sw->xoffset, y-sw->yoffset, cx, cy, 0, 360*64)); \
            if (g_ownbackstore) \
                XDrawArc(g_display, g_backstore, g_gc, x, y, cx, cy, 0, 360*64); \
            break; \
        case 1: /* Filled */ \
            XFillArc(g_display, g_wnd, g_gc, x, y, cx, cy, 0, 360*64); \
            ON_ALL_SEAMLESS_WINDOWS(XFillArc, (g_display, sw->wnd, g_gc, x-sw->xoffset, y-sw->yoffset, cx, cy, 0, 360*64)); \
            if (g_ownbackstore) \
                XFillArc(g_display, g_backstore, g_gc, x, y, cx, cy, 0, 360*64); \
            break; \
    } \
}

/* colour maps */
extern RD_BOOL g_owncolmap;
static Colormap g_xcolmap;
static uint32 *g_colmap = NULL;

#define TRANSLATE(col)        ( g_server_depth != 8 ? translate_colour(col) : g_owncolmap ? col : g_colmap[col] )
#define SET_FOREGROUND(col)    XSetForeground(g_display, g_gc, TRANSLATE(col));
#define SET_BACKGROUND(col)    XSetBackground(g_display, g_gc, TRANSLATE(col));

static int rop2_map[] =
{
    GXclear,        /* 0 */
    GXnor,            /* DPon */
    GXandInverted,        /* DPna */
    GXcopyInverted,        /* Pn */
    GXandReverse,        /* PDna */
    GXinvert,        /* Dn */
    GXxor,            /* DPx */
    GXnand,            /* DPan */
    GXand,            /* DPa */
    GXequiv,        /* DPxn */
    GXnoop,            /* D */
    GXorInverted,        /* DPno */
    GXcopy,            /* P */
    GXorReverse,        /* PDno */
    GXor,            /* DPo */
    GXset            /* 1 */
};

#define SET_FUNCTION(rop2)    { if (rop2 != ROP2_COPY) XSetFunction(g_display, g_gc, rop2_map[rop2]); }
#define RESET_FUNCTION(rop2)    { if (rop2 != ROP2_COPY) XSetFunction(g_display, g_gc, GXcopy); }

static seamless_window *
sw_get_window_by_id(unsigned long id)
{
    seamless_window *sw;
    for (sw = g_seamless_windows; sw; sw = sw->next)
    {
        if (sw->id == id)
            return sw;
    }
    return NULL;
}


static seamless_window *
sw_get_window_by_wnd(Window wnd)
{
    seamless_window *sw;
    for (sw = g_seamless_windows; sw; sw = sw->next)
    {
        if (sw->wnd == wnd)
            return sw;
    }
    return NULL;
}


static void
sw_remove_window(seamless_window * win)
{
    seamless_window *sw, **prevnext = &g_seamless_windows;
    for (sw = g_seamless_windows; sw; sw = sw->next)
    {
        if (sw == win)
        {
            *prevnext = sw->next;
            sw->group->refcnt--;
            if (sw->group->refcnt == 0)
            {
                XDestroyWindow(g_display, sw->group->wnd);
                xfree(sw->group);
            }
            xfree(sw->position_timer);
            xfree(sw);
            return;
        }
        prevnext = &sw->next;
    }
    return;
}


/* Move all windows except wnd to new desktop */
static void
sw_all_to_desktop(Window wnd, unsigned int desktop)
{
    seamless_window *sw;
    for (sw = g_seamless_windows; sw; sw = sw->next)
    {
        if (sw->wnd == wnd)
            continue;
        if (sw->desktop != desktop)
        {
            ewmh_move_to_desktop(sw->wnd, desktop);
            sw->desktop = desktop;
        }
    }
}


/* Send our position */
static void
sw_update_position(seamless_window * sw)
{
    XWindowAttributes wa;
    int x, y;
    Window child_return;
    unsigned int serial;

    XGetWindowAttributes(g_display, sw->wnd, &wa);
    XTranslateCoordinates(g_display, sw->wnd, wa.root,
                          -wa.border_width, -wa.border_width, &x, &y, &child_return);

    serial = seamless_send_position(sw->id, x, y, wa.width, wa.height, 0);

    sw->outstanding_position = True;
    sw->outpos_serial = serial;

    sw->outpos_xoffset = x;
    sw->outpos_yoffset = y;
    sw->outpos_width = wa.width;
    sw->outpos_height = wa.height;
}


/* Check if it's time to send our position */
static void
sw_check_timers()
{
    seamless_window *sw;
    struct timeval now;

    gettimeofday(&now, NULL);
    for (sw = g_seamless_windows; sw; sw = sw->next)
    {
        if (timerisset(sw->position_timer) && timercmp(sw->position_timer, &now, <))
        {
            timerclear(sw->position_timer);
            sw_update_position(sw);
        }
    }
}


static void
sw_restack_window(seamless_window * sw, unsigned long behind)
{
    seamless_window *sw_above;

    /* Remove window from stack */
    for (sw_above = g_seamless_windows; sw_above; sw_above = sw_above->next)
    {
        if (sw_above->behind == sw->id)
            break;
    }

    if (sw_above)
        sw_above->behind = sw->behind;

    /* And then add it at the new position */
    for (sw_above = g_seamless_windows; sw_above; sw_above = sw_above->next)
    {
        if (sw_above->behind == behind)
            break;
    }

    if (sw_above)
        sw_above->behind = sw->id;

    sw->behind = behind;
}


static void
sw_handle_restack(seamless_window * sw)
{
    Status status;
    Window root, parent, *children;
    unsigned int nchildren, i;
    seamless_window *sw_below;

    status = XQueryTree(g_display, RootWindowOfScreen(g_screen),
                        &root, &parent, &children, &nchildren);
    if (!status || !nchildren)
        return;

    sw_below = NULL;

    i = 0;
    while (children[i] != sw->wnd)
    {
        i++;
        if (i >= nchildren)
            goto end;
    }

    for (i++; i < nchildren; i++)
    {
        sw_below = sw_get_window_by_wnd(children[i]);
        if (sw_below)
            break;
    }

    if (!sw_below && !sw->behind)
        goto end;
    if (sw_below && (sw_below->id == sw->behind))
        goto end;

    if (sw_below)
    {
        seamless_send_zchange(sw->id, sw_below->id, 0);
        sw_restack_window(sw, sw_below->id);
    }
    else
    {
        seamless_send_zchange(sw->id, 0, 0);
        sw_restack_window(sw, 0);
    }

end:
    XFree(children);
}


static seamless_group *
sw_find_group(unsigned long id, RD_BOOL dont_create)
{
    seamless_window *sw;
    seamless_group *sg;
    XSetWindowAttributes attribs;

    for (sw = g_seamless_windows; sw; sw = sw->next)
    {
        if (sw->group->id == id)
            return sw->group;
    }

    if (dont_create)
        return NULL;

    sg = xmalloc(sizeof(seamless_group));

    sg->wnd =
        XCreateWindow(g_display, RootWindowOfScreen(g_screen), -1, -1, 1, 1, 0,
                      CopyFromParent, CopyFromParent, CopyFromParent, 0, &attribs);

    sg->id = id;
    sg->refcnt = 0;

    return sg;
}


static void
mwm_hide_decorations(Window wnd)
{
    PropMotifWmHints motif_hints;
    Atom hintsatom;

    /* setup the property */
    motif_hints.flags = MWM_HINTS_DECORATIONS;
    motif_hints.decorations = 0;

    /* get the atom for the property */
    hintsatom = XInternAtom(g_display, "_MOTIF_WM_HINTS", False);
    if (!hintsatom)
    {
        warning("Failed to get atom _MOTIF_WM_HINTS: probably your window manager does not support MWM hints\n");
        return;
    }

    XChangeProperty(g_display, wnd, hintsatom, hintsatom, 32, PropModeReplace,
                    (unsigned char *) &motif_hints, PROP_MOTIF_WM_HINTS_ELEMENTS);

}

typedef struct _sw_configurenotify_context
{
    Window window;
    unsigned long serial;
} sw_configurenotify_context;

/* Predicate procedure for sw_wait_configurenotify */
static Bool
sw_configurenotify_p(Display * display, XEvent * xevent, XPointer arg)
{
    sw_configurenotify_context *context = (sw_configurenotify_context *) arg;
    if (xevent->xany.type == ConfigureNotify
            && xevent->xconfigure.window == context->window
            && xevent->xany.serial >= context->serial)
        return True;

    return False;
}

/* Wait for a ConfigureNotify, with a equal or larger serial, on the
   specified window. The event will be removed from the queue. We
   could use XMaskEvent(StructureNotifyMask), but we would then risk
   throwing away crucial events like DestroyNotify.

   After a ConfigureWindow, according to ICCCM section 4.1.5, we
   should recieve a ConfigureNotify, either a real or synthetic
   one. This indicates that the configure has been "completed".
   However, some WMs such as several versions of Metacity fails to
   send synthetic events. See bug
   http://bugzilla.gnome.org/show_bug.cgi?id=322840. We need to use a
   timeout to avoid a hang. Tk uses the same approach. */
static void
sw_wait_configurenotify(Window wnd, unsigned long serial)
{
    XEvent xevent;
    sw_configurenotify_context context;
    struct timeval now;
    struct timeval nextsecond;
    RD_BOOL got = False;

    context.window = wnd;
    context.serial = serial;

    gettimeofday(&nextsecond, NULL);
    nextsecond.tv_sec += 1;

    do
    {
        if (XCheckIfEvent(g_display, &xevent, sw_configurenotify_p, (XPointer) & context))
        {
            got = True;
            break;
        }
        usleep(1000000);
        gettimeofday(&now, NULL);
    }
    while (timercmp(&now, &nextsecond, <));

    if (!got)
    {
        warning("Broken Window Manager: Timeout while waiting for ConfigureNotify\n");
    }
}

/* Get the toplevel window, in case of reparenting */
static Window
sw_get_toplevel(Window wnd)
{
    Window root, parent;
    Window *child_list;
    unsigned int num_children;

    while (1)
    {
        XQueryTree(g_display, wnd, &root, &parent, &child_list, &num_children);
        if (root == parent)
        {
            break;
        }
        else if (!parent)
        {
            warning("Internal error: sw_get_toplevel called with root window\n");
        }

        wnd = parent;
    }

    return wnd;
}


/* Check if wnd is already behind a window wrt stacking order */
static RD_BOOL
sw_window_is_behind(Window wnd, Window behind)
{
    Window dummy1, dummy2;
    Window *child_list;
    unsigned int num_children;
    unsigned int i;
    RD_BOOL found_behind = False;
    RD_BOOL found_wnd = False;

    wnd = sw_get_toplevel(wnd);
    behind = sw_get_toplevel(behind);

    XQueryTree(g_display, RootWindowOfScreen(g_screen), &dummy1, &dummy2, &child_list,
               &num_children);

    for (i = num_children - 1; i >= 0; i--)
    {
        if (child_list[i] == behind)
        {
            found_behind = True;
        }
        else if (child_list[i] == wnd)
        {
            found_wnd = True;
            break;
        }
    }

    if (child_list)
        XFree(child_list);

    if (!found_wnd)
    {
        warning("sw_window_is_behind: Unable to find window 0x%lx\n", wnd);

        if (!found_behind)
        {
            warning("sw_window_is_behind: Unable to find behind window 0x%lx\n",
                    behind);
        }
    }

    return found_behind;
}


/* Test if the window manager correctly handles window restacking. In
   particular, we are testing if it's possible to place a window
   between two other windows. Many WMs such as Metacity can only stack
   windows on the top or bottom. The window creation should mostly
   match ui_seamless_create_window. */
static void
seamless_restack_test()
{
    /* The goal is to have the middle window between top and
       bottom.  The middle window is initially at the top,
       though. */
    Window wnds[3];        /* top, middle and bottom */
    int i;
    XEvent xevent;
    XWindowChanges values;
    unsigned long restack_serial;

    for (i = 0; i < 3; i++)
    {
        char name[64];
        wnds[i] =
            XCreateSimpleWindow(g_display, RootWindowOfScreen(g_screen), 0, 0, 20, 20,
                                0, 0, 0);
        snprintf(name, sizeof(name), "SeamlessRDP restack test - window %d", i);
        XStoreName(g_display, wnds[i], name);
        ewmh_set_wm_name(wnds[i], name);

        /* Hide decorations. Often this means that no
           reparenting will be done, which makes the restack
           easier. Besides, we want to mimic our other
           seamless windows as much as possible. We must still
           handle the case with reparenting, though. */
        mwm_hide_decorations(wnds[i]);

        /* Prevent windows from appearing in task bar */
        XSetTransientForHint(g_display, wnds[i], RootWindowOfScreen(g_screen));
        ewmh_set_window_popup(wnds[i]);

        /* We need to catch MapNotify/ConfigureNotify */
        XSelectInput(g_display, wnds[i], StructureNotifyMask);
    }

    /* Map Windows. Currently, we assume that XMapRaised places
       the window on the top of the stack. Should be fairly safe;
       the window is configured before it's mapped. */
    XMapRaised(g_display, wnds[2]);    /* bottom */
    do
    {
        XWindowEvent(g_display, wnds[2], StructureNotifyMask, &xevent);
    }
    while (xevent.type != MapNotify);
    XMapRaised(g_display, wnds[0]);    /* top */
    do
    {
        XWindowEvent(g_display, wnds[0], StructureNotifyMask, &xevent);
    }
    while (xevent.type != MapNotify);
    XMapRaised(g_display, wnds[1]);    /* middle */
    do
    {
        XWindowEvent(g_display, wnds[1], StructureNotifyMask, &xevent);
    }
    while (xevent.type != MapNotify);

    /* The stacking order should now be 1 - 0 - 2 */
    if (!sw_window_is_behind(wnds[0], wnds[1]) || !sw_window_is_behind(wnds[2], wnds[1]))
    {
        /* Ok, technically a WM is allowed to stack windows arbitrarily, but... */
        warning("Broken Window Manager: Unable to test window restacking\n");
        g_seamless_broken_restack = True;
        for (i = 0; i < 3; i++)
            XDestroyWindow(g_display, wnds[i]);
        return;
    }

    /* Restack, using XReconfigureWMWindow, which should correctly
       handle reparented windows as well as nonreparenting WMs. */
    values.stack_mode = Below;
    values.sibling = wnds[0];
    restack_serial = XNextRequest(g_display);
    XReconfigureWMWindow(g_display, wnds[1], DefaultScreen(g_display), CWStackMode | CWSibling,
                         &values);
    sw_wait_configurenotify(wnds[1], restack_serial);

    /* Now verify that middle is behind top but not behind
       bottom */
    if (!sw_window_is_behind(wnds[1], wnds[0]))
    {
        warning("Broken Window Manager: doesn't handle restack (restack request was ignored)\n");
        g_seamless_broken_restack = True;
    }
    else if (sw_window_is_behind(wnds[1], wnds[2]))
    {
        warning("Broken Window Manager: doesn't handle restack (window was moved to bottom)\n");
        g_seamless_broken_restack = True;
    }

    /* Destroy windows */
    for (i = 0; i < 3; i++)
        XDestroyWindow(g_display, wnds[i]);
}

#define SPLITCOLOUR15(colour, rv) \
{ \
    rv.red = ((colour >> 7) & 0xf8) | ((colour >> 12) & 0x7); \
    rv.green = ((colour >> 2) & 0xf8) | ((colour >> 8) & 0x7); \
    rv.blue = ((colour << 3) & 0xf8) | ((colour >> 2) & 0x7); \
}

#define SPLITCOLOUR16(colour, rv) \
{ \
    rv.red = ((colour >> 8) & 0xf8) | ((colour >> 13) & 0x7); \
    rv.green = ((colour >> 3) & 0xfc) | ((colour >> 9) & 0x3); \
    rv.blue = ((colour << 3) & 0xf8) | ((colour >> 2) & 0x7); \
} \

#define SPLITCOLOUR24(colour, rv) \
{ \
    rv.blue = (colour & 0xff0000) >> 16; \
    rv.green = (colour & 0x00ff00) >> 8; \
    rv.red = (colour & 0x0000ff); \
}

#define MAKECOLOUR(pc) \
    ((pc.red >> g_red_shift_r) << g_red_shift_l) \
        | ((pc.green >> g_green_shift_r) << g_green_shift_l) \
        | ((pc.blue >> g_blue_shift_r) << g_blue_shift_l) \

#define BSWAP16(x) { x = (((x & 0xff) << 8) | (x >> 8)); }
#define BSWAP24(x) { x = (((x & 0xff) << 16) | (x >> 16) | (x & 0xff00)); }
#define BSWAP32(x) { x = (((x & 0xff00ff) << 8) | ((x >> 8) & 0xff00ff)); \
            x = (x << 16) | (x >> 16); }

/* The following macros output the same octet sequences
   on both BE and LE hosts: */

#define BOUT16(o, x) { *(o++) = x >> 8; *(o++) = x; }
#define BOUT24(o, x) { *(o++) = x >> 16; *(o++) = x >> 8; *(o++) = x; }
#define BOUT32(o, x) { *(o++) = x >> 24; *(o++) = x >> 16; *(o++) = x >> 8; *(o++) = x; }
#define LOUT16(o, x) { *(o++) = x; *(o++) = x >> 8; }
#define LOUT24(o, x) { *(o++) = x; *(o++) = x >> 8; *(o++) = x >> 16; }
#define LOUT32(o, x) { *(o++) = x; *(o++) = x >> 8; *(o++) = x >> 16; *(o++) = x >> 24; }

static uint32
translate_colour(uint32 colour)
{
    PixelColour pc;
    switch (g_server_depth)
    {
    case 15:
        SPLITCOLOUR15(colour, pc);
        break;
    case 16:
        SPLITCOLOUR16(colour, pc);
        break;
    case 24:
    case 32:
        SPLITCOLOUR24(colour, pc);
        break;
    default:
        /* Avoid warning */
        pc.red = 0;
        pc.green = 0;
        pc.blue = 0;
        break;
    }
    return MAKECOLOUR(pc);
}

/* indent is confused by UNROLL8 */
/* *INDENT-OFF* */

/* repeat and unroll, similar to bitmap.c */
/* potentialy any of the following translate */
/* functions can use repeat but just doing */
/* the most common ones */

#define UNROLL8(stm) { stm stm stm stm stm stm stm stm }
/* 2 byte output repeat */
#define REPEAT2(stm) \
{ \
    while (out <= end - 8 * 2) \
        UNROLL8(stm) \
    while (out < end) \
        { stm } \
}
/* 3 byte output repeat */
#define REPEAT3(stm) \
{ \
    while (out <= end - 8 * 3) \
        UNROLL8(stm) \
    while (out < end) \
        { stm } \
}
/* 4 byte output repeat */
#define REPEAT4(stm) \
{ \
    while (out <= end - 8 * 4) \
        UNROLL8(stm) \
    while (out < end) \
        { stm } \
}
/* *INDENT-ON* */

static void
translate8to8(const uint8 * data, uint8 * out, uint8 * end)
{
    while (out < end)
        *(out++) = (uint8) g_colmap[*(data++)];
}

static void
translate8to16(const uint8 * data, uint8 * out, uint8 * end)
{
    uint16 value;

    if (g_compatible_arch)
    {
        /* *INDENT-OFF* */
        REPEAT2
        (
            *((uint16 *) out) = g_colmap[*(data++)];
            out += 2;
        )
        /* *INDENT-ON* */
    }
    else if (g_xserver_be)
    {
        while (out < end)
        {
            value = (uint16) g_colmap[*(data++)];
            BOUT16(out, value);
        }
    }
    else
    {
        while (out < end)
        {
            value = (uint16) g_colmap[*(data++)];
            LOUT16(out, value);
        }
    }
}

/* little endian - conversion happens when colourmap is built */
static void
translate8to24(const uint8 * data, uint8 * out, uint8 * end)
{
    uint32 value;

    if (g_compatible_arch)
    {
        while (out < end)
        {
            value = g_colmap[*(data++)];
            BOUT24(out, value);
        }
    }
    else
    {
        while (out < end)
        {
            value = g_colmap[*(data++)];
            LOUT24(out, value);
        }
    }
}

static void
translate8to32(const uint8 * data, uint8 * out, uint8 * end)
{
    uint32 value;

    if (g_compatible_arch)
    {
        /* *INDENT-OFF* */
        REPEAT4
        (
            *((uint32 *) out) = g_colmap[*(data++)];
            out += 4;
        )
        /* *INDENT-ON* */
    }
    else if (g_xserver_be)
    {
        while (out < end)
        {
            value = g_colmap[*(data++)];
            BOUT32(out, value);
        }
    }
    else
    {
        while (out < end)
        {
            value = g_colmap[*(data++)];
            LOUT32(out, value);
        }
    }
}

static void
translate15to16(const uint16 * data, uint8 * out, uint8 * end)
{
    uint16 pixel;
    uint16 value;
    PixelColour pc;

    if (g_xserver_be)
    {
        while (out < end)
        {
            pixel = *(data++);
            if (g_host_be)
            {
                BSWAP16(pixel);
            }
            SPLITCOLOUR15(pixel, pc);
            value = MAKECOLOUR(pc);
            BOUT16(out, value);
        }
    }
    else
    {
        while (out < end)
        {
            pixel = *(data++);
            if (g_host_be)
            {
                BSWAP16(pixel);
            }
            SPLITCOLOUR15(pixel, pc);
            value = MAKECOLOUR(pc);
            LOUT16(out, value);
        }
    }
}

static void
translate15to24(const uint16 * data, uint8 * out, uint8 * end)
{
    uint32 value;
    uint16 pixel;
    PixelColour pc;

    if (g_compatible_arch)
    {
        /* *INDENT-OFF* */
        REPEAT3
        (
            pixel = *(data++);
            SPLITCOLOUR15(pixel, pc);
            *(out++) = pc.blue;
            *(out++) = pc.green;
            *(out++) = pc.red;
        )
        /* *INDENT-ON* */
    }
    else if (g_xserver_be)
    {
        while (out < end)
        {
            pixel = *(data++);
            if (g_host_be)
            {
                BSWAP16(pixel);
            }
            SPLITCOLOUR15(pixel, pc);
            value = MAKECOLOUR(pc);
            BOUT24(out, value);
        }
    }
    else
    {
        while (out < end)
        {
            pixel = *(data++);
            if (g_host_be)
            {
                BSWAP16(pixel);
            }
            SPLITCOLOUR15(pixel, pc);
            value = MAKECOLOUR(pc);
            LOUT24(out, value);
        }
    }
}

static void
translate15to32(const uint16 * data, uint8 * out, uint8 * end)
{
    uint16 pixel;
    uint32 value;
    PixelColour pc;

    if (g_compatible_arch)
    {
        /* *INDENT-OFF* */
        REPEAT4
        (
            pixel = *(data++);
            SPLITCOLOUR15(pixel, pc);
            *(out++) = pc.blue;
            *(out++) = pc.green;
            *(out++) = pc.red;
            *(out++) = 0;
        )
        /* *INDENT-ON* */
    }
    else if (g_xserver_be)
    {
        while (out < end)
        {
            pixel = *(data++);
            if (g_host_be)
            {
                BSWAP16(pixel);
            }
            SPLITCOLOUR15(pixel, pc);
            value = MAKECOLOUR(pc);
            BOUT32(out, value);
        }
    }
    else
    {
        while (out < end)
        {
            pixel = *(data++);
            if (g_host_be)
            {
                BSWAP16(pixel);
            }
            SPLITCOLOUR15(pixel, pc);
            value = MAKECOLOUR(pc);
            LOUT32(out, value);
        }
    }
}

static void
translate16to16(const uint16 * data, uint8 * out, uint8 * end)
{
    uint16 pixel;
    uint16 value;
    PixelColour pc;

    if (g_xserver_be)
    {
        if (g_host_be)
        {
            while (out < end)
            {
                pixel = *(data++);
                BSWAP16(pixel);
                SPLITCOLOUR16(pixel, pc);
                value = MAKECOLOUR(pc);
                BOUT16(out, value);
            }
        }
        else
        {
            while (out < end)
            {
                pixel = *(data++);
                SPLITCOLOUR16(pixel, pc);
                value = MAKECOLOUR(pc);
                BOUT16(out, value);
            }
        }
    }
    else
    {
        if (g_host_be)
        {
            while (out < end)
            {
                pixel = *(data++);
                BSWAP16(pixel);
                SPLITCOLOUR16(pixel, pc);
                value = MAKECOLOUR(pc);
                LOUT16(out, value);
            }
        }
        else
        {
            while (out < end)
            {
                pixel = *(data++);
                SPLITCOLOUR16(pixel, pc);
                value = MAKECOLOUR(pc);
                LOUT16(out, value);
            }
        }
    }
}

static void
translate16to24(const uint16 * data, uint8 * out, uint8 * end)
{
    uint32 value;
    uint16 pixel;
    PixelColour pc;

    if (g_compatible_arch)
    {
        /* *INDENT-OFF* */
        REPEAT3
        (
            pixel = *(data++);
            SPLITCOLOUR16(pixel, pc);
            *(out++) = pc.blue;
            *(out++) = pc.green;
            *(out++) = pc.red;
        )
        /* *INDENT-ON* */
    }
    else if (g_xserver_be)
    {
        if (g_host_be)
        {
            while (out < end)
            {
                pixel = *(data++);
                BSWAP16(pixel);
                SPLITCOLOUR16(pixel, pc);
                value = MAKECOLOUR(pc);
                BOUT24(out, value);
            }
        }
        else
        {
            while (out < end)
            {
                pixel = *(data++);
                SPLITCOLOUR16(pixel, pc);
                value = MAKECOLOUR(pc);
                BOUT24(out, value);
            }
        }
    }
    else
    {
        if (g_host_be)
        {
            while (out < end)
            {
                pixel = *(data++);
                BSWAP16(pixel);
                SPLITCOLOUR16(pixel, pc);
                value = MAKECOLOUR(pc);
                LOUT24(out, value);
            }
        }
        else
        {
            while (out < end)
            {
                pixel = *(data++);
                SPLITCOLOUR16(pixel, pc);
                value = MAKECOLOUR(pc);
                LOUT24(out, value);
            }
        }
    }
}

static void
translate16to32(const uint16 * data, uint8 * out, uint8 * end)
{
    uint16 pixel;
    uint32 value;
    PixelColour pc;

    if (g_compatible_arch)
    {
        /* *INDENT-OFF* */
        REPEAT4
        (
            pixel = *(data++);
            SPLITCOLOUR16(pixel, pc);
            *(out++) = pc.blue;
            *(out++) = pc.green;
            *(out++) = pc.red;
            *(out++) = 0;
        )
        /* *INDENT-ON* */
    }
    else if (g_xserver_be)
    {
        if (g_host_be)
        {
            while (out < end)
            {
                pixel = *(data++);
                BSWAP16(pixel);
                SPLITCOLOUR16(pixel, pc);
                value = MAKECOLOUR(pc);
                BOUT32(out, value);
            }
        }
        else
        {
            while (out < end)
            {
                pixel = *(data++);
                SPLITCOLOUR16(pixel, pc);
                value = MAKECOLOUR(pc);
                BOUT32(out, value);
            }
        }
    }
    else
    {
        if (g_host_be)
        {
            while (out < end)
            {
                pixel = *(data++);
                BSWAP16(pixel);
                SPLITCOLOUR16(pixel, pc);
                value = MAKECOLOUR(pc);
                LOUT32(out, value);
            }
        }
        else
        {
            while (out < end)
            {
                pixel = *(data++);
                SPLITCOLOUR16(pixel, pc);
                value = MAKECOLOUR(pc);
                LOUT32(out, value);
            }
        }
    }
}

static void
translate24to16(const uint8 * data, uint8 * out, uint8 * end)
{
    uint32 pixel = 0;
    uint16 value;
    PixelColour pc;

    while (out < end)
    {
        pixel = *(data++) << 16;
        pixel |= *(data++) << 8;
        pixel |= *(data++);
        SPLITCOLOUR24(pixel, pc);
        value = MAKECOLOUR(pc);
        if (g_xserver_be)
        {
            BOUT16(out, value);
        }
        else
        {
            LOUT16(out, value);
        }
    }
}

static void
translate24to24(const uint8 * data, uint8 * out, uint8 * end)
{
    uint32 pixel;
    uint32 value;
    PixelColour pc;

    if (g_xserver_be)
    {
        while (out < end)
        {
            pixel = *(data++) << 16;
            pixel |= *(data++) << 8;
            pixel |= *(data++);
            SPLITCOLOUR24(pixel, pc);
            value = MAKECOLOUR(pc);
            BOUT24(out, value);
        }
    }
    else
    {
        while (out < end)
        {
            pixel = *(data++) << 16;
            pixel |= *(data++) << 8;
            pixel |= *(data++);
            SPLITCOLOUR24(pixel, pc);
            value = MAKECOLOUR(pc);
            LOUT24(out, value);
        }
    }
}

static void
translate24to32(const uint8 * data, uint8 * out, uint8 * end)
{
    uint32 pixel;
    uint32 value;
    PixelColour pc;

    if (g_compatible_arch)
    {
        /* *INDENT-OFF* */
#ifdef NEED_ALIGN
        REPEAT4
        (
            *(out++) = *(data++);
            *(out++) = *(data++);
            *(out++) = *(data++);
            *(out++) = 0;
        )
#else
        REPEAT4
        (
            /* Only read 3 bytes. Reading 4 bytes means reading beyond buffer. */
            *((uint32 *) out) = *((uint16 *) data) + (*((uint8 *) data + 2) << 16);
            out += 4;
            data += 3;
        )
#endif
        /* *INDENT-ON* */
    }
    else if (g_xserver_be)
    {
        while (out < end)
        {
            pixel = *(data++) << 16;
            pixel |= *(data++) << 8;
            pixel |= *(data++);
            SPLITCOLOUR24(pixel, pc);
            value = MAKECOLOUR(pc);
            BOUT32(out, value);
        }
    }
    else
    {
        while (out < end)
        {
            pixel = *(data++) << 16;
            pixel |= *(data++) << 8;
            pixel |= *(data++);
            SPLITCOLOUR24(pixel, pc);
            value = MAKECOLOUR(pc);
            LOUT32(out, value);
        }
    }
}

static void
translate_pnrdp(const uint8 * data, uint8 * out, uint8 * end)
{
//    if (g_compatible_arch)
//    {
        /* *INDENT-OFF* */
        REPEAT4
        (
            *(out++) = data[2];
            *(out++) = data[1];
            *(out++) = data[0];
            *(out++) = 0;
            data += 3;
        )
        /* *INDENT-ON* */
//    }
}

static uint8 *
translate_pnrdp_image(int width, int height, uint8 * data)
{
    int size;
    uint8 *out;
    uint8 *end;

    size = width * height * (g_bpp / 8);
    out = (uint8 *) xmalloc(size);
    end = out + size;
    
    switch (g_bpp)
    {
        case 32:
            translate24to32(data, out, end);
            translate_pnrdp(data, out, end);
            break;
        case 24:
            translate24to24(data, out, end);
            break;
        case 16:
            translate24to16(data, out, end);
            break;
    }
    

    return out;
}

static uint8 *
translate_image(int width, int height, uint8 * data)
{
    int size;
    uint8 *out;
    uint8 *end;

    /*
       If RDP depth and X Visual depths match,
       and arch(endian) matches, no need to translate:
       just return data.
       Note: select_visual should've already ensured g_no_translate
       is only set for compatible depths, but the RDP depth might've
       changed during connection negotiations.
     */

    /* todo */
    if (g_server_depth == 32 && g_depth == 24)
    {
        return data;
    }

    if (g_no_translate_image)
    {
        if ((g_depth == 15 && g_server_depth == 15) ||
                (g_depth == 16 && g_server_depth == 16) ||
                (g_depth == 24 && g_server_depth == 24))
            return data;
    }

    size = width * height * (g_bpp / 8);
    out = (uint8 *) xmalloc(size);
    end = out + size;

    switch (g_server_depth)
    {
    case 24:
        switch (g_bpp)
        {
        case 32:
            translate24to32(data, out, end);
            break;
        case 24:
            translate24to24(data, out, end);
            break;
        case 16:
            translate24to16(data, out, end);
            break;
        }
        break;
    case 16:
        switch (g_bpp)
        {
        case 32:
            translate16to32((uint16 *) data, out, end);
            break;
        case 24:
            translate16to24((uint16 *) data, out, end);
            break;
        case 16:
            translate16to16((uint16 *) data, out, end);
            break;
        }
        break;
    case 15:
        switch (g_bpp)
        {
        case 32:
            translate15to32((uint16 *) data, out, end);
            break;
        case 24:
            translate15to24((uint16 *) data, out, end);
            break;
        case 16:
            translate15to16((uint16 *) data, out, end);
            break;
        }
        break;
    case 8:
        switch (g_bpp)
        {
        case 8:
            translate8to8(data, out, end);
            break;
        case 16:
            translate8to16(data, out, end);
            break;
        case 24:
            translate8to24(data, out, end);
            break;
        case 32:
            translate8to32(data, out, end);
            break;
        }
        break;
    }
    return out;
}

static void
xwin_refresh_pointer_map(void)
{
    unsigned char phys_to_log_map[sizeof(g_pointer_log_to_phys_map)];
    int i, pointer_buttons;

    pointer_buttons = XGetPointerMapping(g_display, phys_to_log_map, sizeof(phys_to_log_map));
    if (pointer_buttons > sizeof(phys_to_log_map))
        pointer_buttons = sizeof(phys_to_log_map);

    /* if multiple physical buttons map to the same logical button, then
     * use the lower numbered physical one */
    for (i = pointer_buttons - 1; i >= 0; i--)
    {
        /* a user could specify arbitrary values for the logical button
         * number, ignore any that are abnormally large */
        if (phys_to_log_map[i] > sizeof(g_pointer_log_to_phys_map))
            continue;
        g_pointer_log_to_phys_map[phys_to_log_map[i] - 1] = i + 1;
    }
}

RD_BOOL
get_key_state(unsigned int state, uint32 keysym)
{
    int modifierpos, key, keysymMask = 0;
    int offset;

    KeyCode keycode = XKeysymToKeycode(g_display, keysym);

    if (keycode == NoSymbol)
        return False;

    for (modifierpos = 0; modifierpos < 8; modifierpos++)
    {
        offset = g_mod_map->max_keypermod * modifierpos;

        for (key = 0; key < g_mod_map->max_keypermod; key++)
        {
            if (g_mod_map->modifiermap[offset + key] == keycode)
                keysymMask |= 1 << modifierpos;
        }
    }

    return (state & keysymMask) ? True : False;
}

static void
calculate_shifts(uint32 mask, int *shift_r, int *shift_l)
{
    *shift_l = ffs(mask) - 1;
    mask >>= *shift_l;
    *shift_r = 8 - ffs(mask & ~(mask >> 1));
}

/* Given a mask of a colour channel (e.g. XVisualInfo.red_mask),
   calculates the bits-per-pixel of this channel (a.k.a. colour weight).
 */
static unsigned
calculate_mask_weight(uint32 mask)
{
    unsigned weight = 0;
    do
    {
        weight += (mask & 1);
    }
    while (mask >>= 1);
    return weight;
}

static RD_BOOL
select_visual(int screen_num)
{
    XPixmapFormatValues *pfm;
    int pixmap_formats_count, visuals_count;
    XVisualInfo *vmatches = NULL;
    XVisualInfo template;
    int i;
    unsigned red_weight, blue_weight, green_weight;

    red_weight = blue_weight = green_weight = 0;

    if (g_server_depth == -1)
    {
        g_server_depth = DisplayPlanes(g_display, DefaultScreen(g_display));
    }

    pfm = XListPixmapFormats(g_display, &pixmap_formats_count);
    if (pfm == NULL)
    {
        error("Unable to get list of pixmap formats from display.\n");
        XCloseDisplay(g_display);
        return False;
    }

    /* Search for best TrueColor visual */
    template.class = TrueColor;
    template.screen = screen_num;
    vmatches =
    XGetVisualInfo(g_display, VisualClassMask | VisualScreenMask, &template,
                   &visuals_count);
    g_visual = NULL;
    g_no_translate_image = False;
    g_compatible_arch = False;
    if (vmatches != NULL)
    {
        for (i = 0; i < visuals_count; ++i)
        {
            XVisualInfo *visual_info = &vmatches[i];
            RD_BOOL can_translate_to_bpp = False;
            int j;

            /* Try to find a no-translation visual that'll
               allow us to use RDP bitmaps directly as ZPixmaps. */
            if (!g_xserver_be && (((visual_info->depth == 15) &&
                                   /* R5G5B5 */
                                   (visual_info->red_mask == 0x7c00) &&
                                   (visual_info->green_mask == 0x3e0) &&
                                   (visual_info->blue_mask == 0x1f)) ||
                                  ((visual_info->depth == 16) &&
                                   /* R5G6B5 */
                                   (visual_info->red_mask == 0xf800) &&
                                   (visual_info->green_mask == 0x7e0) &&
                                   (visual_info->blue_mask == 0x1f)) ||
                                  ((visual_info->depth == 24) &&
                                   /* R8G8B8 */
                                   (visual_info->red_mask == 0xff0000) &&
                                   (visual_info->green_mask == 0xff00) &&
                                   (visual_info->blue_mask == 0xff))))
            {
                g_visual = visual_info->visual;
                g_depth = visual_info->depth;
                g_compatible_arch = !g_host_be;
                g_no_translate_image = (visual_info->depth == g_server_depth);
                if (g_no_translate_image)
                    /* We found the best visual */
                    break;
            }
            else
            {
                g_compatible_arch = False;
            }

            if (visual_info->depth > 24)
            {
                /* Avoid 32-bit visuals and likes like the plague.
                   They're either untested or proven to work bad
                   (e.g. nvidia's Composite 32-bit visual).
                   Most implementation offer a 24-bit visual anyway. */
                continue;
            }

            /* Only care for visuals, for whose BPPs (not depths!)
               we have a translateXtoY function. */
            for (j = 0; j < pixmap_formats_count; ++j)
            {
                if (pfm[j].depth == visual_info->depth)
                {
                    if ((pfm[j].bits_per_pixel == 16) ||
                            (pfm[j].bits_per_pixel == 24) ||
                            (pfm[j].bits_per_pixel == 32))
                    {
                        can_translate_to_bpp = True;
                    }
                    break;
                }
            }

            /* Prefer formats which have the most colour depth.
               We're being truly aristocratic here, minding each
               weight on its own. */
            if (can_translate_to_bpp)
            {
                unsigned vis_red_weight =
                    calculate_mask_weight(visual_info->red_mask);
                unsigned vis_green_weight =
                    calculate_mask_weight(visual_info->green_mask);
                unsigned vis_blue_weight =
                    calculate_mask_weight(visual_info->blue_mask);
                if ((vis_red_weight >= red_weight)
                        && (vis_green_weight >= green_weight)
                        && (vis_blue_weight >= blue_weight))
                {
                    red_weight = vis_red_weight;
                    green_weight = vis_green_weight;
                    blue_weight = vis_blue_weight;
                    g_visual = visual_info->visual;
                    g_depth = visual_info->depth;
                }
            }
        }
        XFree(vmatches);
    }

    if (g_visual != NULL)
    {
        g_owncolmap = False;
        calculate_shifts(g_visual->red_mask, &g_red_shift_r, &g_red_shift_l);
        calculate_shifts(g_visual->green_mask, &g_green_shift_r, &g_green_shift_l);
        calculate_shifts(g_visual->blue_mask, &g_blue_shift_r, &g_blue_shift_l);
    }
    else
    {
        template.class = PseudoColor;
        template.depth = 8;
        template.colormap_size = 256;
        vmatches =
            XGetVisualInfo(g_display,
                           VisualClassMask | VisualDepthMask | VisualColormapSizeMask,
                           &template, &visuals_count);
        if (vmatches == NULL)
        {
            error("No usable TrueColor or PseudoColor visuals on this display.\n");
            XCloseDisplay(g_display);
            XFree(pfm);
            return False;
        }

        /* we use a colourmap, so the default visual should do */
        g_owncolmap = True;
        g_visual = vmatches[0].visual;
        g_depth = vmatches[0].depth;
    }

    g_bpp = 0;
    for (i = 0; i < pixmap_formats_count; ++i)
    {
        XPixmapFormatValues *pf = &pfm[i];
        if (pf->depth == g_depth)
        {
            g_bpp = pf->bits_per_pixel;

            if (g_no_translate_image)
            {
                switch (g_server_depth)
                {
                case 15:
                case 16:
                    if (g_bpp != 16)
                        g_no_translate_image = False;
                    break;
                case 24:
                    /* Yes, this will force image translation
                       on most modern servers which use 32 bits
                       for R8G8B8. */
                    if (g_bpp != 24)
                        g_no_translate_image = False;
                    break;
                default:
                    g_no_translate_image = False;
                    break;
                }
            }

            /* Pixmap formats list is a depth-to-bpp mapping --
               there's just a single entry for every depth,
               so we can safely break here */
            break;
        }
    }
    XFree(pfm);
    pfm = NULL;
    return True;
}

static XErrorHandler g_old_error_handler;
static RD_BOOL g_error_expected = False;

/* Check if the X11 window corresponding to a seamless window with
   specified id exists. */
RD_BOOL
sw_window_exists(unsigned long id)
{
    seamless_window *sw;
    char *name;
    Status sts = 0;

    sw = sw_get_window_by_id(id);
    if (!sw)
        return False;

    g_error_expected = True;
    sts = XFetchName(g_display, sw->wnd, &name);
    g_error_expected = False;
    if (sts)
    {
        XFree(name);
    }

    return sts;
}

static int
error_handler(Display * dpy, XErrorEvent * eev)
{
    if (g_error_expected)
        return 0;

    return g_old_error_handler(dpy, eev);
}

RD_BOOL
ui_init(void)
{
    XineramaScreenInfo* x_screens = NULL;
    int screen_num;
    int xnum;

    g_display = XOpenDisplay(NULL);
    if (g_display == NULL)
    {
        error("Failed to open display: %s\n", XDisplayName(NULL));
        return False;
    }

    {
        uint16 endianess_test = 1;
        g_host_be = !(RD_BOOL) (*(uint8 *) (&endianess_test));
    }

    x_screens = XineramaQueryScreens(g_display, &xnum);

    g_old_error_handler = XSetErrorHandler(error_handler);
    g_xserver_be = (ImageByteOrder(g_display) == MSBFirst);
    screen_num = DefaultScreen(g_display);
    g_x_socket = ConnectionNumber(g_display);
    g_screen = ScreenOfDisplay(g_display, screen_num);
    g_depth = DefaultDepthOfScreen(g_screen);

    if (!select_visual(screen_num))
        return False;

    if (g_no_translate_image)
    {
        DEBUG(("Performance optimization possible: avoiding image translation (colour depth conversion).\n"));
    }

    if (g_server_depth > g_bpp)
    {
        warning("Remote desktop colour depth %d higher than display colour depth %d.\n",
                g_server_depth, g_bpp);
    }

    DEBUG(("RDP depth: %d, display depth: %d, display bpp: %d, X server BE: %d, host BE: %d\n",
           g_server_depth, g_depth, g_bpp, g_xserver_be, g_host_be));

    if (!g_owncolmap)
    {
        g_xcolmap =
            XCreateColormap(g_display, RootWindowOfScreen(g_screen), g_visual,
                            AllocNone);
        if (g_depth <= 8)
            warning("Display colour depth is %d bit: you may want to use -C for a private colourmap.\n", g_depth);
    }

    if ((!g_ownbackstore) && (DoesBackingStore(g_screen) != Always))
    {
        warning("External BackingStore not available. Using internal.\n");
        g_ownbackstore = True;
    }

    /*
     * Determine desktop size
     */
    if (g_fullscreen)
    {
        g_width = g_span_monitors ? WidthOfScreen(g_screen) : xnum > 0 ? x_screens[0].width : 0;
        g_height = g_span_monitors ? HeightOfScreen(g_screen) : xnum > 0 ? x_screens[0].height : 0;

        g_using_full_workarea = True;
    }
    else if (g_width < 0)
    {
        /* Percent of screen */
        if (-g_width >= 100)
            g_using_full_workarea = True;
        g_height = HeightOfScreen(g_screen) * (-g_width) / 100;
        g_width = WidthOfScreen(g_screen) * (-g_width) / 100;
    }
    else if (g_width == 0)
    {
        /* Fetch geometry from _NET_WORKAREA */
        uint32 x, y, cx, cy;
        if (get_current_workarea(&x, &y, &cx, &cy) == 0)
        {
            g_width = cx;
            g_height = cy;
            g_using_full_workarea = True;
        }
        else
        {
            warning("Failed to get workarea: probably your window manager does not support extended hints\n");
            g_width = WidthOfScreen(g_screen);
            g_height = HeightOfScreen(g_screen);
        }
    }

    if (g_seamless_rdp)
    {
        /* Fetch geometry from _NET_WORKAREA */
        uint32 x, y, cx, cy;
        if (get_current_workarea(&x, &y, &cx, &cy) == 0)
        {
            g_width = cx;
            g_height = cy;
            g_using_full_workarea = True;
        }
    }

    /* make sure width is a multiple of 4 */
    g_width = (g_width + 3) & ~3;

    //syslog (LOG_NOTICE, "___WidthOfScreen(g_screen)=%d,  g_width=%d\n", WidthOfScreen(g_screen),  g_width);
    //syslog (LOG_NOTICE, "___HeightOfScreen(g_screen)=%d, g_height=%d\n", HeightOfScreen(g_screen), g_height);

    g_mod_map = XGetModifierMapping(g_display);
    xwin_refresh_pointer_map();

    xkeymap_init();

    if (g_enable_compose)
        g_IM = XOpenIM(g_display, NULL, NULL, NULL);

    xclip_init();
    ewmh_init();
    if (g_seamless_rdp)
    {
        seamless_init();
    }

    DEBUG_RDP5(("server bpp %d client bpp %d depth %d\n", g_server_depth, g_bpp, g_depth));

    XInitThreads();

    return True;
}

void
ui_deinit(void)
{
    while (g_seamless_windows)
    {
        XDestroyWindow(g_display, g_seamless_windows->wnd);
        sw_remove_window(g_seamless_windows);
    }

    xclip_deinit();

    if (g_IM != NULL)
        XCloseIM(g_IM);

    if (g_null_cursor != NULL)
        ui_destroy_cursor(g_null_cursor);

    XFreeModifiermap(g_mod_map);

    if (g_ownbackstore)
        XFreePixmap(g_display, g_backstore);

    XFreeGC(g_display, g_gc);
    XCloseDisplay(g_display);
    g_display = NULL;
}


static void
get_window_attribs(XSetWindowAttributes * attribs)
{
    attribs->background_pixel = BlackPixelOfScreen(g_screen);
    attribs->background_pixel = WhitePixelOfScreen(g_screen);
    attribs->border_pixel = WhitePixelOfScreen(g_screen);
    attribs->backing_store = g_ownbackstore ? NotUseful : Always;
    attribs->override_redirect = g_fullscreen;
    attribs->colormap = g_xcolmap;
}

static void
get_input_mask(long *input_mask)
{
    *input_mask = KeyPressMask | KeyReleaseMask | ButtonPressMask | ButtonReleaseMask |
                  VisibilityChangeMask | FocusChangeMask | StructureNotifyMask;

    if (g_sendmotion)
        *input_mask |= PointerMotionMask;
    if (g_ownbackstore)
        *input_mask |= ExposureMask;
    if (g_fullscreen || g_grab_keyboard)
        *input_mask |= EnterWindowMask;
    if (g_grab_keyboard)
        *input_mask |= LeaveWindowMask;
}

void switchToFullscreen()
{
    const int longDataFormat = 32;

    Atom fullscreenAtom = XInternAtom(g_display, "_NET_WM_STATE_FULLSCREEN", True);
    Atom stateAtom = XInternAtom(g_display, "_NET_WM_STATE", True);

    XClientMessageEvent event;
    event.type = ClientMessage;
    event.serial = 0;
    event.send_event = True;
    event.window = g_wnd;
    event.message_type = stateAtom;
    event.format = longDataFormat;
    event.data.l[0] = 1;
    event.data.l[1] = fullscreenAtom;
    event.data.l[2] = 0;

    XSendEvent(g_display, DefaultRootWindow(g_display), False, SubstructureRedirectMask | SubstructureNotifyMask,
               (XEvent*) &event);
}

RD_BOOL
ui_create_window(void)
{
    uint8 null_pointer_mask[1] = { 0x80 };
    uint8 null_pointer_data[24] = { 0x00 };
    pid_t pid;
    unsigned long pid_l;
    Atom pid_atom;
    int rc;
    char hostname[90];
    char* x_hostnames;

    XTextProperty hostname_property;

    XSetWindowAttributes attribs;
    XClassHint *classhints;
    XSizeHints *sizehints;
    int wndwidth, wndheight;
    long input_mask, ic_input_mask;
    XEvent xevent;

    wndwidth = (g_fullscreen && g_span_monitors) ? WidthOfScreen(g_screen) : g_width;
    wndheight = (g_fullscreen && g_span_monitors) ? HeightOfScreen(g_screen) : g_height;

    /* Handle -x-y portion of geometry string */
    if (g_xpos < 0 || (g_xpos == 0 && (g_pos & 2)))
        g_xpos = WidthOfScreen(g_screen) + g_xpos - g_width;
    if (g_ypos < 0 || (g_ypos == 0 && (g_pos & 4)))
        g_ypos = HeightOfScreen(g_screen) + g_ypos - g_height;

    if (g_seamless_rdp)
    {
        // Fetch geometry from _NET_WORKAREA
        uint32 x, y, cx, cy;
        if (get_current_workarea(&x, &y, &cx, &cy) == 0)
        {
            g_xpos = x;
            g_ypos = y;
            wndwidth =  g_width;
            wndheight = g_height;
        }
    }

    syslog (LOG_NOTICE, "g_xpos=%d, wndwidth=%d,  g_width=%d\n", g_xpos, wndwidth,  g_width);
    syslog (LOG_NOTICE, "g_ypos=%d, wndheight=%d, g_height=%d\n", g_ypos, wndheight, g_height);

    get_window_attribs(&attribs);

    g_wnd = XCreateWindow(g_display, RootWindowOfScreen(g_screen), g_xpos, g_ypos, wndwidth,
                          wndheight, 0, g_depth, InputOutput, g_visual,
                          CWBackPixel | CWBackingStore | CWColormap | CWOverrideRedirect |
                          CWBorderPixel, &attribs);

    /* XSetWMProperties(g_dislpay, g_wnd, window_name, icon_name,
        argv, argc, normal_hints, wm_hints, class_hints); also does the
        XSetWMClientMachine bit too */
    if (gethostname(hostname, sizeof(hostname)) != 0)
    {
        fprintf(stderr, "error: could not determine hostname.\n");
        exit(1);
    }
    x_hostnames = hostname;

    if (XStringListToTextProperty(&x_hostnames, 1, &hostname_property) != 0)
    {
        XSetWMClientMachine(g_display, g_wnd, &hostname_property);
        XFree(hostname_property.value);
    }

    pid = getpid();
    pid_l = pid;
    pid_atom = XInternAtom(g_display, "_NET_WM_PID", False);
    rc = XChangeProperty(g_display, g_wnd,
                         pid_atom, XA_CARDINAL, 32, PropModeReplace,
                         (unsigned char*)&pid_l, 1);

    if (g_gc == NULL)
    {
        XGCValues gcv;
        g_gc = XCreateGC(g_display, g_wnd, 0, &gcv);
        ui_reset_clip();
    }

    if (g_create_bitmap_gc == NULL)
        g_create_bitmap_gc = XCreateGC(g_display, g_wnd, 0, NULL);

    if ((g_ownbackstore) && (g_backstore == 0))
    {
       syslog( LOG_INFO, "Creating local pixmap backstore" );
        g_backstore = XCreatePixmap(g_display, g_wnd, g_width, g_height, g_depth);
        g_backstore2 = XCreatePixmap(g_display, g_wnd, g_width, g_height, g_depth);

        /* clear to prevent rubbish being exposed at startup */
        XSetForeground(g_display, g_gc, BlackPixelOfScreen(g_screen));
        XFillRectangle(g_display, g_backstore, g_gc, 0, 0, g_width, g_height);
        XFillRectangle(g_display, g_backstore2, g_gc, 0, 0, g_width, g_height);
    }

    XStoreName(g_display, g_wnd, g_title);
    ewmh_set_wm_name(g_wnd, g_title);

    if (g_hide_decorations)
        mwm_hide_decorations(g_wnd);

    classhints = XAllocClassHint();
    if (classhints != NULL)
    {
        classhints->res_name = classhints->res_class = "rdesktop";
        XSetClassHint(g_display, g_wnd, classhints);
        XFree(classhints);
    }

    sizehints = XAllocSizeHints();
    if (sizehints)
    {
        sizehints->flags = PMinSize | PMaxSize;
        if (g_pos)
            sizehints->flags |= PPosition;
        sizehints->min_width = sizehints->max_width = g_width;
        sizehints->min_height = sizehints->max_height = g_height;
        XSetWMNormalHints(g_display, g_wnd, sizehints);
        XFree(sizehints);
    }

    if (g_embed_wnd)
    {
        XReparentWindow(g_display, g_wnd, (Window) g_embed_wnd, 0, 0);
    }

    get_input_mask(&input_mask);

    if (g_IM != NULL)
    {
        g_IC = XCreateIC(g_IM, XNInputStyle, (XIMPreeditNothing | XIMStatusNothing),
                         XNClientWindow, g_wnd, XNFocusWindow, g_wnd, NULL);

        if ((g_IC != NULL)
                && (XGetICValues(g_IC, XNFilterEvents, &ic_input_mask, NULL) == NULL))
            input_mask |= ic_input_mask;
    }

    XSelectInput(g_display, g_wnd, input_mask);

    if (!g_seamless_rdp)
    {
        XMapWindow(g_display, g_wnd);

        /* wait for VisibilityNotify */
        do
        {
            XMaskEvent(g_display, VisibilityChangeMask, &xevent);
        }
        while (xevent.type != VisibilityNotify);
        g_Unobscured = xevent.xvisibility.state == VisibilityUnobscured;
    }

    g_focused = False;
    g_mouse_in_wnd = False;

    /* handle the WM_DELETE_WINDOW protocol */
    g_protocol_atom = XInternAtom(g_display, "WM_PROTOCOLS", True);
    g_kill_atom = XInternAtom(g_display, "WM_DELETE_WINDOW", True);
    XSetWMProtocols(g_display, g_wnd, &g_kill_atom, 1);

    /* create invisible 1x1 cursor to be used as null cursor */
    if (g_null_cursor == NULL)

// _CURSOR_FIX
        g_null_cursor = ui_create_cursor(0, 0, 1, 1, null_pointer_mask, null_pointer_data, 24);

    if (g_seamless_rdp)
    {
        seamless_restack_test();
    }

    return True;
}

void
ui_resize_window()
{
    XSizeHints *sizehints;
    Pixmap bs;

    sizehints = XAllocSizeHints();
    if (sizehints)
    {
        sizehints->flags = PMinSize | PMaxSize;
        sizehints->min_width = sizehints->max_width = g_width;
        sizehints->min_height = sizehints->max_height = g_height;
        XSetWMNormalHints(g_display, g_wnd, sizehints);
        XFree(sizehints);
    }

    if (!(g_fullscreen || g_embed_wnd))
    {
        XResizeWindow(g_display, g_wnd, g_width, g_height);
    }

    /* create new backstore pixmap */
    if (g_backstore != 0)
    {
        bs = XCreatePixmap(g_display, g_wnd, g_width, g_height, g_depth);
        XSetForeground(g_display, g_gc, BlackPixelOfScreen(g_screen));
        XFillRectangle(g_display, bs, g_gc, 0, 0, g_width, g_height);
        XCopyArea(g_display, g_backstore, bs, g_gc, 0, 0, g_width, g_height, 0, 0);
        XFreePixmap(g_display, g_backstore);
        g_backstore = bs;

        // Do this for backstore 2 as well
        bs = XCreatePixmap(g_display, g_wnd, g_width, g_height, g_depth);
        XSetForeground(g_display, g_gc, BlackPixelOfScreen(g_screen));
        XFillRectangle(g_display, bs, g_gc, 0, 0, g_width, g_height);
        XCopyArea(g_display, g_backstore2, bs, g_gc, 0, 0, g_width, g_height, 0, 0);
        XFreePixmap(g_display, g_backstore2);
        g_backstore2 = bs;
    }
}

void
ui_destroy_window(void)
{
    if (g_IC != NULL)
        XDestroyIC(g_IC);

    XDestroyWindow(g_display, g_wnd);
}

void
xwin_toggle_fullscreen(void)
{
    Pixmap contents = 0;

    if (g_seamless_active)
        /* Turn off SeamlessRDP mode */
        ui_seamless_toggle();

    if (!g_ownbackstore)
    {
        /* need to save contents of window */
        contents = XCreatePixmap(g_display, g_wnd, g_width, g_height, g_depth);
        XCopyArea(g_display, g_wnd, contents, g_gc, 0, 0, g_width, g_height, 0, 0);
    }

    ui_destroy_window();
    g_fullscreen = !g_fullscreen;
    ui_create_window();

    XDefineCursor(g_display, g_wnd, g_current_cursor);

    if (!g_ownbackstore)
    {
        XCopyArea(g_display, contents, g_wnd, g_gc, 0, 0, g_width, g_height, 0, 0);
        XFreePixmap(g_display, contents);
    }
}

static void
handle_button_event(XEvent xevent, RD_BOOL down)
{
    uint16 button, flags = 0;
    g_last_gesturetime = xevent.xbutton.time;
    /* Reverse the pointer button mapping, e.g. in the case of
       "left-handed mouse mode"; the RDP session expects to
       receive physical buttons (true in mstsc as well) and
       logical button behavior depends on the remote desktop's own
       mouse settings */
    xevent.xbutton.button = g_pointer_log_to_phys_map[xevent.xbutton.button - 1];
    button = xkeymap_translate_button(xevent.xbutton.button);
    if (button == 0)
        return;

    if (down)
        flags = MOUSE_FLAG_DOWN;

    qprocess_moving_wnd_seamless (xevent);

    /* Stop moving window when button is released, regardless of cursor position */
    if (g_moving_wnd && (xevent.type == ButtonRelease))
        g_moving_wnd = False;

    /* If win_button_size is nonzero, enable single app mode */
    if (xevent.xbutton.y < g_win_button_size)
    {
        /*  Check from right to left: */
        if (xevent.xbutton.x >= g_width - g_win_button_size)
        {
            /* The close button, continue */
            ;
        }
        else if (xevent.xbutton.x >= g_width - g_win_button_size * 2)
        {
            /* The maximize/restore button. Do not send to
               server.  It might be a good idea to change the
               cursor or give some other visible indication
               that rdesktop inhibited this click */
            if (xevent.type == ButtonPress)
                return;
        }
        else if (xevent.xbutton.x >= g_width - g_win_button_size * 3)
        {
            /* The minimize button. Iconify window. */
            if (xevent.type == ButtonRelease)
            {
                /* Release the mouse button outside the minimize button, to prevent the
                   actual minimazation to happen */
                rdp_send_input(time(NULL), RDP_INPUT_MOUSE, button, 1, 1);
                XIconifyWindow(g_display, g_wnd, DefaultScreen(g_display));
                return;
            }
        }
        else if (xevent.xbutton.x <= g_win_button_size)
        {
            /* The system menu. Ignore. */
            if (xevent.type == ButtonPress)
                return;
        }
        else
        {
            /* The title bar. */
            if (xevent.type == ButtonPress)
            {
                if (!g_fullscreen && g_hide_decorations && !g_using_full_workarea)
                {
                    g_moving_wnd = True;
                    g_move_x_offset = xevent.xbutton.x;
                    g_move_y_offset = xevent.xbutton.y;
                }
                return;
            }
        }
    }

    if (xevent.xmotion.window == g_wnd)
    {
        rdp_send_input(time(NULL), RDP_INPUT_MOUSE,
                       flags | button, xevent.xbutton.x, xevent.xbutton.y);
    }
    else
    {
        /* SeamlessRDP */
        rdp_send_input(time(NULL), RDP_INPUT_MOUSE,
                       flags | button, xevent.xbutton.x_root - g_xpos, xevent.xbutton.y_root - g_ypos);
    }
}


/* Process events in Xlib queue
   Returns 0 after user quit, 1 otherwise */
static int
xwin_process_events(void)
{
    XEvent xevent;
    KeySym keysym;
    uint32 ev_time;
    char str[256];
    Status status;
    int events = 0;
    seamless_window *sw;

    while ((XPending(g_display) > 0) && events++ < 20)
    {
        XNextEvent(g_display, &xevent);

        // Lock the display
        XLockDisplay( g_display );

        if ((g_IC != NULL) && (XFilterEvent(&xevent, None) == True))
        {
            DEBUG_KBD(("Filtering event\n"));
            continue;
        }

        switch (xevent.type)
        {
        case VisibilityNotify:
            if (xevent.xvisibility.window == g_wnd)
                g_Unobscured =
                    xevent.xvisibility.state == VisibilityUnobscured;

            break;
        case ClientMessage:
            /* the window manager told us to quit */
            if ((xevent.xclient.message_type == g_protocol_atom)
                    && ((Atom) xevent.xclient.data.l[0] == g_kill_atom))
            {
                /* When killing a seamless window, close the window on the
                   serverside instead of terminating rdesktop */
                sw = sw_get_window_by_wnd(xevent.xclient.window);
                if (!sw)
                {
                    /* Otherwise, quit */
                    return 0;
                }
                /* send seamless destroy process message */
                //seamless_send_destroy(sw->id);
                if (sw_window_exists(sw->id))
                    qseamless_send_message_SY_CLOSE(sw->id);
            }
            break;

        case KeyPress:
            g_last_gesturetime = xevent.xkey.time;
            if (g_IC != NULL)
                /* Multi_key compatible version */
            {
                XmbLookupString(g_IC,
                                &xevent.xkey, str, sizeof(str), &keysym,
                                &status);
                if (!((status == XLookupKeySym) || (status == XLookupBoth)))
                {
                    error("XmbLookupString failed with status 0x%x\n",
                          status);
                    break;
                }
            }
            else
            {
                /* Plain old XLookupString */
                DEBUG_KBD(("\nNo input context, using XLookupString\n"));
                XLookupString((XKeyEvent *) & xevent,
                              str, sizeof(str), &keysym, NULL);
            }

            DEBUG_KBD(("KeyPress for keysym (0x%lx, %s)\n", keysym,
                       get_ksname(keysym)));

            ev_time = time(NULL);
            if (handle_special_keys(keysym, xevent.xkey.state, ev_time, True))
                break;

            xkeymap_send_keys(keysym, xevent.xkey.keycode, xevent.xkey.state,
                              ev_time, True, 0);
            break;

        case KeyRelease:
            g_last_gesturetime = xevent.xkey.time;
            XLookupString((XKeyEvent *) & xevent, str,
                          sizeof(str), &keysym, NULL);

            DEBUG_KBD(("\nKeyRelease for keysym (0x%lx, %s)\n", keysym,
                       get_ksname(keysym)));

            ev_time = time(NULL);
            if (handle_special_keys(keysym, xevent.xkey.state, ev_time, False))
                break;

            xkeymap_send_keys(keysym, xevent.xkey.keycode, xevent.xkey.state,
                              ev_time, False, 0);
            break;

        case ButtonPress:
            handle_button_event(xevent, True);
            break;

        case ButtonRelease:
            handle_button_event(xevent, False);
            break;

        case MotionNotify:
                qprocess_moving_wnd_seamless_size (xevent);
                qprocess_moving_wnd_seamless_move (xevent);

            if (g_moving_wnd)
            {
                XMoveWindow(g_display, g_wnd,
                            xevent.xmotion.x_root - g_move_x_offset,
                            xevent.xmotion.y_root - g_move_y_offset);
                break;
            }

            if (g_fullscreen && !g_focused)
                XSetInputFocus(g_display, g_wnd, RevertToPointerRoot,
                               CurrentTime);

            if (xevent.xmotion.window == g_wnd)
            {
                rdp_send_input(time(NULL), RDP_INPUT_MOUSE, MOUSE_FLAG_MOVE,
                               xevent.xmotion.x, xevent.xmotion.y);
            }
            else
            {
                /* SeamlessRDP */
                rdp_send_input(time(NULL), RDP_INPUT_MOUSE, MOUSE_FLAG_MOVE,
                               xevent.xmotion.x_root - g_xpos,
                               xevent.xmotion.y_root - g_ypos);
            }
            break;

        case FocusIn:
            if (xevent.xfocus.mode == NotifyGrab)
                break;
            g_focused = True;
            reset_modifier_keys();
            if (g_grab_keyboard && g_mouse_in_wnd)
                XGrabKeyboard(g_display, g_wnd, True,
                              GrabModeAsync, GrabModeAsync, CurrentTime);

            sw = sw_get_window_by_wnd(xevent.xfocus.window);
            if (!sw)
                break;

            if (sw->tooltip)
                break;

            /* Menu windows are real X11 windows,
               with focus. When such a window is
               destroyed, focus is reverted to the
               main application window, which
               would cause us to send FOCUS. This
               breaks window switching in, say,
               Seamonkey. We shouldn't need to
               send FOCUS: Windows should also
               revert focus to some other window
               when the menu window is
               destroyed. So, we only send FOCUS
               if the previous focus window still
               exists. */
            if (sw->id != g_seamless_focused)
            {

                if (sw_window_exists(g_seamless_focused))
                {
                        //seamless_send_focus(sw->id, 0);
                        qseamless_send_message_WA_CLICKACTIVE(sw->id);
                }
                g_seamless_focused = sw->id;
            }
            break;

        case FocusOut:
            if (xevent.xfocus.mode == NotifyUngrab)
                break;
            g_focused = False;
            if (xevent.xfocus.mode == NotifyWhileGrabbed)
                XUngrabKeyboard(g_display, CurrentTime);
            break;

        case EnterNotify:
            /* we only register for this event when in fullscreen mode */
            /* or grab_keyboard */
            g_mouse_in_wnd = True;
            if (g_fullscreen)
            {
                XSetInputFocus(g_display, g_wnd, RevertToPointerRoot,
                               CurrentTime);
                break;
            }
            if (g_focused)
                XGrabKeyboard(g_display, g_wnd, True,
                              GrabModeAsync, GrabModeAsync, CurrentTime);
            break;

        case LeaveNotify:
            /* we only register for this event when grab_keyboard */
            g_mouse_in_wnd = False;
            XUngrabKeyboard(g_display, CurrentTime);
            break;

        case Expose:
            if (xevent.xexpose.window == g_wnd)
            {
                //syslog( LOG_INFO, "Handling an expose event");
                XCopyArea(g_display, g_backstore, xevent.xexpose.window,
                          g_gc,
                          xevent.xexpose.x, xevent.xexpose.y,
                          xevent.xexpose.width, xevent.xexpose.height,
                          xevent.xexpose.x, xevent.xexpose.y);
                }
                else if (xevent.xexpose.window == gq_fake_wnd && g_moving_wnd_seamless)
                {
                    seamless_window *sw = sw_get_window_by_id (g_moving_wnd_seamless);
                    if (!sw)
                        break;

                    if (g_moving_wnd_seamless_start_move)
                    {
                        qseamless_draw_fake(sw->width, sw->height);
                    }
            }
            else
            {
                sw = sw_get_window_by_wnd(xevent.xexpose.window);
                if (!sw)
                    break;
                XCopyArea(g_display, g_backstore,
                          xevent.xexpose.window, g_gc,
                          xevent.xexpose.x + sw->xoffset,
                          xevent.xexpose.y + sw->yoffset,
                          xevent.xexpose.width,
                          xevent.xexpose.height, xevent.xexpose.x,
                          xevent.xexpose.y);
            }

            break;

        case MappingNotify:
            /* Refresh keyboard mapping if it has changed. This is important for
               Xvnc, since it allocates keycodes dynamically */
            if (xevent.xmapping.request == MappingKeyboard
                    || xevent.xmapping.request == MappingModifier)
                XRefreshKeyboardMapping(&xevent.xmapping);

            if (xevent.xmapping.request == MappingModifier)
            {
                XFreeModifiermap(g_mod_map);
                g_mod_map = XGetModifierMapping(g_display);
            }

            if (xevent.xmapping.request == MappingPointer)
            {
                xwin_refresh_pointer_map();
            }

            break;

            /* clipboard stuff */
        case SelectionNotify:
            xclip_handle_SelectionNotify(&xevent.xselection);
            break;
        case SelectionRequest:
            xclip_handle_SelectionRequest(&xevent.xselectionrequest);
            break;
        case SelectionClear:
            xclip_handle_SelectionClear();
            break;
        case PropertyNotify:
            xclip_handle_PropertyNotify(&xevent.xproperty);
            if (xevent.xproperty.window == g_wnd)
                break;
            if (xevent.xproperty.window == DefaultRootWindow(g_display))
                break;

            /* seamless */
            sw = sw_get_window_by_wnd(xevent.xproperty.window);
            if (!sw)
                break;

            if ((xevent.xproperty.atom == g_net_wm_state_atom)
                    && (xevent.xproperty.state == PropertyNewValue))
            {
                    //sw->state = ewmh_get_window_state(sw->wnd);
                    //seamless_send_state(sw->id, sw->state, 0);
                    qprocess_send_state (sw);
            }

            if ((xevent.xproperty.atom == g_net_wm_desktop_atom)
                    && (xevent.xproperty.state == PropertyNewValue))
            {
                sw->desktop = ewmh_get_window_desktop(sw->wnd);
                    //sw_all_to_desktop(sw->wnd, sw->desktop);
                    qprocess_send_all_to_desktop(sw);
            }

            break;
        case MapNotify:
            if (!g_seamless_active)
                rdp_send_client_window_status(1);
            break;
        case UnmapNotify:
            if (!g_seamless_active)
                rdp_send_client_window_status(0);
            break;
        case ConfigureNotify:
            if (!g_seamless_active)
                break;
            if ( 0 ) {
                sw = sw_get_window_by_wnd(xevent.xconfigure.window);
                if (!sw)
                    break;

                gettimeofday(sw->position_timer, NULL);
                if (sw->position_timer->tv_usec + SEAMLESSRDP_POSITION_TIMER >=
                        1000000)
                {
                    sw->position_timer->tv_usec +=
                        SEAMLESSRDP_POSITION_TIMER - 1000000;
                    sw->position_timer->tv_sec += 1;
                }
                else
                {
                    sw->position_timer->tv_usec += SEAMLESSRDP_POSITION_TIMER;
                }

                sw_handle_restack(sw);
            }
            break;
        }

        xwin_dump_screen();

        //Unlock the display
        XUnlockDisplay( g_display );
    }
    /* Keep going */
    return 1;
}

/* Returns 0 after user quit, 1 otherwise */
int
ui_select(int rdp_socket)
{
    int n;
    fd_set rfds, wfds;
    struct timeval tv;
    RD_BOOL s_timeout = False;

    while (True)
    {
        n = (rdp_socket > g_x_socket) ? rdp_socket : g_x_socket;
        /* Process any events already waiting */
        if (!xwin_process_events())
            /* User quit */
            return 0;

        if (g_seamless_active)
            sw_check_timers();

        FD_ZERO(&rfds);
        FD_ZERO(&wfds);
        FD_SET(rdp_socket, &rfds);
        FD_SET(g_x_socket, &rfds);

        /* default timeout */
        tv.tv_sec = 0;
        tv.tv_usec = 200;

#ifdef WITH_RDPSND
        rdpsnd_add_fds(&n, &rfds, &wfds, &tv);
#endif

        /* add redirection handles */
        rdpdr_add_fds(&n, &rfds, &wfds, &tv, &s_timeout);
        seamless_select_timeout(&tv);

        addin_add_fds(&n, &rfds);

        n++;

        switch (select(n, &rfds, &wfds, NULL, &tv))
        {
        case -1:
            error("select: %s\n", strerror(errno));

        case 0:
#ifdef WITH_RDPSND
            rdpsnd_check_fds(&rfds, &wfds);
#endif
            qaudio_process();

            /* Abort serial read calls */
            if (s_timeout)
                rdpdr_check_fds(&rfds, &wfds, (RD_BOOL) True);
            continue;
        }

#ifdef WITH_RDPSND
        rdpsnd_check_fds(&rfds, &wfds);
#endif
        qaudio_process();

        rdpdr_check_fds(&rfds, &wfds, (RD_BOOL) False);

        addin_check_fds(&rfds);

        if (FD_ISSET(rdp_socket, &rfds))
            return 1;

    }
}

void
ui_move_pointer(int x, int y)
{
    XWarpPointer(g_display, g_wnd, g_wnd, 0, 0, 0, 0, x, y);
}

RD_HBITMAP
ui_create_bitmap(int width, int height, uint8 * data)
{
    XImage *image;
    Pixmap bitmap;
    uint8 *tdata;
    int bitmap_pad;

    if (g_server_depth == 8)
    {
        bitmap_pad = 8;
    }
    else
    {
        bitmap_pad = g_bpp;

        if (g_bpp == 24)
            bitmap_pad = 32;
    }

    tdata = (g_owncolmap ? data : translate_image(width, height, data));
    bitmap = XCreatePixmap(g_display, g_wnd, width, height, g_depth);
    image = XCreateImage(g_display, g_visual, g_depth, ZPixmap, 0,
                         (char *) tdata, width, height, bitmap_pad, 0);

    XPutImage(g_display, bitmap, g_create_bitmap_gc, image, 0, 0, 0, 0, width, height);

    XFree(image);
    if (tdata != data)
        xfree(tdata);
    return (RD_HBITMAP) bitmap;
}

void
ui_pnrdp_paint_bitmap(int x, int y, int cx, int cy, int width, int height, uint8 * data)
{
    XImage *image;
    uint8 *tdata;
    int bitmap_pad;

    if (g_server_depth == 8)
    {
        bitmap_pad = 8;
    }
    else
    {
        bitmap_pad = g_bpp;

        if (g_bpp == 24)
            bitmap_pad = 32;
    }


    tdata = (g_owncolmap ? data : translate_pnrdp_image(width, height, data));
    image = XCreateImage(g_display, g_visual, g_depth, ZPixmap, 0,
                         (char *) tdata, width, height, bitmap_pad, 0);

    if (g_ownbackstore)
    {
        XPutImage(g_display, g_backstore, g_gc, image, 0, 0, x, y, cx, cy);
        XCopyArea(g_display, g_backstore, g_wnd, g_gc, x, y, cx, cy, x, y);
    }
    else
    {
        XPutImage(g_display, g_wnd, g_gc, image, 0, 0, x, y, cx, cy);
    }

    XFree(image);
    if (tdata != data)
        xfree(tdata);
}

void
ui_paint_bitmap(int x, int y, int cx, int cy, int width, int height, uint8 * data)
{
    XImage *image;
    uint8 *tdata;
    int bitmap_pad;

    if (g_server_depth == 8)
    {
        bitmap_pad = 8;
    }
    else
    {
        bitmap_pad = g_bpp;

        if (g_bpp == 24)
            bitmap_pad = 32;
    }

    tdata = (g_owncolmap ? data : translate_image(width, height, data));
    image = XCreateImage(g_display, g_visual, g_depth, ZPixmap, 0,
                         (char *) tdata, width, height, bitmap_pad, 0);

    if (g_ownbackstore)
    {
        XPutImage(g_display, g_backstore, g_gc, image, 0, 0, x, y, cx, cy);
        XCopyArea(g_display, g_backstore, g_wnd, g_gc, x, y, cx, cy, x, y);
        ON_ALL_SEAMLESS_WINDOWS(XCopyArea,
                                (g_display, g_backstore, sw->wnd, g_gc, x, y, cx, cy,
                                 x - sw->xoffset, y - sw->yoffset));
    }
    else
    {
        XPutImage(g_display, g_wnd, g_gc, image, 0, 0, x, y, cx, cy);
        ON_ALL_SEAMLESS_WINDOWS(XCopyArea,
                                (g_display, g_wnd, sw->wnd, g_gc, x, y, cx, cy,
                                 x - sw->xoffset, y - sw->yoffset));
    }

    XFree(image);
    if (tdata != data)
        xfree(tdata);
}

void
ui_destroy_bitmap(RD_HBITMAP bmp)
{
    XFreePixmap(g_display, (Pixmap) bmp);
}

RD_HGLYPH
ui_create_glyph(int width, int height, uint8 * data)
{
    XImage *image;
    Pixmap bitmap;
    int scanline;

    scanline = (width + 7) / 8;

    bitmap = XCreatePixmap(g_display, g_wnd, width, height, 1);
    if (g_create_glyph_gc == 0)
        g_create_glyph_gc = XCreateGC(g_display, bitmap, 0, NULL);

    image = XCreateImage(g_display, g_visual, 1, ZPixmap, 0, (char *) data,
                         width, height, 8, scanline);
    image->byte_order = MSBFirst;
    image->bitmap_bit_order = MSBFirst;
    XInitImage(image);

    XPutImage(g_display, bitmap, g_create_glyph_gc, image, 0, 0, 0, 0, width, height);

    XFree(image);
    return (RD_HGLYPH) bitmap;
}


void
ui_destroy_glyph(RD_HGLYPH glyph)
{
    XFreePixmap(g_display, (Pixmap) glyph);
}

// _CURSOR_FIX
static int
get_next_xor_pixel(uint8 * xormask, int bpp, int * k)
{
    int rv = 0;
    uint8 * s8 = xormask + *k / 8;

    switch (bpp)
    {
    case 1:
        rv = s8[0] & (0x80 >> *k % 8);
        rv = rv ? 0xffffff : 0;
        break;
    case 4:
        rv = s8[0] & (0xf0 >> *k % 8);
        rv = rv ? 0xff000000 : 0xffffffff;
        break;
    case 8:
        rv = s8[0] ? 0xff000000 : 0xffffffff;
        break;
    case 15:
        rv = s8[0] & (s8[1] << 8);
        rv = (rv & 0x1f) | (rv & 0x3e0) << 3 | (rv & 0x7c00) << 6;
        rv |= 0xff000000;
        break;
    case 16:
        rv = s8[0] & (s8[1] << 8);
        rv = (rv & 0x1f) | (rv & 0x730) << 3 | (rv & 0xf800) << 5;
        rv |= 0xff000000;
        break;
    case 24:
        rv = s8[0] | s8[1] << 8 | s8[2] << 16 | 0xff000000;
        break;
    case 32:
        rv = s8[0] | s8[1] << 8 | s8[2] << 16 | s8[3] << 24;
        break;
    default:
        error("unknown bpp in get_next_xor_pixel %d\n", bpp);
    }

    *k += (bpp == 15 ? 16 : bpp);

    return rv;
}

int get_andbit(uint8 *mask, int * k)
{
    int rv = mask[*k / 8] & (0x80 >> (*k % 8));
    ++*k;
    return rv;
}


RD_HCURSOR
ui_create_cursor(unsigned int x, unsigned int y, int width, int height,
                 uint8 * andmask, uint8 * xormask, int bpp)
{
    XcursorImage *image;
    int pixel;
    XcursorPixel *pixelp;
    Cursor c;
    int i, j, k, l;
    int andmaskpadbits = (16 - width % 16) % 16;
    int xormaskpadbits;

    if (bpp == 15 || bpp == 16)
        xormaskpadbits = 0;
    else
        xormaskpadbits = (16 - width * bpp % 16) % 16;

    exit_if_null(image = XcursorImageCreate(width, height));
    image->xhot = x;
    image->yhot = y;

    k = l = 0;

    syslog( LOG_NOTICE, "Creating cursor" );

    /* Start at the last scanline and work up; image is flipped */
    for (i = 0; i < height; ++i)
    {
        pixelp = image->pixels + width * (bpp == 1 ? i : height - i - 1);
        for (j = 0; j < width; ++j)
        {
            pixel = get_next_xor_pixel(xormask, bpp, &k);

            if (bpp == 1)
                if (get_andbit(andmask, &l))
                    *(pixelp++) = pixel ? 0xff000000 : 0;
                else
                    *(pixelp++) = pixel | 0xff000000;
            else if (bpp == 24 && get_andbit(andmask, &l))
                *(pixelp++) = 0;
            else
                *(pixelp++) = pixel;

        }
        k += xormaskpadbits;
        l += andmaskpadbits;
    }

    c = XcursorImageLoadCursor(g_display, image);
    XcursorImageDestroy(image);
    return (RD_HCURSOR) c;

}

void
ui_set_cursor(RD_HCURSOR cursor)
{
    g_current_cursor = (Cursor) cursor;
    XDefineCursor(g_display, g_wnd, g_current_cursor);
    ON_ALL_SEAMLESS_WINDOWS(XDefineCursor, (g_display, sw->wnd, g_current_cursor));
}

void
ui_destroy_cursor(RD_HCURSOR cursor)
{
    XFreeCursor(g_display, (Cursor) cursor);
}

void
ui_set_null_cursor(void)
{
    ui_set_cursor(g_null_cursor);
}

#define MAKE_XCOLOR(xc,c) \
        (xc)->red   = ((c)->red   << 8) | (c)->red; \
        (xc)->green = ((c)->green << 8) | (c)->green; \
        (xc)->blue  = ((c)->blue  << 8) | (c)->blue; \
        (xc)->flags = DoRed | DoGreen | DoBlue;


RD_HCOLOURMAP
ui_create_colourmap(COLOURMAP * colours)
{
    COLOURENTRY *entry;
    int i, ncolours = colours->ncolours;
    if (!g_owncolmap)
    {
        uint32 *map = (uint32 *) xmalloc(sizeof(*g_colmap) * ncolours);
        XColor xentry;
        XColor xc_cache[256];
        uint32 colour;
        int colLookup = 256;
        for (i = 0; i < ncolours; i++)
        {
            entry = &colours->colours[i];
            MAKE_XCOLOR(&xentry, entry);

            if (XAllocColor(g_display, g_xcolmap, &xentry) == 0)
            {
                /* Allocation failed, find closest match. */
                int j = 256;
                int nMinDist = 3 * 256 * 256;
                long nDist = nMinDist;

                /* only get the colors once */
                while (colLookup--)
                {
                    xc_cache[colLookup].pixel = colLookup;
                    xc_cache[colLookup].red = xc_cache[colLookup].green =
                                                  xc_cache[colLookup].blue = 0;
                    xc_cache[colLookup].flags = 0;
                    XQueryColor(g_display,
                                DefaultColormap(g_display,
                                                DefaultScreen(g_display)),
                                &xc_cache[colLookup]);
                }
                colLookup = 0;

                /* approximate the pixel */
                while (j--)
                {
                    if (xc_cache[j].flags)
                    {
                        nDist = ((long) (xc_cache[j].red >> 8) -
                                 (long) (xentry.red >> 8)) *
                                ((long) (xc_cache[j].red >> 8) -
                                 (long) (xentry.red >> 8)) +
                                ((long) (xc_cache[j].green >> 8) -
                                 (long) (xentry.green >> 8)) *
                                ((long) (xc_cache[j].green >> 8) -
                                 (long) (xentry.green >> 8)) +
                                ((long) (xc_cache[j].blue >> 8) -
                                 (long) (xentry.blue >> 8)) *
                                ((long) (xc_cache[j].blue >> 8) -
                                 (long) (xentry.blue >> 8));
                    }
                    if (nDist < nMinDist)
                    {
                        nMinDist = nDist;
                        xentry.pixel = j;
                    }
                }
            }
            colour = xentry.pixel;

            /* update our cache */
            if (xentry.pixel < 256)
            {
                xc_cache[xentry.pixel].red = xentry.red;
                xc_cache[xentry.pixel].green = xentry.green;
                xc_cache[xentry.pixel].blue = xentry.blue;

            }

            map[i] = colour;
        }
        return map;
    }
    else
    {
        XColor *xcolours, *xentry;
        Colormap map;

        xcolours = (XColor *) xmalloc(sizeof(XColor) * ncolours);
        for (i = 0; i < ncolours; i++)
        {
            entry = &colours->colours[i];
            xentry = &xcolours[i];
            xentry->pixel = i;
            MAKE_XCOLOR(xentry, entry);
        }

        map = XCreateColormap(g_display, g_wnd, g_visual, AllocAll);
        XStoreColors(g_display, map, xcolours, ncolours);

        xfree(xcolours);
        return (RD_HCOLOURMAP) map;
    }
}

void
ui_destroy_colourmap(RD_HCOLOURMAP map)
{
    if (!g_owncolmap)
        xfree(map);
    else
        XFreeColormap(g_display, (Colormap) map);
}

void
ui_set_colourmap(RD_HCOLOURMAP map)
{
    if (!g_owncolmap)
    {
        if (g_colmap)
            xfree(g_colmap);

        g_colmap = (uint32 *) map;
    }
    else
    {
        XSetWindowColormap(g_display, g_wnd, (Colormap) map);
        ON_ALL_SEAMLESS_WINDOWS(XSetWindowColormap, (g_display, sw->wnd, (Colormap) map));
    }
}

void
ui_set_clip(int x, int y, int cx, int cy)
{
    g_clip_rectangle.x = x;
    g_clip_rectangle.y = y;
    g_clip_rectangle.width = cx;
    g_clip_rectangle.height = cy;
    XSetClipRectangles(g_display, g_gc, 0, 0, &g_clip_rectangle, 1, YXBanded);
}

void
ui_reset_clip(void)
{
    g_clip_rectangle.x = 0;
    g_clip_rectangle.y = 0;
    g_clip_rectangle.width = g_width;
    g_clip_rectangle.height = g_height;
    XSetClipRectangles(g_display, g_gc, 0, 0, &g_clip_rectangle, 1, YXBanded);
}

void
ui_bell(void)
{
    XBell(g_display, 0);
}

void
ui_destblt(uint8 opcode,
           /* dest */ int x, int y, int cx, int cy)
{
    SET_FUNCTION(opcode);
    FILL_RECTANGLE(x, y, cx, cy);
    RESET_FUNCTION(opcode);
}

static uint8 hatch_patterns[] =
{
    0x00, 0x00, 0x00, 0xff, 0x00, 0x00, 0x00, 0x00,    /* 0 - bsHorizontal */
    0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08, 0x08,    /* 1 - bsVertical */
    0x80, 0x40, 0x20, 0x10, 0x08, 0x04, 0x02, 0x01,    /* 2 - bsFDiagonal */
    0x01, 0x02, 0x04, 0x08, 0x10, 0x20, 0x40, 0x80,    /* 3 - bsBDiagonal */
    0x08, 0x08, 0x08, 0xff, 0x08, 0x08, 0x08, 0x08,    /* 4 - bsCross */
    0x81, 0x42, 0x24, 0x18, 0x18, 0x24, 0x42, 0x81    /* 5 - bsDiagCross */
};

void
ui_patblt(uint8 opcode,
          /* dest */ int x, int y, int cx, int cy,
          /* brush */ BRUSH * brush, int bgcolour, int fgcolour)
{
    Pixmap fill;
    uint8 i, ipattern[8];

    SET_FUNCTION(opcode);

    switch (brush->style)
    {
    case 0:    /* Solid */
        SET_FOREGROUND(fgcolour);
        FILL_RECTANGLE_BACKSTORE(x, y, cx, cy);
        break;

    case 2:    /* Hatch */
        fill = (Pixmap) ui_create_glyph(8, 8,
                                        hatch_patterns + brush->pattern[0] * 8);
        SET_FOREGROUND(fgcolour);
        SET_BACKGROUND(bgcolour);
        XSetFillStyle(g_display, g_gc, FillOpaqueStippled);
        XSetStipple(g_display, g_gc, fill);
        XSetTSOrigin(g_display, g_gc, brush->xorigin, brush->yorigin);
        FILL_RECTANGLE_BACKSTORE(x, y, cx, cy);
        XSetFillStyle(g_display, g_gc, FillSolid);
        XSetTSOrigin(g_display, g_gc, 0, 0);
        ui_destroy_glyph((RD_HGLYPH) fill);
        break;

    case 3:    /* Pattern */
        for (i = 0; i != 8; i++)
            ipattern[7 - i] = brush->pattern[i];
        fill = (Pixmap) ui_create_glyph(8, 8, ipattern);
        SET_FOREGROUND(bgcolour);
        SET_BACKGROUND(fgcolour);
        XSetFillStyle(g_display, g_gc, FillOpaqueStippled);
        XSetStipple(g_display, g_gc, fill);
        XSetTSOrigin(g_display, g_gc, brush->xorigin, brush->yorigin);
        FILL_RECTANGLE_BACKSTORE(x, y, cx, cy);
        XSetFillStyle(g_display, g_gc, FillSolid);
        XSetTSOrigin(g_display, g_gc, 0, 0);
        ui_destroy_glyph((RD_HGLYPH) fill);
        break;

    default:
        unimpl("brush %d\n", brush->style);
    }

    RESET_FUNCTION(opcode);

    if (g_ownbackstore)
        XCopyArea(g_display, g_backstore, g_wnd, g_gc, x, y, cx, cy, x, y);
    ON_ALL_SEAMLESS_WINDOWS(XCopyArea,
                            (g_display, g_ownbackstore ? g_backstore : g_wnd, sw->wnd, g_gc,
                             x, y, cx, cy, x - sw->xoffset, y - sw->yoffset));
}

void
ui_screenblt(uint8 opcode,
             /* dest */ int x, int y, int cx, int cy,
             /* src */ int srcx, int srcy)
{
    SET_FUNCTION(opcode);
    if (g_ownbackstore)
    {
        XCopyArea(g_display, /*g_Unobscured ? g_wnd : */g_backstore,
                  g_wnd, g_gc, srcx, srcy, cx, cy, x, y);
        XCopyArea(g_display, g_backstore, g_backstore, g_gc, srcx, srcy, cx, cy, x, y);

        // Request a screendump for the updated, but it may not dump if too fast
        //  xwin_dump_screen();  commented because of performance issues

    }
    else
    {
        XCopyArea(g_display, g_wnd, g_wnd, g_gc, srcx, srcy, cx, cy, x, y);
    }

    ON_ALL_SEAMLESS_WINDOWS(XCopyArea,
                            (g_display, g_ownbackstore ? g_backstore : g_wnd,
                             sw->wnd, g_gc, x, y, cx, cy, x - sw->xoffset, y - sw->yoffset));

    RESET_FUNCTION(opcode);
}

void
ui_memblt(uint8 opcode,
          /* dest */ int x, int y, int cx, int cy,
          /* src */ RD_HBITMAP src, int srcx, int srcy)
{
    SET_FUNCTION(opcode);
    XCopyArea(g_display, (Pixmap) src, g_wnd, g_gc, srcx, srcy, cx, cy, x, y);
    ON_ALL_SEAMLESS_WINDOWS(XCopyArea,
                            (g_display, (Pixmap) src, sw->wnd, g_gc,
                             srcx, srcy, cx, cy, x - sw->xoffset, y - sw->yoffset));
    if (g_ownbackstore)
    {
        XCopyArea(g_display, (Pixmap) src, g_backstore, g_gc, srcx, srcy, cx, cy, x, y);
        XCopyArea(g_display, g_backstore, g_backstore2, g_gc, 0, 0, g_width, g_height, 0, 0);
        // syslog( LOG_INFO, "Updated backstore" );

        // Request a screendump for the updated, but it may not dump if too fast
        //  xwin_dump_screen();  commented because of performance issues
    }
    RESET_FUNCTION(opcode);
}

void
ui_triblt(uint8 opcode,
          /* dest */ int x, int y, int cx, int cy,
          /* src */ RD_HBITMAP src, int srcx, int srcy,
          /* brush */ BRUSH * brush, int bgcolour, int fgcolour)
{
    /* This is potentially difficult to do in general. Until someone
       comes up with a more efficient way of doing it I am using cases. */

    switch (opcode)
    {
    case 0x69:    /* PDSxxn */
        ui_memblt(ROP2_XOR, x, y, cx, cy, src, srcx, srcy);
        ui_patblt(ROP2_NXOR, x, y, cx, cy, brush, bgcolour, fgcolour);
        break;

    case 0xb8:    /* PSDPxax */
        ui_patblt(ROP2_XOR, x, y, cx, cy, brush, bgcolour, fgcolour);
        ui_memblt(ROP2_AND, x, y, cx, cy, src, srcx, srcy);
        ui_patblt(ROP2_XOR, x, y, cx, cy, brush, bgcolour, fgcolour);
        break;

    case 0xc0:    /* PSa */
        ui_memblt(ROP2_COPY, x, y, cx, cy, src, srcx, srcy);
        ui_patblt(ROP2_AND, x, y, cx, cy, brush, bgcolour, fgcolour);
        break;

    default:
        unimpl("triblt 0x%x\n", opcode);
        ui_memblt(ROP2_COPY, x, y, cx, cy, src, srcx, srcy);
    }
}

void
ui_line(uint8 opcode,
        /* dest */ int startx, int starty, int endx, int endy,
        /* pen */ PEN * pen)
{
    SET_FUNCTION(opcode);
    SET_FOREGROUND(pen->colour);
    XDrawLine(g_display, g_wnd, g_gc, startx, starty, endx, endy);
    ON_ALL_SEAMLESS_WINDOWS(XDrawLine, (g_display, sw->wnd, g_gc,
                                        startx - sw->xoffset, starty - sw->yoffset,
                                        endx - sw->xoffset, endy - sw->yoffset));
    if (g_ownbackstore)
        XDrawLine(g_display, g_backstore, g_gc, startx, starty, endx, endy);
    RESET_FUNCTION(opcode);
}

void
ui_rect(
    /* dest */ int x, int y, int cx, int cy,
    /* brush */ int colour)
{
    SET_FOREGROUND(colour);
    FILL_RECTANGLE(x, y, cx, cy);
}

void
ui_polygon(uint8 opcode,
           /* mode */ uint8 fillmode,
           /* dest */ RD_POINT * point, int npoints,
           /* brush */ BRUSH * brush, int bgcolour, int fgcolour)
{
    uint8 style, i, ipattern[8];
    Pixmap fill;

    SET_FUNCTION(opcode);

    switch (fillmode)
    {
    case ALTERNATE:
        XSetFillRule(g_display, g_gc, EvenOddRule);
        break;
    case WINDING:
        XSetFillRule(g_display, g_gc, WindingRule);
        break;
    default:
        unimpl("fill mode %d\n", fillmode);
    }

    if (brush)
        style = brush->style;
    else
        style = 0;

    switch (style)
    {
    case 0:    /* Solid */
        SET_FOREGROUND(fgcolour);
        FILL_POLYGON((XPoint *) point, npoints);
        break;

    case 2:    /* Hatch */
        fill = (Pixmap) ui_create_glyph(8, 8,
                                        hatch_patterns + brush->pattern[0] * 8);
        SET_FOREGROUND(fgcolour);
        SET_BACKGROUND(bgcolour);
        XSetFillStyle(g_display, g_gc, FillOpaqueStippled);
        XSetStipple(g_display, g_gc, fill);
        XSetTSOrigin(g_display, g_gc, brush->xorigin, brush->yorigin);
        FILL_POLYGON((XPoint *) point, npoints);
        XSetFillStyle(g_display, g_gc, FillSolid);
        XSetTSOrigin(g_display, g_gc, 0, 0);
        ui_destroy_glyph((RD_HGLYPH) fill);
        break;

    case 3:    /* Pattern */
        for (i = 0; i != 8; i++)
            ipattern[7 - i] = brush->pattern[i];
        fill = (Pixmap) ui_create_glyph(8, 8, ipattern);
        SET_FOREGROUND(bgcolour);
        SET_BACKGROUND(fgcolour);
        XSetFillStyle(g_display, g_gc, FillOpaqueStippled);
        XSetStipple(g_display, g_gc, fill);
        XSetTSOrigin(g_display, g_gc, brush->xorigin, brush->yorigin);
        FILL_POLYGON((XPoint *) point, npoints);
        XSetFillStyle(g_display, g_gc, FillSolid);
        XSetTSOrigin(g_display, g_gc, 0, 0);
        ui_destroy_glyph((RD_HGLYPH) fill);
        break;

    default:
        unimpl("brush %d\n", brush->style);
    }

    RESET_FUNCTION(opcode);
}

void
ui_polyline(uint8 opcode,
            /* dest */ RD_POINT * points, int npoints,
            /* pen */ PEN * pen)
{
    /* TODO: set join style */
    SET_FUNCTION(opcode);
    SET_FOREGROUND(pen->colour);
    XDrawLines(g_display, g_wnd, g_gc, (XPoint *) points, npoints, CoordModePrevious);
    if (g_ownbackstore)
        XDrawLines(g_display, g_backstore, g_gc, (XPoint *) points, npoints,
                   CoordModePrevious);

    ON_ALL_SEAMLESS_WINDOWS(seamless_XDrawLines,
                            (sw->wnd, (XPoint *) points, npoints, sw->xoffset, sw->yoffset));

    RESET_FUNCTION(opcode);
}

void
ui_ellipse(uint8 opcode,
           /* mode */ uint8 fillmode,
           /* dest */ int x, int y, int cx, int cy,
           /* brush */ BRUSH * brush, int bgcolour, int fgcolour)
{
    uint8 style, i, ipattern[8];
    Pixmap fill;

    SET_FUNCTION(opcode);

    if (brush)
        style = brush->style;
    else
        style = 0;

    switch (style)
    {
    case 0:    /* Solid */
        SET_FOREGROUND(fgcolour);
        DRAW_ELLIPSE(x, y, cx, cy, fillmode);
        break;

    case 2:    /* Hatch */
        fill = (Pixmap) ui_create_glyph(8, 8,
                                        hatch_patterns + brush->pattern[0] * 8);
        SET_FOREGROUND(fgcolour);
        SET_BACKGROUND(bgcolour);
        XSetFillStyle(g_display, g_gc, FillOpaqueStippled);
        XSetStipple(g_display, g_gc, fill);
        XSetTSOrigin(g_display, g_gc, brush->xorigin, brush->yorigin);
        DRAW_ELLIPSE(x, y, cx, cy, fillmode);
        XSetFillStyle(g_display, g_gc, FillSolid);
        XSetTSOrigin(g_display, g_gc, 0, 0);
        ui_destroy_glyph((RD_HGLYPH) fill);
        break;

    case 3:    /* Pattern */
        for (i = 0; i != 8; i++)
            ipattern[7 - i] = brush->pattern[i];
        fill = (Pixmap) ui_create_glyph(8, 8, ipattern);
        SET_FOREGROUND(bgcolour);
        SET_BACKGROUND(fgcolour);
        XSetFillStyle(g_display, g_gc, FillOpaqueStippled);
        XSetStipple(g_display, g_gc, fill);
        XSetTSOrigin(g_display, g_gc, brush->xorigin, brush->yorigin);
        DRAW_ELLIPSE(x, y, cx, cy, fillmode);
        XSetFillStyle(g_display, g_gc, FillSolid);
        XSetTSOrigin(g_display, g_gc, 0, 0);
        ui_destroy_glyph((RD_HGLYPH) fill);
        break;

    default:
        unimpl("brush %d\n", brush->style);
    }

    RESET_FUNCTION(opcode);
}

/* warning, this function only draws on wnd or backstore, not both */
void
ui_draw_glyph(int mixmode,
              /* dest */ int x, int y, int cx, int cy,
              /* src */ RD_HGLYPH glyph, int srcx, int srcy,
              int bgcolour, int fgcolour)
{
    SET_FOREGROUND(fgcolour);
    SET_BACKGROUND(bgcolour);

    XSetFillStyle(g_display, g_gc,
                  (mixmode == MIX_TRANSPARENT) ? FillStippled : FillOpaqueStippled);
    XSetStipple(g_display, g_gc, (Pixmap) glyph);
    XSetTSOrigin(g_display, g_gc, x, y);

    FILL_RECTANGLE_BACKSTORE(x, y, cx, cy);

    XSetFillStyle(g_display, g_gc, FillSolid);
}

#define DO_GLYPH(ttext,idx) \
{\
  glyph = cache_get_font (font, ttext[idx]);\
  if (!(flags & TEXT2_IMPLICIT_X))\
  {\
    xyoffset = ttext[++idx];\
    if ((xyoffset & 0x80))\
    {\
      if (flags & TEXT2_VERTICAL)\
        y += ttext[idx+1] | (ttext[idx+2] << 8);\
      else\
        x += ttext[idx+1] | (ttext[idx+2] << 8);\
      idx += 2;\
    }\
    else\
    {\
      if (flags & TEXT2_VERTICAL)\
        y += xyoffset;\
      else\
        x += xyoffset;\
    }\
  }\
  if (glyph != NULL)\
  {\
    x1 = x + glyph->offset;\
    y1 = y + glyph->baseline;\
    XSetStipple(g_display, g_gc, (Pixmap) glyph->pixmap);\
    XSetTSOrigin(g_display, g_gc, x1, y1);\
    FILL_RECTANGLE_BACKSTORE(x1, y1, glyph->width, glyph->height);\
    if (flags & TEXT2_IMPLICIT_X)\
      x += glyph->width;\
  }\
}

void
ui_draw_text(uint8 font, uint8 flags, uint8 opcode, int mixmode, int x, int y,
             int clipx, int clipy, int clipcx, int clipcy,
             int boxx, int boxy, int boxcx, int boxcy, BRUSH * brush,
             int bgcolour, int fgcolour, uint8 * text, uint8 length)
{
    /* TODO: use brush appropriately */

    FONTGLYPH *glyph;
    int i, j, xyoffset, x1, y1;
    DATABLOB *entry;

    SET_FOREGROUND(bgcolour);

    /* Sometimes, the boxcx value is something really large, like
       32691. This makes XCopyArea fail with Xvnc. The code below
       is a quick fix. */
    if (boxx + boxcx > g_width)
        boxcx = g_width - boxx;

    if (boxcx > 1)
    {
        FILL_RECTANGLE_BACKSTORE(boxx, boxy, boxcx, boxcy);
    }
    else if (mixmode == MIX_OPAQUE)
    {
        FILL_RECTANGLE_BACKSTORE(clipx, clipy, clipcx, clipcy);
    }

    SET_FOREGROUND(fgcolour);
    SET_BACKGROUND(bgcolour);
    XSetFillStyle(g_display, g_gc, FillStippled);

    /* Paint text, character by character */
    for (i = 0; i < length;)
    {
        switch (text[i])
        {
        case 0xff:
            /* At least two bytes needs to follow */
            if (i + 3 > length)
            {
                warning("Skipping short 0xff command:");
                for (j = 0; j < length; j++)
                    fprintf(stderr, "%02x ", text[j]);
                fprintf(stderr, "\n");
                i = length = 0;
                break;
            }
            cache_put_text(text[i + 1], text, text[i + 2]);
            i += 3;
            length -= i;
            /* this will move pointer from start to first character after FF command */
            text = &(text[i]);
            i = 0;
            break;

        case 0xfe:
            /* At least one byte needs to follow */
            if (i + 2 > length)
            {
                warning("Skipping short 0xfe command:");
                for (j = 0; j < length; j++)
                    fprintf(stderr, "%02x ", text[j]);
                fprintf(stderr, "\n");
                i = length = 0;
                break;
            }
            entry = cache_get_text(text[i + 1]);
            if (entry->data != NULL)
            {
                if ((((uint8 *) (entry->data))[1] == 0)
                        && (!(flags & TEXT2_IMPLICIT_X)) && (i + 2 < length))
                {
                    if (flags & TEXT2_VERTICAL)
                        y += text[i + 2];
                    else
                        x += text[i + 2];
                }
                for (j = 0; j < entry->size; j++)
                    DO_GLYPH(((uint8 *) (entry->data)), j);
            }
            if (i + 2 < length)
                i += 3;
            else
                i += 2;
            length -= i;
            /* this will move pointer from start to first character after FE command */
            text = &(text[i]);
            i = 0;
            break;

        default:
            DO_GLYPH(text, i);
            i++;
            break;
        }
    }

    XSetFillStyle(g_display, g_gc, FillSolid);

    if (g_ownbackstore)
    {
        if (boxcx > 1)
        {
            XCopyArea(g_display, g_backstore, g_wnd, g_gc, boxx,
                      boxy, boxcx, boxcy, boxx, boxy);
            ON_ALL_SEAMLESS_WINDOWS(XCopyArea,
                                    (g_display, g_backstore, sw->wnd, g_gc,
                                     boxx, boxy,
                                     boxcx, boxcy,
                                     boxx - sw->xoffset, boxy - sw->yoffset));
        }
        else
        {
            XCopyArea(g_display, g_backstore, g_wnd, g_gc, clipx,
                      clipy, clipcx, clipcy, clipx, clipy);
            ON_ALL_SEAMLESS_WINDOWS(XCopyArea,
                                    (g_display, g_backstore, sw->wnd, g_gc,
                                     clipx, clipy,
                                     clipcx, clipcy, clipx - sw->xoffset,
                                     clipy - sw->yoffset));
        }
    }
}

void
ui_desktop_save(uint32 offset, int x, int y, int cx, int cy)
{
    Pixmap pix;
    XImage *image;

    if (g_ownbackstore)
    {

        image = XGetImage(g_display, g_backstore, x, y, cx, cy, AllPlanes, ZPixmap);
        exit_if_null(image);
    }
    else
    {
        pix = XCreatePixmap(g_display, g_wnd, cx, cy, g_depth);
        XCopyArea(g_display, g_wnd, pix, g_gc, x, y, cx, cy, 0, 0);
        image = XGetImage(g_display, pix, 0, 0, cx, cy, AllPlanes, ZPixmap);
        exit_if_null(image);
        XFreePixmap(g_display, pix);
    }

    offset *= g_bpp / 8;
    cache_put_desktop(offset, cx, cy, image->bytes_per_line, g_bpp / 8, (uint8 *) image->data);

    XDestroyImage(image);

}


void xwin_dump_screen( void )
{
        XImage *ximage;
        static struct timeval time0;
        struct timeval time1;

        // Figure out how long since the last screen update
        // If shorter than the target interval, then just return
        gettimeofday( &time1, NULL );
        if ( difftime( time1.tv_sec, time0.tv_sec ) < QTHUMBNAIL_SCREENSHOT_SECS )
        {
            return;
        }

        // syslog( LOG_INFO, "Okay to update the thumbnail Time now %i, last %i", time1.tv_sec, time0.tv_sec );
        time0 = time1;

        // syslog( LOG_INFO, "Before XGetImage.");
        ximage = XGetImage(g_display, g_backstore2, 0, 0, g_width, g_height, AllPlanes, ZPixmap);
        // XInitImage( ximage );
        // syslog( LOG_INFO, "After XGetImage.");

        // Do something with the image
        // If the image is empty, don't send it.
        if ( (ximage->height != 0) && (ximage->width != 0) ) au_dump_screenshot( ximage );

        XDestroyImage( ximage );

        return;
}

void
ui_desktop_restore(uint32 offset, int x, int y, int cx, int cy)
{
    XImage *image;
    uint8 *data;

    offset *= g_bpp / 8;
    data = cache_get_desktop(offset, cx, cy, g_bpp / 8);
    if (data == NULL)
        return;

    image = XCreateImage(g_display, g_visual, g_depth, ZPixmap, 0,
                         (char *) data, cx, cy, g_bpp, 0);

    if (g_ownbackstore)
    {
        XPutImage(g_display, g_backstore, g_gc, image, 0, 0, x, y, cx, cy);
        XCopyArea(g_display, g_backstore, g_wnd, g_gc, x, y, cx, cy, x, y);
        ON_ALL_SEAMLESS_WINDOWS(XCopyArea,
                                (g_display, g_backstore, sw->wnd, g_gc,
                                 x, y, cx, cy, x - sw->xoffset, y - sw->yoffset));
    }
    else
    {
        XPutImage(g_display, g_wnd, g_gc, image, 0, 0, x, y, cx, cy);
        ON_ALL_SEAMLESS_WINDOWS(XCopyArea,
                                (g_display, g_wnd, sw->wnd, g_gc, x, y, cx, cy,
                                 x - sw->xoffset, y - sw->yoffset));
    }

    XFree(image);
}

/* these do nothing here but are used in uiports */
void
ui_begin_update(void)
{
}

void
ui_end_update(void)
{
}


void
ui_seamless_begin(RD_BOOL hidden)
{
    if (!g_seamless_rdp)
        return;

    if (g_seamless_started)
        return;

    g_seamless_started = True;
    g_seamless_hidden = hidden;

    if (!hidden)
        ui_seamless_toggle();
}


void
ui_seamless_end()
{
    /* Destroy all seamless windows */
    while (g_seamless_windows)
    {
        XDestroyWindow(g_display, g_seamless_windows->wnd);
        sw_remove_window(g_seamless_windows);
    }

    g_seamless_started = False;
    g_seamless_active = False;
    g_seamless_hidden = False;
}


void
ui_seamless_hide_desktop()
{
    if (!g_seamless_rdp)
        return;

    if (!g_seamless_started)
        return;

    if (g_seamless_active)
        ui_seamless_toggle();

    g_seamless_hidden = True;
}


void
ui_seamless_unhide_desktop()
{
    if (!g_seamless_rdp)
        return;

    if (!g_seamless_started)
        return;

    g_seamless_hidden = False;

    ui_seamless_toggle();
}


void
ui_seamless_toggle()
{
    if (!g_seamless_rdp)
        return;

    if (!g_seamless_started)
        return;

    if (g_seamless_hidden)
        return;

    if (g_seamless_active)
    {
        /* Deactivate */
        while (g_seamless_windows)
        {
            qseamless_send_message_WA_INACTIVE(g_seamless_windows->id);
            XDestroyWindow(g_display, g_seamless_windows->wnd);
            sw_remove_window(g_seamless_windows);
        }
        pnts_send_message ("DM0");  // fullscreen=0, seamless=1

        XMapWindow(g_display, g_wnd);
    }
    else
    {
        /* Activate */
        XUnmapWindow(g_display, g_wnd);
        //seamless_send_sync();
        pnts_send_message ("DM1");  // fullscreen=0, seamless=1
    }

    g_seamless_active = !g_seamless_active;
}


void
ui_seamless_create_window(unsigned long id, unsigned long group, unsigned long parent,
                          unsigned long flags)
{
    Window wnd;
    XSetWindowAttributes attribs;
    XClassHint *classhints;
    XSizeHints *sizehints;
    XWMHints *wmhints;
    long input_mask;
    seamless_window *sw, *sw_parent;

    if (!g_seamless_active)
        return;

    /* Ignore CREATEs for existing windows */
    sw = sw_get_window_by_id(id);
    if (sw)
        return;

    get_window_attribs(&attribs);
    wnd = XCreateWindow(g_display, RootWindowOfScreen(g_screen), -1, -1, 1, 1, 0, g_depth,
                        InputOutput, g_visual,
                        CWBackPixel | CWBackingStore | CWColormap | CWBorderPixel, &attribs);

    XStoreName(g_display, wnd, "SeamlessRDP");
    ewmh_set_wm_name(wnd, "SeamlessRDP");

    mwm_hide_decorations(wnd);

    classhints = XAllocClassHint();
    if (classhints != NULL)
    {
        classhints->res_name = "rdesktop";
        classhints->res_class = "SeamlessRDP";
        XSetClassHint(g_display, wnd, classhints);
        XFree(classhints);
    }

    /* WM_NORMAL_HINTS */
    sizehints = XAllocSizeHints();
    if (sizehints != NULL)
    {
        sizehints->flags = USPosition;
        XSetWMNormalHints(g_display, wnd, sizehints);
        XFree(sizehints);
    }

    /* Parent-less transient windows */
    if (parent == 0xFFFFFFFF)
    {
        XSetTransientForHint(g_display, wnd, RootWindowOfScreen(g_screen));
        /* Some buggy wm:s (kwin) do not handle the above, so fake it
           using some other hints. */
        ewmh_set_window_popup(wnd);
    }
    /* Normal transient windows */
    else if (parent != 0x00000000)
    {
        sw_parent = sw_get_window_by_id(parent);
        if (sw_parent)
            XSetTransientForHint(g_display, wnd, sw_parent->wnd);
        else
            warning("ui_seamless_create_window: No parent window 0x%lx\n", parent);
    }

    if (flags & SEAMLESSRDP_CREATE_MODAL)
    {
        /* We do this to support buggy wm:s (*cough* metacity *cough*)
           somewhat at least */
        if (parent == 0x00000000)
            XSetTransientForHint(g_display, wnd, RootWindowOfScreen(g_screen));
        ewmh_set_window_modal(wnd);
    }

    if (flags & SEAMLESSRDP_CREATE_TOPMOST)
    {
        /* Make window always-on-top */
        ewmh_set_window_above(wnd);
    }

    /* FIXME: Support for Input Context:s */

    get_input_mask(&input_mask);
    input_mask |= PropertyChangeMask;

    XSelectInput(g_display, wnd, input_mask);

    /* handle the WM_DELETE_WINDOW protocol. */
    XSetWMProtocols(g_display, wnd, &g_kill_atom, 1);

    sw = xmalloc(sizeof(seamless_window));

    memset(sw, 0, sizeof(seamless_window));

    sw->wnd = wnd;
    sw->id = id;
    sw->group = sw_find_group(group, False);
    sw->group->refcnt++;
    sw->state = SEAMLESSRDP_NOTYETMAPPED;
    sw->desktop = 0;
    sw->position_timer = xmalloc(sizeof(struct timeval));
    timerclear(sw->position_timer);

    sw->outstanding_position = False;
    sw->outpos_serial = 0;
    sw->outpos_xoffset = sw->outpos_yoffset = 0;
    sw->outpos_width = sw->outpos_height = 0;

    sw->next = g_seamless_windows;
    g_seamless_windows = sw;

    /* WM_HINTS */
    wmhints = XAllocWMHints();
    if (wmhints)
    {
        wmhints->flags = WindowGroupHint;
        wmhints->window_group = sw->group->wnd;
        XSetWMHints(g_display, sw->wnd, wmhints);
        XFree(wmhints);
    }

    XChangeProperty(g_display, sw->wnd, g_net_wm_actions_atom, XA_ATOM, 32,
            PropModeReplace, (unsigned char *)g_allowed_actions,
            Q_ALLOWED_ACTIONS_SIZE);
}


void
ui_seamless_destroy_window(unsigned long id, unsigned long flags)
{
    seamless_window *sw;

    if (!g_seamless_active)
        return;

    sw = sw_get_window_by_id(id);
    if (!sw)
    {
        warning("ui_seamless_destroy_window: No information for window 0x%lx\n", id);
        return;
    }

    XDestroyWindow(g_display, sw->wnd);
    sw_remove_window(sw);
}


void
ui_seamless_destroy_group(unsigned long id, unsigned long flags)
{
    seamless_window *sw, *sw_next;

    if (!g_seamless_active)
        return;

    for (sw = g_seamless_windows; sw; sw = sw_next)
    {
        sw_next = sw->next;

        if (sw->group->id == id)
        {
            XDestroyWindow(g_display, sw->wnd);
            sw_remove_window(sw);
        }
    }
}


void
ui_seamless_seticon(unsigned long id, const char *format, int width, int height, int chunk,
                    const char *data, int chunk_len)
{
    seamless_window *sw;

    if (!g_seamless_active)
        return;

    sw = sw_get_window_by_id(id);
    if (!sw)
    {
        warning("ui_seamless_seticon: No information for window 0x%lx\n", id);
        return;
    }

    if (chunk == 0)
    {
        if (sw->icon_size)
            warning("ui_seamless_seticon: New icon started before previous completed\n");

        if (strcmp(format, "RGBA") != 0)
        {
            warning("ui_seamless_seticon: Uknown icon format \"%s\"\n", format);
            return;
        }

        sw->icon_size = width * height * 4;
        if (sw->icon_size > 32 * 32 * 4)
        {
            warning("ui_seamless_seticon: Icon too large (%d bytes)\n", sw->icon_size);
            sw->icon_size = 0;
            return;
        }

        sw->icon_offset = 0;
    }
    else
    {
        if (!sw->icon_size)
            return;
    }

    if (chunk_len > (sw->icon_size - sw->icon_offset))
    {
        warning("ui_seamless_seticon: Too large chunk received (%d bytes > %d bytes)\n",
                chunk_len, sw->icon_size - sw->icon_offset);
        sw->icon_size = 0;
        return;
    }

    memcpy(sw->icon_buffer + sw->icon_offset, data, chunk_len);
    sw->icon_offset += chunk_len;

    if (sw->icon_offset == sw->icon_size)
    {
        ewmh_set_icon(sw->wnd, width, height, sw->icon_buffer);
        sw->icon_size = 0;
    }
}


void
ui_seamless_delicon(unsigned long id, const char *format, int width, int height)
{
    seamless_window *sw;

    if (!g_seamless_active)
        return;

    sw = sw_get_window_by_id(id);
    if (!sw)
    {
        warning("ui_seamless_seticon: No information for window 0x%lx\n", id);
        return;
    }

    if (strcmp(format, "RGBA") != 0)
    {
        warning("ui_seamless_seticon: Uknown icon format \"%s\"\n", format);
        return;
    }

    ewmh_del_icon(sw->wnd, width, height);
}


void
ui_seamless_move_window(unsigned long id, int x, int y, int width, int height, unsigned long flags)
{
    seamless_window *sw;

    if (!g_seamless_active)
        return;

    sw = sw_get_window_by_id(id);
    if (!sw)
    {
        warning("ui_seamless_move_window: No information for window 0x%lx\n", id);
        return;
    }

    /* We ignore server updates until it has handled our request. */
    if (sw->outstanding_position)
        return;

    if (!width || !height)
        /* X11 windows must be at least 1x1 */
        return;

    sw->xoffset = x;
    sw->yoffset = y;
    sw->width = width;
    sw->height = height;

    /* If we move the window in a maximized state, then KDE won't
       accept restoration */
    switch (sw->state)
    {
    case SEAMLESSRDP_MINIMIZED:
    case SEAMLESSRDP_MAXIMIZED:
        return;
    }

    /* FIXME: Perhaps use ewmh_net_moveresize_window instead */
    XMoveResizeWindow(g_display, sw->wnd, sw->xoffset+g_xpos, sw->yoffset+g_ypos, sw->width, sw->height);
}


void
ui_seamless_restack_window(unsigned long id, unsigned long behind, unsigned long flags)
{
    seamless_window *sw;
    XWindowChanges values;
    unsigned long restack_serial;

    if (!g_seamless_active)
        return;

    sw = sw_get_window_by_id(id);
    if (!sw)
    {
        warning("ui_seamless_restack_window: No information for window 0x%lx\n", id);
        return;
    }

    if (behind)
    {
        seamless_window *sw_behind;

        sw_behind = sw_get_window_by_id(behind);
        if (!sw_behind)
        {
            warning("ui_seamless_restack_window: No information for behind window 0x%lx\n", behind);
            return;
        }

        if (!g_seamless_broken_restack)
        {
            values.stack_mode = Below;
            values.sibling = sw_behind->wnd;
            restack_serial = XNextRequest(g_display);
            XReconfigureWMWindow(g_display, sw->wnd, DefaultScreen(g_display),
                                 CWStackMode | CWSibling, &values);
            sw_wait_configurenotify(sw->wnd, restack_serial);
        }
    }
    else
    {
        values.stack_mode = Above;
        restack_serial = XNextRequest(g_display);
        XReconfigureWMWindow(g_display, sw->wnd, DefaultScreen(g_display), CWStackMode,
                             &values);
        sw_wait_configurenotify(sw->wnd, restack_serial);
    }

    sw_restack_window(sw, behind);

    if (flags & SEAMLESSRDP_CREATE_TOPMOST)
    {
        /* Make window always-on-top */
        ewmh_set_window_above(sw->wnd);
    }
}


void
ui_seamless_settitle(unsigned long id, const char *title, unsigned long flags)
{
    seamless_window *sw;

    if (!g_seamless_active)
        return;

    sw = sw_get_window_by_id(id);
    if (!sw)
    {
        warning("ui_seamless_settitle: No information for window 0x%lx\n", id);
        return;
    }

    /* FIXME: Might want to convert the name for non-EWMH WMs */
    XStoreName(g_display, sw->wnd, title);
    ewmh_set_wm_name(sw->wnd, title);
}


void
ui_seamless_setstate(unsigned long id, unsigned int state, unsigned long flags)
{
    seamless_window *sw;

    if (!g_seamless_active)
        return;

    sw = sw_get_window_by_id(id);
    if (!sw)
    {
        warning("ui_seamless_setstate: No information for window 0x%lx\n", id);
        return;
    }
    //syslog (LOG_NOTICE, "ui_seamless_setstate:  id=0x%lx, sw->state='%i', new state='%i'\n", id, sw->state, state);

    switch (state)
    {
    case SEAMLESSRDP_NORMAL:
    case SEAMLESSRDP_MAXIMIZED:
        ewmh_change_state(sw->wnd, state);
        XMapWindow(g_display, sw->wnd);
        break;
    case SEAMLESSRDP_MINIMIZED:
        /* EWMH says: "if an Application asks to toggle _NET_WM_STATE_HIDDEN
           the Window Manager should probably just ignore the request, since
           _NET_WM_STATE_HIDDEN is a function of some other aspect of the window
           such as minimization, rather than an independent state." Besides,
           XIconifyWindow is easier. */
        if (sw->state == SEAMLESSRDP_NOTYETMAPPED)
        {
            XWMHints *hints;
            hints = XGetWMHints(g_display, sw->wnd);
            if (hints)
            {
                hints->flags |= StateHint;
                hints->initial_state = IconicState;
                XSetWMHints(g_display, sw->wnd, hints);
                XFree(hints);
            }
            XMapWindow(g_display, sw->wnd);
        }
        else
            XIconifyWindow(g_display, sw->wnd, DefaultScreen(g_display));
        break;
    default:
        warning("SeamlessRDP: Invalid state %d\n", state);
        break;
    }

    sw->state = state;

    XChangeProperty(g_display, sw->wnd, g_net_wm_actions_atom, XA_ATOM, 32,
            PropModeReplace, (unsigned char *)g_allowed_actions,
            Q_ALLOWED_ACTIONS_SIZE);
}


void
ui_seamless_syncbegin(unsigned long flags)
{
    if (!g_seamless_active)
        return;

    /* Destroy all seamless windows */
    while (g_seamless_windows)
    {
        XDestroyWindow(g_display, g_seamless_windows->wnd);
        sw_remove_window(g_seamless_windows);
    }
}


void
ui_seamless_ack(unsigned int serial)
{
    seamless_window *sw;
    for (sw = g_seamless_windows; sw; sw = sw->next)
    {
        if (sw->outstanding_position && (sw->outpos_serial == serial))
        {
            sw->xoffset = sw->outpos_xoffset;
            sw->yoffset = sw->outpos_yoffset;
            sw->width = sw->outpos_width;
            sw->height = sw->outpos_height;
            sw->outstanding_position = False;

            /* Do a complete redraw of the window as part of the
               completion of the move. This is to remove any
               artifacts caused by our lack of synchronization. */
            XCopyArea(g_display, g_backstore,
                      sw->wnd, g_gc,
                      sw->xoffset, sw->yoffset, sw->width, sw->height, 0, 0);

            break;
        }
    }
}

void
ui_pnrdp_set_region(Region r)
{
    XSetRegion(g_display, g_gc, r);
}

#ifdef WITH_MM_SUPPORT

void ui_mm_print_full_info()
{
    int xnum;
    int i;

    XineramaScreenInfo * screens = XineramaQueryScreens(g_display, &xnum);

    if (xnum > 0)
    {
        for (i = 0; i < xnum; ++i)
        {
            printf("Screen %d\n", screens[i].screen_number);
            printf(" x_org:  %d\n", screens[i].x_org);
            printf(" y_org: %d\n", screens[i].y_org);
            printf(" width:  %d\n", screens[i].width);
            printf(" height: %d\n", screens[i].height);
        }
    }
    else
    {
        int num = XScreenCount(g_display);
        for (i = 0; i < num; ++i)
        {
            Screen *tmpscr = XScreenOfDisplay(g_display, i);
            printf("Screen %d\n", i);
            printf(" width:  %d\n", XWidthOfScreen(tmpscr));
            printf(" height: %d\n", XHeightOfScreen(tmpscr));
        }
    }
}

int ui_mm_number_of_screens()
{
    int num = XScreenCount(g_display);
    int xnum;

    XineramaQueryScreens(g_display, &xnum);

    if (xnum != 0)
    {
        num = xnum;
    }
    return num;
}

BOUNDS ui_mm_get_screen_bounds(int num)
{
    int xnum;
    BOUNDS bounds = {0,0,0,0};

    XineramaScreenInfo * screens = XineramaQueryScreens(g_display, &xnum);

    if (xnum > 0)
    {
        bounds.left = screens[num].x_org;
        bounds.top = screens[num].y_org;
        bounds.right = screens[num].x_org + screens[num].width - 1;
        bounds.bottom = screens[num].y_org + screens[num].height - 1;
    }
    else // if Xinerama extention doesn't work
    {
        Screen *scr;
        // it seems we cannot find out the disposition of screens... :(
        // the only thing we can do is suppose that the screens follow
        // each other from left to right; guess it's better than just to set
        // left to zero for each screen
        int num = XScreenCount(g_display);
        int i;
        for (i = 0; i < num; ++i)
        {
            scr = XScreenOfDisplay(g_display, i);
            bounds.left += XWidthOfScreen(scr);
        }
        bounds.left -= XWidthOfScreen(scr);
        bounds.top += 0;
        bounds.right = XWidthOfScreen(scr) + bounds.left;
        bounds.bottom = XHeightOfScreen(scr) + bounds.top;
    }

    return bounds;
}

BOUNDS ui_mm_get_screen_bounds_pntsc_process(int num)
{
    return ui_mm_get_screen_bounds(num);
}

void ui_mm_get_x_y_shifts(int* x_shift, int* y_shift)
{
    // Calculating Ox and Oy shifts to make left-top of primary monitor (0,0)

    int xnum;
    XineramaScreenInfo * screens = XineramaQueryScreens(g_display, &xnum);

    // At first we detect the primary monitor. Seems that this is a problem on Linux
    // So we have two options here:
    // 1. assume XInerama screen[0] as primary;
    // 2. assume leftmost monitor if it is only one and bottommost monitor in other case
    // Here we use #2 option
    int i;
    int leftmostIndex = 0, leftmostCoordinate = screens[0].x_org;
    for (i = 1; i < ui_mm_number_of_screens(); ++i)
    {
        if (leftmostCoordinate > screens[i].x_org)
        {
            leftmostCoordinate = screens[i].x_org;
            leftmostIndex = i;
        }
    }
    int leftmostCount = 0;
    for (i = 0; i < ui_mm_number_of_screens(); ++i)
    {
        if (leftmostCoordinate == screens[i].x_org)
        {
            ++leftmostCount;
        }
    }

    int primaryMonitorIndex;
    if (leftmostCount == 1)
    {
        primaryMonitorIndex = leftmostIndex;
    }
    else
    {
        int bottommostIndex = 0, bottommostCoordinate = screens[0].y_org + screens[0].height;
        for (i = 1; i < ui_mm_number_of_screens(); ++i)
        {
            if (bottommostCoordinate < screens[i].y_org + screens[i].height)
            {
                bottommostCoordinate = screens[i].y_org + screens[i].height;
                bottommostIndex = i;
            }
        }
        primaryMonitorIndex = bottommostIndex;
    }

    // calculating shifts
    *x_shift = screens[primaryMonitorIndex].x_org;
    *y_shift = screens[primaryMonitorIndex].y_org;
}

BOUNDS ui_mm_get_screen_bounds_msc_initial(int num, int x_shift, int y_shift)
{
    // should be called for creating Client Monitor Data structure of MSC Connect Inital PDU
    BOUNDS bounds = ui_mm_get_screen_bounds(num);

    if (g_fullscreen && g_span_monitors && ui_mm_number_of_screens() > 1)
    {
        // Microsoft MM presumes that primary monitor left-top is always (0,0)
        // So we convert coordinates to Microsoft scheme and back to Linux in pntsc process fnc
        bounds.left -= x_shift;
        bounds.right -= x_shift;
        bounds.top -= y_shift;
        bounds.bottom -= y_shift;
    }
    return bounds;
}

#endif

/////////////////////////////////////////////////////////////////////////////////////
void qseamless_send_message_WR(unsigned long id, int x, int y, int width, int height, unsigned long flags);
void qseamless_send_message_SW(unsigned long id, int state);

/*
   WS_OVERLAPPED    = 0x00000000,
   WS_POPUP    = unchecked((int)0x80000000),
   WS_CHILD         = 0x40000000,
   WS_MINIMIZE      = 0x20000000,
   WS_VISIBLE       = 0x10000000,
   WS_DISABLED      = 0x08000000,
   WS_CLIPSIBLINGS  = 0x04000000,
   WS_CLIPCHILDREN  = 0x02000000,
   WS_MAXIMIZE      = 0x01000000,
   WS_CAPTION       = 0x00C00000,
   WS_BORDER        = 0x00800000,
   WS_DLGFRAME      = 0x00400000,
   WS_VSCROLL       = 0x00200000,
   WS_HSCROLL       = 0x00100000,
   WS_SYSMENU       = 0x00080000,
   WS_THICKFRAME    = 0x00040000,
   WS_GROUP         = 0x00020000,
   WS_TABSTOP       = 0x00010000,
   WS_MINIMIZEBOX   = 0x00020000,
   WS_MAXIMIZEBOX   = 0x00010000,
   WS_TILED         = 0x00000000,
   WS_ICONIC        = 0x20000000,
   WS_SIZEBOX       = 0x00040000,
   WS_OVERLAPPEDWINDOW  = (0x00000000|0x00C00000|0x00080000|0x00040000|0x00020000|0x00010000),
   WS_POPUPWINDOW   = (unchecked((int)0x80000000)|0x00800000|0x00080000),
   WS_CHILDWINDOW   = 0x40000000,

   WS_EX_DLGMODALFRAME  = 0x00000001,
   WS_EX_NOPARENTNOTIFY = 0x00000004,
   WS_EX_TOPMOST        = 0x00000008,
   WS_EX_ACCEPTFILES    = 0x00000010,
   WS_EX_TRANSPARENT    = 0x00000020,
   WS_EX_MDICHILD       = 0x00000040,
   WS_EX_TOOLWINDOW     = 0x00000080,
   WS_EX_WINDOWEDGE     = 0x00000100,
   WS_EX_CLIENTEDGE     = 0x00000200,
   WS_EX_CONTEXTHELP    = 0x00000400,
   WS_EX_RIGHT          = 0x00001000,
   WS_EX_LEFT           = 0x00000000,
   WS_EX_RTLREADING     = 0x00002000,
   WS_EX_LTRREADING     = 0x00000000,
   WS_EX_LEFTSCROLLBAR  = 0x00004000,
   WS_EX_RIGHTSCROLLBAR = 0x00000000,
   WS_EX_CONTROLPARENT  = 0x00010000,
   WS_EX_STATICEDGE     = 0x00020000,
   WS_EX_APPWINDOW      = 0x00040000,
   WS_EX_OVERLAPPEDWINDOW = (0x00000100|0x00000200),
   WS_EX_PALETTEWINDOW  = (0x00000100|0x00000080|0x00000008),
   WS_EX_LAYERED        = 0x00080000,
   WS_EX_NOINHERITLAYOUT = 0x00100000,
   WS_EX_LAYOUTRTL      = 0x00400000,
   WS_EX_NOACTIVATE     = 0x08000000,
*/

enum _WIND_PROP
{
    WS_POPUP             = 0x80000000,
    WS_CHILD             = 0x40000000,
    WS_VISIBLE           = 0x10000000,
    WS_CLIPSIBLINGS      = 0x04000000,
    WS_CLIPCHILDREN      = 0x02000000,
    WS_CAPTION           = 0x00C00000,

    WS_EX_DLGMODALFRAME  = 0x00000001,
    WS_EX_TOPMOST        = 0x00000008,
    WS_EX_TRANSPARENT    = 0x00000020,
    WS_EX_MDICHILD       = 0x00000040,
    WS_EX_TOOLWINDOW     = 0x00000080,
    WS_EX_NOPARENTNOTIFY = 0x00000004,
    WS_EX_WINDOWEDGE     = 0x00000100
};


void qseamless_update_position(seamless_window * sw, int x, int y)
{
    int rx =  x - g_xpos;
    int ry =  y - g_ypos;

    //  Send a WINDOWRECT (WR) message to the server.
    qseamless_send_message_WR(sw->id, rx, ry, sw->width, sw->height, 0);

    sw->xoffset = (rx>=0?rx:0);
    sw->yoffset = (ry>=0?ry:0);
    sw->width = sw->width;
    sw->height = sw->height;
    syslog (LOG_NOTICE, "qseamless_update_position: sw->xoffset=%d,sw->yoffset=%d, sw->width=%d, sw->height=%d \n", sw->xoffset,sw->yoffset, sw->width, sw->height);
}


void qseamless_update_position_size(seamless_window * sw, int x, int y, int width, int height)
{
    int rx =  x - g_xpos;
    int ry =  y - g_ypos;

    //  Send a WINDOWRECT (WR) message to the server.
    qseamless_send_message_WR(sw->id, rx, ry, width, height, 0);

    sw->xoffset = (rx>=0?rx:0);
    sw->yoffset =(ry>=0?ry:0);
    sw->width = width;
    sw->height = height;
    syslog (LOG_NOTICE, "qseamless_update_position_size: sw->xoffset=%d,sw->yoffset=%d, sw->width=%d, sw->height=%d \n", sw->xoffset,sw->yoffset, sw->width, sw->height);
}


void qseamless_update_position_size1(seamless_window * sw, int x, int y, int width, int height)
{
    sw->xoffset = (x>0?x:0);
    sw->yoffset =(y>0?y:0);
    sw->width = width;
    sw->height = height;
    syslog (LOG_NOTICE, "qseamless_update_position_size1: sw->xoffset=%d,sw->yoffset=%d, sw->width=%d, sw->height=%d \n", sw->xoffset,sw->yoffset, sw->width, sw->height);
}

int qseamless_process_CR_check_title(char *title)
{
    if (!strcmp(title, "Program Manager"))
    {
         ui_seamless_toggle();
        pnts_send_message ("DM0"); // fullscreen=0, seamless=1
        return 1;
    }

    if (!strcmp(title, "____")) //PNTrayWindowName____"))
    {
        return 1;
    }

    if (!strcmp(title, "pnshell.cmd"))
    {
        return 1;
    }
    if (!strcmp(title, "UsrLogon.cmd"))
    {
        return 1;
    }

    if (!strcmp(title, "Quest vWorkspace Shell"))
    {
        return 1;
    }

    if (!strcmp(title, "Desktop Alert"))
    {
        return 1;
    }

    return 0;
}

int qseamless_process_CR_check_style(unsigned long id, unsigned long *parent, unsigned long *style, unsigned long *exStyle, unsigned long *tooltipwindow, unsigned long *flags)
{
    if ((*style == 0x14000000) && (*exStyle == 0)) return 1;

    if ((*style & WS_CAPTION) != WS_CAPTION)
    {
        if ((*style & WS_CHILD) == 0)
        {
            if ((*style == 0x160F0000) && (*exStyle == 0x00050100) && (*parent == 0))
            {
                //  Skype main window? FIXME
                //  DISPLAY IT
            }
            else
        {
            *style |= WS_POPUP;
            *exStyle |= WS_EX_TOOLWINDOW;
            }
        }
    }
    else
    {
        if ((*exStyle == 0x00050100) && (*parent == 0) && (*style == 0x16C80000))
        {
            //  This is first or last skype Login window.
            //  If first - set flag to prevent logoff after closing this window
            //  (before Main skype window is going to appear).
            gq_support_skype_id = id;
        }
    }

    if ((*style & WS_VISIBLE) && ((*style & 0x0F000000) == WS_CLIPSIBLINGS) && (*style & WS_CAPTION) && ((*style & 0x000F0000) == 0)  && (*exStyle & WS_EX_WINDOWEDGE) && (*parent == 0))
    {
        return 1;
    }

    if ((*style & 0x000000F0) == WS_EX_TOOLWINDOW)
    {
        *exStyle |= WS_EX_TOOLWINDOW;
    }

    if (*exStyle & WS_EX_TOPMOST)
    {
        *flags |= SEAMLESSRDP_CREATE_TOPMOST;
        if ((*style & WS_POPUP) && (*style & WS_VISIBLE) && (*style & WS_CLIPSIBLINGS))
        {
            *tooltipwindow = 1;
        }
    }

    if (*exStyle & WS_EX_TOOLWINDOW)
    {
        //  parent=0x40018, style=0x96000000, exStyle=0x88
        //  parent=0x40018, style=0x94000000, exStyle=0x8      WS_EX_TOPMOST
        //  parent=0x0,     style=0x94800000, exStyle=0x88
        //  parent=0x500fe, style=0x94000000, exStyle=0x800a0  WS_EX_TRANSPARENT + WS_EX_TOOLWINDOW
        //  parent=0x0,     style=0x96800000, exStyle=0x80     WS_EX_TOOLWINDOW
        if ((*style & WS_POPUP) && (*style & WS_VISIBLE) && (*style & WS_CLIPSIBLINGS) && ((*exStyle & WS_EX_TOPMOST) || (*exStyle & WS_EX_TOOLWINDOW)))
        {
            *tooltipwindow = 1;
        }
        else
        {
            // If this is a toolwindow, then don't actually display the window.  This keeps
            // the window from automatically getting the input focus when the window is shown.
            *style &= ~WS_VISIBLE;
            syslog (LOG_NOTICE, "style = 0x%lx  NOT-VISIBLE.\n", *style);

            if ((*style & WS_EX_DLGMODALFRAME) && (*style & WS_EX_MDICHILD))
            {
                *flags |= SEAMLESSRDP_CREATE_TOPMOST;
            }
            else
            {
                return 1;
            }
        }
    }

    if (*tooltipwindow)
    {
        // Don't allow any tooltips while moving/resizing.`
        // Mike M. 5/31/2007
        if (g_moving_wnd_seamless_start_move || g_moving_wnd_seamless_start_size)
        {
            return 1;
        }
    }

    if ((*style & WS_POPUP) && (*parent == 0))
    {
        *parent = 0xFFFFFFFF;
    }

    return 0;
}

int qseamless_process_CR(unsigned long id, unsigned long parent, unsigned long style, unsigned long exStyle, int x, int y, int width, int height, char *title)
{
        //syslog (LOG_NOTICE, "qseamless_process_CR: id=0x%lx, parent=0x%lx, style=0x%lx, exStyle=0x%lx, x='%i', y='%i', width='%i', height='%i', title=%s\n", id, parent, style, exStyle, x, y, width, height, title);

        if ((width == 0) || (height == 0))
        {
            return 1;
        }

        if (!gq_is_AS)
        {
            // Application hasn't been started yet.
            // qseamless_process_SW (Show Window) can make this window visible on the screen.
            // For now this window is invisible.
            style &= ~WS_VISIBLE;
        }

        if (qseamless_process_CR_check_title(title))
        {
                return 1;
        }

        unsigned long flags = 0;
        unsigned long tooltipwindow = 0;
        if (qseamless_process_CR_check_style(id, &parent, &style, &exStyle, &tooltipwindow, &flags))
        {
            return 1;
        }

        //  To deal with group of windows:
        //  - after receiving "AS" message the group doesn't exist.
        //  - after receiving "AS" message the first window is MAIN in the group - and must be created in any cases.
        unsigned long group = gq_is_AS;
        syslog (LOG_NOTICE, "group=%ld\n", group);

        ui_seamless_create_window(id, group, parent, flags);  //  here new group is created

        if (strlen(title)>0)
        {
            ui_seamless_settitle(id, title, 0);
        }

        seamless_window *sw;
        sw = sw_get_window_by_id(id);
        if (!sw)
            return 1;

        //  add support for FMMR and MMR clients
        if (strcasestr(g_response_message, "iexplore.exe") || strcasestr(g_response_message, "wmplayer.exe"))
        {
            pid_t pid;
            unsigned long pid_l;
            Atom pid_atom;
            int rc;
            char hostname[90];
            char* x_hostnames;

            XTextProperty hostname_property;
            /* XSetWMProperties(g_dislpay, g_wnd, window_name, icon_name,
                argv, argc, normal_hints, wm_hints, class_hints); also does the
                XSetWMClientMachine bit too */
            if (gethostname(hostname, sizeof(hostname)) != 0)
            {
                syslog (LOG_NOTICE, "error: could not determine hostname.\n");
                return 1;
            }
            x_hostnames = hostname;

            if (XStringListToTextProperty(&x_hostnames, 1, &hostname_property) != 0)
            {
                XSetWMClientMachine(g_display, sw->wnd, &hostname_property);
                XFree(hostname_property.value);
            }

            pid = getpid();
            pid_l = pid;
            pid_atom = XInternAtom(g_display, "_NET_WM_PID", False);
            rc = XChangeProperty(g_display, sw->wnd,
                                 pid_atom, XA_CARDINAL, 32, PropModeReplace,
                                 (unsigned char*)&pid_l, 1);
        }

        sw->parent = parent;
        int state = SEAMLESSRDP_NORMAL;
        if (x < 0 && y < 0)
        {
            state = SEAMLESSRDP_MAXIMIZED;
        }

        if (tooltipwindow)
        {
            sw->tooltip = 1;
        }
        else
        {
            sw->tooltip = 0;
        }

        if (sw->tooltip == 0)
        {
            if (width < QWIN_MIN_WIDTH)
            {
                width = QWIN_MIN_WIDTH;
            }
            if (height < QWIN_MIN_HEIGHT)
            {
                height = QWIN_MIN_HEIGHT;
            }
        }

        XMoveResizeWindow(g_display, sw->wnd, g_xpos+x, g_ypos+y, width, height);
        qseamless_update_position_size1(sw, x, y, width, height);

        if ((style & WS_VISIBLE) != 0)
        {
            if (((strlen(title) > 0) || (sw->tooltip)) && id)
            {
                ui_seamless_setstate(id, state, 0);  //  exec XMapWindow
            }
        }

        if (flags == SEAMLESSRDP_CREATE_TOPMOST)
        {
            g_seamless_focused = sw->id;  //  fixme
        }

        return 0;
}


void qseamless_process_DE(unsigned long id)
{
   if (!sw_window_exists(id))
        return;

    ui_seamless_destroy_window(id, 0);

    if (!g_seamless_windows)
    {
        if (gq_is_AS && !gq_support_skype_id)
        {
            pnts_send_message("LO");
        }
    }
}


int qseamless_process_WP(unsigned long id, int x, int y, int width, int height, unsigned long flags, unsigned long state)
{
    if (g_moving_wnd_seamless_start_move || g_moving_wnd_seamless_start_size)
        return 1;

    //syslog (LOG_NOTICE, "qseamless_process_WP: 0x%lx x='%i', y='%i', width='%i', height='%i' flags=%li, state=%li\n", id, x, y, width, height, flags, state);

    seamless_window *sw = sw_get_window_by_id(id);
    if (!sw)
        return 1;

    if (sw->tooltip == 0)
    {
        if (width < QWIN_MIN_WIDTH)
        {
            width = QWIN_MIN_WIDTH;
        }
        if (height < QWIN_MIN_HEIGHT)
        {
            height = QWIN_MIN_HEIGHT;
        }
    }

    /*  QUEST states from server:
                1 normal
                2 minimized
                3 maximized

        SeamlessRDP states:
                #define SEAMLESSRDP_NOTYETMAPPED -1
                #define SEAMLESSRDP_NORMAL 0
                #define SEAMLESSRDP_MINIMIZED 1
                #define SEAMLESSRDP_MAXIMIZED 2
    */

    if ((state-1) == SEAMLESSRDP_MINIMIZED)
    {
        //  Skype issue:
        //  The main window never exits.
        //
        //  When main Skype window is closed by using:
        //    1. click on CloseButton on the right top of window
        //  or
        //    2. click "Close" item from Menu
        //  then message "Minimize window" is received
        //  instead of "DE" message.
        //  Here we use workaround to make it closed.
        if (gq_support_skype_id == id)
        {
            //qseamless_process_skype_exit (id);
            //return 0;
        }

        if (sw->state != (state-1))
        {
            qseamless_send_message_WR(sw->id, x, y, width, height, 0);
            qseamless_update_position_size1(sw, x, y, width, height);

            sw->state = SEAMLESSRDP_MINIMIZED;
            XIconifyWindow(g_display, sw->wnd, DefaultScreen(g_display));
        }
        else if (gq_window_manager_process)
        {
            qseamless_send_message_WR(sw->id, x, y, width, height, 0);
            qseamless_update_position_size1(sw, x, y, width, height);

            XIconifyWindow(g_display, sw->wnd, DefaultScreen(g_display));
            gq_window_manager_process = 0;
        }
        //  no need to change window size
        return 0;
    }

    if ((state-1) == SEAMLESSRDP_MAXIMIZED)
    {
        if (sw->state != (state-1))
        {
            qseamless_send_message_WR(sw->id, x, y, width, height, 0);
            qseamless_update_position_size1(sw, x, y, width, height);

            XMoveResizeWindow(g_display, sw->wnd, g_xpos+x, g_ypos+y, width, height);
            sw->state == state-1;
        }
        else if (gq_window_manager_process)
        {
            qseamless_send_message_WR(sw->id, x, y, width, height, 0);
            qseamless_update_position_size1(sw, x, y, width, height);

            ui_seamless_setstate(id, state - 1, 0);  //0=flags,  also converts state from QUEST format
            gq_window_manager_process = 0;

            /* Do a complete redraw of the window as part of the
               completion of the move. This is to remove any
               artifacts caused by our lack of synchronization. */
            XCopyArea(g_display, g_backstore,
                      sw->wnd, g_gc,
                      sw->xoffset, sw->yoffset, sw->width, sw->height, 0, 0);
        }
        return 0;
    }

    if ((state-1) == SEAMLESSRDP_NORMAL)
    {
        if (gq_window_manager_process && sw->state == (state-1))
        {
            qseamless_send_message_WR(sw->id, x, y, width, height, 0);
            qseamless_update_position_size1(sw, x, y, width, height);

            gq_window_manager_process = 0;
            ui_seamless_setstate(id, state - 1, 0);  //0=flags,  also converts state from QUEST format
            //qwaitevent(sw->wnd, id, state, 1);

            XMoveResizeWindow(g_display, sw->wnd, g_xpos+x, g_ypos+y, width, height);
            /* Do a complete redraw of the window as part of the
               completion of the move. This is to remove any
               artifacts caused by our lack of synchronization. */
            XCopyArea(g_display, g_backstore,
                      sw->wnd, g_gc,
                      sw->xoffset, sw->yoffset, sw->width, sw->height, 0, 0);
            return 0;
        }
        else
        {
            if (sw->state != (state-1))
            {
                ui_seamless_setstate(id, state - 1, 0);  //0=flags,  also converts state from QUEST format
            }

            if ((x < 0) || (y < 0))
            {
                return 0;
            }
            if ((x != sw->xoffset) || (y != sw->yoffset) || (width != sw->width) || (height != sw->height))
            {
                //qseamless_send_message_WR(sw->id, x, y, width, height, 0);
                qseamless_update_position_size1(sw, x, y, width, height);

                XMoveResizeWindow(g_display, sw->wnd, g_xpos+x, g_ypos+y, width, height);
            }
        }
        return 0;
    }

    return 0;
}


void qseamless_draw_fake( int width, int height)
{
    Pixmap pmap;
    GC shape_gc;
    GC win_gc;
    XGCValues xgcv;

    /* convenience */
    int BLACK_PIXEL = BlackPixel(g_display, DefaultScreen(g_display));
    unsigned int GREEN_PIXEL = 0xFFFFL;

    if (height <= 0 || width <= 0)
        return;

    unsigned int line_width = 5;
    int line_style = LineDoubleDash; // LineOnOffDash LineSolid;
    int cap_style = CapButt;         // CapProjecting CapRound
    int join_style = JoinBevel;      // JoinMiter JoinRound
    int dash_offset = 0;
    char dash_list[] = {2, 1};
    int list_length = 2;

    win_gc = XCreateGC(g_display, gq_fake_wnd, 0, &xgcv);

    // create the pixmap that we'll use for shaping the window
    pmap = XCreatePixmap(g_display, gq_fake_wnd, width, height, 1);

    // create a graphics context for drawing on the pixmap
    shape_gc = XCreateGC(g_display, pmap, 0, &xgcv);
    XSetForeground(g_display, shape_gc, BLACK_PIXEL);
    XFillRectangle(g_display, pmap, shape_gc, 0, 0, width, height);

    XSetForeground(g_display, shape_gc, GREEN_PIXEL);
    XSetLineAttributes(g_display, shape_gc, line_width, line_style, cap_style, join_style);
    XSetDashes(g_display, shape_gc, dash_offset, dash_list, list_length);
    XDrawRectangle(g_display, pmap, shape_gc, 0,0, width, height );

    XSetForeground(g_display, win_gc, GREEN_PIXEL);
    XSetLineAttributes(g_display, win_gc, line_width, line_style, cap_style, join_style);
    XSetDashes(g_display, win_gc, dash_offset, dash_list, list_length);
    XDrawRectangle(g_display, gq_fake_wnd, win_gc, 0,0, width, height );

    XShapeCombineMask (g_display, gq_fake_wnd, ShapeBounding,
            0, 0, pmap, ShapeSet);

    XSync(g_display, False);
    XFreePixmap(g_display, pmap);
}


void qseamless_create_fake_of(unsigned long id)
{
    XEvent xevent;

    seamless_window *sw = sw_get_window_by_id(id);
    if (!sw)
        return;

    syslog (LOG_NOTICE, "qseamless_create_fake_of %d - %d, %d - %d\n", sw->xoffset+g_xpos, sw->yoffset+g_ypos, sw->width, sw->height);

    if (sw->width <= 0 || sw->height <= 0)
        return;

    XSetWindowAttributes attribs;
    get_window_attribs(&attribs);
    attribs.override_redirect = True;  //  we don't want Window Manager manages this window

    gq_fake_wnd = XCreateWindow(g_display, RootWindowOfScreen(g_screen), 0, 0, 1, 1, 0, g_depth, InputOutput, g_visual,
                          CWBackPixel | CWBackingStore | CWColormap | CWOverrideRedirect | CWBorderPixel, &attribs);

    char name[64];
    snprintf(name, sizeof(name), "qSeamlessRDP fake window");
    XStoreName(g_display, gq_fake_wnd, name);
    ewmh_set_wm_name(gq_fake_wnd, name);

    /* Hide decorations. Often this means that no
       reparenting will be done, which makes the restack
       easier. Besides, we want to mimic our other
       seamless windows as much as possible. We must still
       handle the case with reparenting, though. */
    mwm_hide_decorations(gq_fake_wnd);

    /* Prevent windows from appearing in task bar */
    XSetTransientForHint(g_display, gq_fake_wnd, RootWindowOfScreen(g_screen));

    Atom _NET_WM_WINDOW_TYPE = XInternAtom(g_display, "_NET_WM_WINDOW_TYPE", False);
    Atom _NET_WM_WINDOW_TYPE_DND = XInternAtom(g_display, "_NET_WM_WINDOW_TYPE_DND", False);
    XChangeProperty(g_display, gq_fake_wnd, _NET_WM_WINDOW_TYPE, XA_ATOM, 32, PropModeReplace, (unsigned char *) &_NET_WM_WINDOW_TYPE_DND, 1);

    /* We need to catch MapNotify/ConfigureNotify */
    XSelectInput(g_display, gq_fake_wnd, StructureNotifyMask | ExposureMask);

    XMapWindow(g_display, gq_fake_wnd);

    /* flush all pending requests to the X server. */
    XFlush(g_display);
    do
    {
        XWindowEvent(g_display, gq_fake_wnd, StructureNotifyMask, &xevent);
    }
    while (xevent.type != MapNotify);

    XMoveResizeWindow(g_display, gq_fake_wnd, sw->xoffset+g_xpos, sw->yoffset+g_ypos, sw->width, sw->height);
}


#define SC_MOVE 0xF010          // Moves the window.
#define SC_MOVE_INTERNAL 0xF012 // Moves the window.
#define SC_SIZE 0xF000          // Sizes the window.
void qseamless_process_SY(unsigned long id, unsigned long param)
{
    if (g_moving_wnd_seamless_start_move || g_moving_wnd_seamless_start_size)
        return;

    seamless_window *sw = sw_get_window_by_id(id);
    if (!sw)
        return;

    if ((param != SC_MOVE) && (param != SC_MOVE_INTERNAL) && (sw->parent > 0))
    {
        g_moving_wnd_seamless = id;

        if (param == SC_SIZE)
        {
            g_moving_wnd_seamless_start_size = QWIN_SIDE_BUTTOM;
            syslog (LOG_NOTICE, "SC_SIZE QWIN_SIDE_BUTTOM, %lx\n", sw->parent);
        }
        else
        {
            g_moving_wnd_seamless_start_size = (param & 0x000F);
            syslog (LOG_NOTICE, "SC_SIZE, %lx, param=%d\n", sw->parent, param);
        }
        qseamless_create_fake_of(id);
    }
    else if ((g_moving_wnd_seamless_start_size == 0) &&
                    (((param & 0xFFF0) == 0xF010) || (param == SC_SIZE)))
    {
        //  It seems SC_MOVE isn't used in last OS or Apps. Instead "0xF000" is used.

        //  in case of not using mouse click on the title bar:
        if( !g_move_x_offset_seamless && !g_move_y_offset_seamless)
        {
            g_moving_wnd_seamless = 0;
            return;
        }
        g_moving_wnd_seamless = id;
        g_moving_wnd_seamless_start_move = id;
        syslog (LOG_NOTICE, "SC_MOVE\n");
        qseamless_create_fake_of(id);
    }
    else
    {
        if((g_moving_wnd_seamless_start_move == 0) &&
                ((g_moving_wnd_seamless_start_size == 0) ||
                    (g_moving_wnd_seamless_start_size == (param & 0x000F) )))
        {
            g_moving_wnd_seamless = id;
            g_moving_wnd_seamless_start_size = (param & 0x000F);
            syslog (LOG_NOTICE, "SC_SIZE\n");
            qseamless_create_fake_of(id);
        }
    }
}


// Process the SETTEXT (ST) message.
void qseamless_process_ST(unsigned long id, char *title)
{
    //  Check whether the window exists
    seamless_window *sw = sw_get_window_by_id(id);
    if (!sw)
        return;

    if (strlen(title)>0)
        ui_seamless_settitle(id, title, 0);
}


/*
    SW_FORCEMINIMIZE 11 Minimizes a window, even if the thread that owns the window is not responding. This flag should only be used when minimizing windows from a different thread.
    SW_HIDE 0 Hides the window and activates another window.
    SW_MAXIMIZE 3 Maximizes the specified window.
    SW_MINIMIZE 6 Minimizes the specified window and activates the next top-level window in the Z order.
    SW_RESTORE 9 Activates and displays the window. If the window is minimized or maximized, the system restores it to its original size and position. An application should specify this flag when restoring a minimized window.
    SW_SHOW 5 Activates the window and displays it in its current size and position.
    SW_SHOWDEFAULT 10 Sets the show state based on the SW_ value specified in the STARTUPINFO structure passed to the CreateProcess function by the program that started the application.
    SW_SHOWMAXIMIZED 3 Activates the window and displays it as a maximized window.
    SW_SHOWMINIMIZED 2 Activates the window and displays it as a minimized window.
    SW_SHOWMINNOACTIVE 7 Displays the window as a minimized window. This value is similar to SW_SHOWMINIMIZED, except the window is not activated.
    SW_SHOWNA 8 Displays the window in its current size and position. This value is similar to SW_SHOW, except that the window is not activated.
    SW_SHOWNOACTIVATE 4 Displays a window in its most recent size and position. This value is similar to SW_SHOWNORMAL, except that the window is not activated.
    SW_SHOWNORMAL 1 Activates and displays a window. If the window is minimized or maximized, the system restores it to its original size and position. An application should specify this flag when displaying the window for the first time.
*/
// Process the Show Window (SW) message.
void qseamless_process_SW(unsigned long id, int state)
{
    if (g_moving_wnd_seamless_start_move || g_moving_wnd_seamless_start_size)
        return;

    //  Check whether the window exists
    seamless_window *sw = sw_get_window_by_id(id);
    if (!sw)
        return;

    ui_seamless_setstate(id, state - 1, 0);  // convert state from QUEST format

    qseamless_send_message_SW(id, state);
}


// Send a WINDOWRECT (WR) message to the server.
void qseamless_send_message_WR(unsigned long id, int x, int y, int width, int height, unsigned long flags)
{
    char message[Q_MESSAGE_LEN];
    int len = sprintf (message, "WR%lx,%d,%d,%d,%d", id, x, y, x + width, y + height);
    message[len] = '\0';
    pnts_send_message(message);
}


void qseamless_process_RG(unsigned long id)
{
    //  syslog (LOG_NOTICE, "qseamless_process_RG id=%lx\n", id);
}

void qseamless_process_skype_exit (unsigned long id)
{
    gq_support_skype_id = 0;
    ui_seamless_destroy_window(id, 0);

    if (!g_seamless_windows)
    {
        if (gq_is_AS)
        {
            pnts_send_message("LO");
        }
    }
}

int get_current_desktop(void);

#define WA_ACTIVE 1      //  Activated by some method other than a mouse click (for example, by a call to the SetActiveWindow function or by use of the keyboard interface to select the window).
#define WA_CLICKACTIVE 2 //  Activated by a mouse click.
#define WA_INACTIVE 0    //  Deactivated.
void qseamless_process_AC(unsigned long id, int state)
{
    seamless_window *sw = sw_get_window_by_id(id);
    if (!sw)
        return;

    switch (state)
    {
        case WA_ACTIVE:
        case WA_CLICKACTIVE:
            if (sw->id != g_seamless_focused)
            {
                if (sw_window_exists(sw->id))
                {
                    if(sw->state == SEAMLESSRDP_NORMAL)  //  || sw->state == SEAMLESSRDP_MAXIMIZED)
                    {
                        int current_desktop = get_current_desktop();
                        if (current_desktop < 0 )
                            return;

                        if ((unsigned int)current_desktop == sw->desktop)
                        {
                            syslog (LOG_NOTICE, "qseamless_process_AC: ACTIVE state, sw->id=%lx.\n", sw->id);
                            XSetInputFocus(g_display, sw->wnd, RevertToPointerRoot, CurrentTime);
                            g_seamless_focused = sw->id;
                        }
                    }
                }
            }
            break;
        case WA_INACTIVE:
            syslog (LOG_NOTICE, "qseamless_process_AC: INACTIVE state.\n");
            break;
        default:
            syslog (LOG_NOTICE, "qseamless_process_AC: unexpected state: %d.\n", state);
            break;
    }
}


//  Activate Window
void qseamless_send_message_WA_CLICKACTIVE(unsigned long id)
{
    char message[Q_MESSAGE_LEN];
       int len = sprintf(message, "AC%lx,%u",id, 2); // WA_CLICKACTIVE
    message[len] = '\0';
    pnts_send_message(message);
}


void qseamless_send_message_WA_INACTIVE(unsigned long id)
{
    char message[Q_MESSAGE_LEN];
    int len = sprintf(message, "AC%lx,%u", id, 0); // 0=WA_INACTIVE
    message[len] = '\0';
    pnts_send_message(message);
}


//  CLOSE window
void qseamless_send_message_SY_CLOSE(unsigned long id)
{
    char message[Q_MESSAGE_LEN];
    int len = sprintf(message, "SY%lx,0xF060", id);  // SC_CLOSE
    message[len] = '\0';
    pnts_send_message(message);

    //  Skype window isn't closed by above message.
    //  This is workaround to make it closed.
    if (gq_support_skype_id == id)
    {
        qseamless_process_skype_exit (id);
    }
}


//  MAXIMIZE window
void qseamless_send_message_SY_MAXIMIZE(unsigned long id)
{
    char message[Q_MESSAGE_LEN];
    int len = sprintf(message, "SY%lx,0xF030", id);  // SC_MAXIMIZE
    message[len] = '\0';
    pnts_send_message(message);
}


//  MINIMIZE window
void qseamless_send_message_SY_MINIMIZE(unsigned long id)
{
    char message[Q_MESSAGE_LEN];
    int len = sprintf(message, "SY%lx,0xF020", id);  // SC_MINIMIZE
    message[len] = '\0';
    pnts_send_message(message);
}


//  RESTORE window
void qseamless_send_message_SY_RESTORE(unsigned long id)
{
    char message[Q_MESSAGE_LEN];
    int len = sprintf(message, "SY%lx,0xF120", id);  // SC_RESTORE
    message[len] = '\0';
    pnts_send_message(message);
}


// Send a Show Window (SW) message to the server.
void qseamless_send_message_SW(unsigned long id, int state)
{
    char message[Q_MESSAGE_LEN];
    int len = sprintf (message, "SW%lx,%d", id, state);
    message[len] = '\0';
    pnts_send_message(message);
}


void qprocess_moving_wnd_seamless (XEvent xevent)
{
    if (!g_seamless_hidden && g_seamless_active && (xevent.type == ButtonPress) && !g_moving_wnd_seamless)
    {
        seamless_window *sw;
        sw = sw_get_window_by_wnd(xevent.xfocus.window);
        if (!sw)
            return;

        g_moving_wnd_seamless = sw->id;
        g_move_x_offset_seamless = xevent.xbutton.x;
        g_move_y_offset_seamless = xevent.xbutton.y;
        syslog (LOG_NOTICE, "START g_moving_wnd_seamless=%lx\n", g_moving_wnd_seamless);
    }

    if (!g_seamless_hidden && g_seamless_active && (xevent.type == ButtonRelease))
    {
        if (g_moving_wnd_seamless)
        {
            syslog (LOG_NOTICE, "STOP g_moving_wnd_seamless=%lx\n", g_moving_wnd_seamless);
            g_moving_wnd_seamless = 0;
            g_move_x_offset_seamless = 0;
            g_move_y_offset_seamless = 0;
        }

        if (g_moving_wnd_seamless_start_move || g_moving_wnd_seamless_start_size)
        {
            seamless_window *sw;
            sw = sw_get_window_by_wnd(xevent.xfocus.window);
            if (!sw)
                return;

            //syslog (LOG_NOTICE, "ButtonRelease - g_moving_wnd_seamless_start_move=%ld, g_moving_wnd_seamless_start_size = %ld, sw->id=%ld,  xevent.xfocus.window=%lx, sw->wnd=%lx  gq_fake_wnd=%lx\n", g_moving_wnd_seamless_start_move, g_moving_wnd_seamless_start_size, sw->id, xevent.xfocus.window, sw->wnd, gq_fake_wnd);

            if (g_moving_wnd_seamless_start_size)
            {
                XWindowAttributes wa;
                XGetWindowAttributes(g_display, gq_fake_wnd, &wa);
                syslog (LOG_NOTICE, "wa:   x=%d, y=%d, width=%d, height=%d \n", wa.x, wa.y, wa.width, wa.height);
                if ((wa.width >= QWIN_MIN_WIDTH) && (wa.height >= QWIN_MIN_HEIGHT))
                {
                    XMoveResizeWindow(g_display, sw->wnd, wa.x, wa.y, wa.width, wa.height);
                    qseamless_update_position_size(sw, wa.x, wa.y, wa.width, wa.height);
                }
            }
            else  //  g_moving_wnd_seamless_start_move
            {
                XWindowAttributes wa;
                XGetWindowAttributes(g_display, gq_fake_wnd, &wa);
                syslog (LOG_NOTICE, "wa:   x=%d, y=%d, width=%d, height=%d \n", wa.x, wa.y, wa.width, wa.height);
                if (wa.x < g_xpos)
                {
                    wa.x = g_xpos;
                }
                if (wa.x > g_xpos + g_width - wa.width)
                {
                    wa.x = g_xpos + g_width - wa.width;
                }
                if (wa.y < g_ypos)
                {
                    wa.y = g_ypos;
                }
                if (wa.y > g_ypos + g_height - wa.height)
                {
                    wa.y = g_ypos + g_height - wa.height;
                }

                XMoveWindow(g_display, sw->wnd, wa.x, wa.y);
                qseamless_update_position(sw, wa.x, wa.y);

                //syslog (LOG_NOTICE, "UPDATED g_moving_wnd_seamless_start_move = %ld, sw->id= %ld, x=%d, y=%d\n", g_moving_wnd_seamless_start_move, sw->id, wa.x, wa.y);
            }

            /* Destroy fake window */
            if (gq_fake_wnd)
            {
                XDestroyWindow(g_display, gq_fake_wnd);
                gq_fake_wnd = 0;
            }

            g_moving_wnd_seamless_start_move = 0;
            g_moving_wnd_seamless_start_size = 0;

            g_seamless_focused = sw->parent;
        }
    }
}


void qprocess_moving_wnd_seamless_size (XEvent xevent)
{
    seamless_window *sw;

    if (g_moving_wnd_seamless && g_moving_wnd_seamless_start_size)
    {
        sw = sw_get_window_by_wnd(xevent.xfocus.window);
        if (!sw)
            return;

        if (g_moving_wnd_seamless == sw->id)
        {
            int x,y,w,h;
            switch (g_moving_wnd_seamless_start_size)
            {
                case QWIN_SIDE_LEFT:
                    x = xevent.xmotion.x_root;
                    y = sw->yoffset+g_ypos;
                    w =sw->xoffset+g_xpos + sw->width - xevent.xmotion.x_root;
                    h = sw->height;
                    if (w <= QWIN_MIN_WIDTH)
                    {
                        x = sw->xoffset+g_xpos + sw->width - QWIN_MIN_WIDTH;
                        w = QWIN_MIN_WIDTH;
                    }
                    break;

                case QWIN_SIDE_RIGHT:
                    x = sw->xoffset+g_xpos;
                    y = sw->yoffset+g_ypos;
                    w = xevent.xmotion.x_root - sw->xoffset - g_xpos;
                    h = sw->height;
                    if (w <= QWIN_MIN_WIDTH)
                    {
                        x = sw->xoffset + g_xpos;
                        w = QWIN_MIN_WIDTH;
                    }
                    break;

                case QWIN_SIDE_BUTTOM:
                    x = sw->xoffset+g_xpos;
                    y = sw->yoffset+g_ypos;
                    w = sw->width;
                    h = xevent.xmotion.y_root - sw->yoffset-g_ypos;
                    if (h <= QWIN_MIN_HEIGHT)
                    {
                        y = sw->yoffset+g_ypos;
                        h = QWIN_MIN_HEIGHT;
                    }
                    break;

                case QWIN_SIDE_UP:
                    x = sw->xoffset+g_xpos;
                    y = xevent.xmotion.y_root;
                    w = sw->width;
                    h = sw->height + sw->yoffset+g_ypos - xevent.xmotion.y_root;
                    if (h <= QWIN_MIN_HEIGHT)
                    {
                        y = sw->yoffset+g_ypos + sw->height - QWIN_MIN_HEIGHT;
                        h = QWIN_MIN_HEIGHT;
                    }
                    break;

                case QWIN_CORNER_UP_LEFT:
                    x = xevent.xmotion.x_root;
                    y = xevent.xmotion.y_root;
                    w = sw->width + sw->xoffset+g_xpos - xevent.xmotion.x_root;
                    h = sw->height + sw->yoffset+g_ypos - xevent.xmotion.y_root;
                    if (w <= QWIN_MIN_WIDTH)
                    {
                        x = sw->xoffset+g_xpos + sw->width - QWIN_MIN_WIDTH;
                        w = QWIN_MIN_WIDTH;
                    }
                    if (h <= QWIN_MIN_HEIGHT)
                    {
                        y = sw->yoffset+g_ypos + sw->height - QWIN_MIN_HEIGHT;
                        h = QWIN_MIN_HEIGHT;
                    }
                    break;

                case QWIN_CORNER_UP_RIGHT:
                    x = sw->xoffset+g_xpos;
                    y = xevent.xmotion.y_root;
                    w = xevent.xmotion.x_root - sw->xoffset-g_xpos;
                    h = sw->height + sw->yoffset+g_ypos - xevent.xmotion.y_root;
                    if (w <= QWIN_MIN_WIDTH)
                    {
                        x = sw->xoffset+g_xpos;
                        w = QWIN_MIN_WIDTH;
                    }
                    if (h <= QWIN_MIN_HEIGHT)
                    {
                        y = sw->yoffset+g_ypos + sw->height - QWIN_MIN_HEIGHT;
                        h = QWIN_MIN_HEIGHT;
                    }
                    break;

                case QWIN_CORNER_BUTTOM_LEFT:
                    x = xevent.xmotion.x_root;
                    y = sw->yoffset+g_ypos;
                    w = sw->width + sw->xoffset+g_xpos - xevent.xmotion.x_root;
                    h = xevent.xmotion.y_root - sw->yoffset-g_ypos;
                    if (w <= QWIN_MIN_WIDTH)
                    {
                        x = sw->xoffset+g_xpos + sw->width - QWIN_MIN_WIDTH;
                        w = QWIN_MIN_WIDTH;
                    }
                    if (h <= QWIN_MIN_HEIGHT)
                    {
                        y = sw->yoffset+g_ypos;
                        h = QWIN_MIN_HEIGHT;
                    }
                    break;

                case QWIN_CORNER_BUTTOM_RIGHT:
                    x = sw->xoffset+g_xpos;
                    y = sw->yoffset+g_ypos;
                    w = xevent.xmotion.x_root - sw->xoffset-g_xpos;
                    h = xevent.xmotion.y_root - sw->yoffset-g_ypos;
                    if (w <= QWIN_MIN_WIDTH)
                    {
                        x = sw->xoffset+g_xpos;
                        w = QWIN_MIN_WIDTH;
                    }
                    if (h <= QWIN_MIN_HEIGHT)
                    {
                        y = sw->yoffset+g_ypos;
                        h = QWIN_MIN_HEIGHT;
                    }
                    break;
                default:
                    syslog (LOG_NOTICE, "Unknown g_moving_wnd_seamless_start_size = %ld. Skip.\n", g_moving_wnd_seamless_start_size);
            }

            XMoveResizeWindow(g_display, gq_fake_wnd, x, y, w, h);
            qseamless_draw_fake(w, h);
        }
    }
}


void qprocess_moving_wnd_seamless_move (XEvent xevent)
{
    seamless_window *sw;
    if (g_moving_wnd_seamless && g_moving_wnd_seamless_start_move)
    {
        sw = sw_get_window_by_wnd(xevent.xfocus.window);
        if (!sw)
            return;

        XMoveWindow(g_display, gq_fake_wnd,
                xevent.xmotion.x_root - g_move_x_offset_seamless,
                xevent.xmotion.y_root - g_move_y_offset_seamless);
    }
}

/* Move all windows of the group to new desktop */
void qprocess_send_all_to_desktop(seamless_window *sw_curr)
{
    syslog (LOG_NOTICE, "qprocess_send_all_to_desktop: sw->group->id=%ld\n", sw_curr->group->id);

    seamless_group *sg = sw_find_group(sw_curr->group->id, True);
    if (!sg)
        return;

    unsigned long id = sg->id;
    unsigned int desktop = sw_curr->desktop;

    seamless_window *sw;
    for (sw = g_seamless_windows; sw; sw = sw->next)
    {
        if (sw->group->id == id)
        {
            if (sw->desktop != desktop)
            {
                ewmh_move_to_desktop(sw->wnd, desktop);
                sw->desktop = desktop;
                if (sw->id == g_seamless_focused)
                    g_seamless_focused = 0;
            }
        }
    }
}


/*
SC_CLOSE
0xF060
Closes the window.

SC_CONTEXTHELP
0xF180
Changes the cursor to a question mark with a pointer. If the user then clicks a control in the dialog box, the control receives a WM_HELP message.

SC_DEFAULT
0xF160
Selects the default item; the user double-clicked the window menu.

SC_HOTKEY
0xF150
Activates the window associated with the application-specified hot key. The lParam parameter identifies the window to activate.

SC_HSCROLL
0xF080
Scrolls horizontally.

SCF_ISSECURE
0x00000001
Indicates whether the screen saver is secure.

SC_KEYMENU
0xF100
Retrieves the window menu as a result of a keystroke. For more information, see the Remarks section.

SC_MAXIMIZE
0xF030
Maximizes the window.

SC_MINIMIZE
0xF020
Minimizes the window.

SC_MONITORPOWER
0xF170
Sets the state of the display. This command supports devices that have power-saving features, such as a battery-powered personal computer.
The lParam parameter can have the following values:

-1 (the display is powering on)
1 (the display is going to low power)
2 (the display is being shut off)
SC_MOUSEMENU
0xF090
Retrieves the window menu as a result of a mouse click.

SC_MOVE
0xF010
Moves the window.

SC_NEXTWINDOW
0xF040
Moves to the next window.

SC_PREVWINDOW
0xF050
Moves to the previous window.

SC_RESTORE
0xF120
Restores the window to its normal position and size.

SC_SCREENSAVE
0xF140
Executes the screen saver application specified in the [boot] section of the System.ini file.

SC_SIZE
0xF000
Sizes the window.

SC_TASKLIST
0xF130
Activates the Start menu.

SC_VSCROLL
0xF070
Scrolls vertically.
*/
void qprocess_send_state (seamless_window * sw)
{
    if (!g_seamless_active)
        return;

    int new_state = ewmh_get_window_state(sw->wnd);
    //syslog (LOG_NOTICE, "PropertyNotify:  sw->id=%lx, sw->state=%d, new_state=%d\n", sw->id, sw->state, new_state);
    if (sw->state != new_state)
    {
        if (new_state == SEAMLESSRDP_MAXIMIZED)
        {
            //syslog (LOG_NOTICE, "ClientMessage MAXIMIZE  sw->id=%lx\n", sw->id);
            qseamless_send_message_SY_MAXIMIZE(sw->id);
        }
        else
        if (new_state == SEAMLESSRDP_MINIMIZED)
        {
            //syslog (LOG_NOTICE, "ClientMessage MINIMIZE  sw->id=%lx\n", sw->id);
            qseamless_send_message_SY_MINIMIZE(sw->id);
        }
        else // SEAMLESSRDP_NORMAL
        {
            //syslog (LOG_NOTICE, "ClientMessage RESTORE  sw->id=%lx\n", sw->id);
            qseamless_send_message_SY_RESTORE(sw->id);
        }

        gq_window_manager_process = 1;
        sw->state = new_state;
    }
}


// SI (Set Icon) section begin
#define BI_BITFIELDS 3

typedef struct tagBITMAPINFOHEADER
{
    unsigned long  biSize;
    long           biWidth;
    long           biHeight;
    unsigned short biPlanes;
    unsigned short biBitCount;
    unsigned long  biCompression;
    unsigned long  biSizeImage;
    long           biXPelsPerMeter;
    long           biYPelsPerMeter;
    unsigned long  biClrUsed;
    unsigned long  biClrImportant;
} BITMAPINFOHEADER;

typedef struct tagRGBQUAD
{
    unsigned char rgbBlue;
    unsigned char rgbGreen;
    unsigned char rgbRed;
    unsigned char rgbReserved;
} RGBQUAD;

typedef struct tagBITMAPINFO
{
    BITMAPINFOHEADER bmiHeader;
    RGBQUAD bmiColors[1];
} BITMAPINFO;

static uint32 numberOfColorTableEntries(BITMAPINFOHEADER *pbmi)
{
    uint32 cEntries = 0;

    // Compute the size of the color table.
    if (pbmi->biClrUsed == 0)
    {
        switch (pbmi->biBitCount)
        {
            case 1:
                cEntries = 2;
                break;
            case 4:
                cEntries = 16;
                break;
            case 8:
                cEntries = 256;
                break;
            case 16:
            case 32:
                if (pbmi->biCompression == BI_BITFIELDS)
                {
                    cEntries = 3;
                }
                break;
        }
    }
    else
    {
        cEntries = pbmi->biClrUsed;
    }

    return cEntries;
}

char convertBmpToRgba(BITMAPINFO* pBitmapInfo, char* pBitmapBits, char* quadBuf, int* width, int* height)
{
    char result = 0;
    if (pBitmapInfo->bmiHeader.biWidth <= 32 && abs(pBitmapInfo->bmiHeader.biHeight) <= 32 &&
       (pBitmapInfo->bmiHeader.biBitCount == 16 || pBitmapInfo->bmiHeader.biBitCount == 1))
    {
        result = 1;
    }
    else
    {
        result = 0;
        return result;
    }

    *width = pBitmapInfo->bmiHeader.biWidth;
    *height = abs(pBitmapInfo->bmiHeader.biHeight);

    char* curQuad = quadBuf;
    int i, j, row, redShift = 0, greenShift = 0, blueShift = 0;
    uint16 word, redMask, greenMask, blueMask;
    unsigned char mask[] = {1, 2, 4, 8, 16, 32, 64, 128};
    int bitNumber, colorIndex, alignmentFactor;
    redMask = (uint16)*(uint32*)(&pBitmapInfo->bmiColors[0]);
    greenMask = (uint16)*(uint32*)(&pBitmapInfo->bmiColors[1]);
    blueMask = (uint16)*(uint32*)(&pBitmapInfo->bmiColors[2]);
    for (i = 0; i < 16; ++i)
    {
        if ((1 << i) & redMask)
        {
            redShift = i;
        }
        if ((1 << i) & greenMask)
        {
            greenShift = i;
        }
        if ((1 << i) & blueMask)
        {
            blueShift = i;
        }
    }
    redShift -= 7;
    greenShift -= 7;
    blueShift -= 7;
    if (pBitmapInfo->bmiHeader.biWidth >= 32)
    {
        alignmentFactor = 1;
    }
    else
    {
        alignmentFactor = 32 / pBitmapInfo->bmiHeader.biWidth;
    }
    for (i = 0; i < abs(pBitmapInfo->bmiHeader.biHeight); ++i)
    {
        if (pBitmapInfo->bmiHeader.biHeight > 0)
        {
            row = pBitmapInfo->bmiHeader.biHeight - i - 1;
        }
        else
        {
            row = i;
        }
        for (j = 0; j < pBitmapInfo->bmiHeader.biWidth; ++j)
        {
            switch(pBitmapInfo->bmiHeader.biBitCount)
            {
            case 16:
                word = *(uint16*)(pBitmapBits + 2 * (row * pBitmapInfo->bmiHeader.biWidth + j));
                if (redShift > 0)
                {
                    curQuad[2] = (word & redMask) >> redShift;
                }
                else
                {
                    curQuad[2] = (word & redMask) << abs(redShift);
                }
                if (greenShift > 0)
                {
                    curQuad[1] = (word & greenMask) >> greenShift;
                }
                else
                {
                    curQuad[1] = (word & greenMask) << abs(greenShift);
                }
                if (blueShift > 0)
                {
                    curQuad[0] = (word & blueMask) >> blueShift;
                }
                else
                {
                    curQuad[0] = (word & blueMask) << abs(blueShift);
                }
                break;
            case 1:
                bitNumber = 7 - (j % 8);
                colorIndex = (pBitmapBits[(alignmentFactor * row * pBitmapInfo->bmiHeader.biWidth + j) / 8] &
                              mask[bitNumber]) >> bitNumber;
                *(RGBQUAD*)curQuad = pBitmapInfo->bmiColors[colorIndex];
                break;
            }
            curQuad += 4;
        }
    }
    return result;
}

char decodeBitmap(const char* pBitmap, char* quadBuf, int* width, int* height)
{
    char result = 0;
    BITMAPINFOHEADER bmiHeader;
    BITMAPINFO* pBitmapInfo;
    unsigned char* pBitmapBits;
    uint32 dwValue;
    uint32  i;

    // Check the string
    if ((strlen(pBitmap) / 2) < sizeof(BITMAPINFOHEADER))
    {
        return result;
    }

    // Decode the BITMAPINFOHEADER
    sscanf(&pBitmap[ 0], "%8lx", &bmiHeader.biSize);
    sscanf(&pBitmap[ 8], "%8lx", &bmiHeader.biWidth);
    sscanf(&pBitmap[16], "%8lx", &bmiHeader.biHeight);
    sscanf(&pBitmap[24], "%4x", &dwValue);
    bmiHeader.biPlanes = (uint16)dwValue;
    sscanf(&pBitmap[28], "%4x", &dwValue);
    bmiHeader.biBitCount = (uint16)dwValue;
    sscanf(&pBitmap[32], "%8lx", &bmiHeader.biCompression);
    sscanf(&pBitmap[40], "%8lx", &bmiHeader.biSizeImage);
    sscanf(&pBitmap[48], "%8lx", &bmiHeader.biXPelsPerMeter);
    sscanf(&pBitmap[56], "%8lx", &bmiHeader.biYPelsPerMeter);
    sscanf(&pBitmap[64], "%8lx", &bmiHeader.biClrUsed);
    sscanf(&pBitmap[72], "%8lx", &bmiHeader.biClrImportant);
    pBitmap += 80;

    // Determine the size of the bitmap header and color table.
    uint32 cColors = numberOfColorTableEntries(&bmiHeader);
    int iSize = sizeof(BITMAPINFOHEADER) + (cColors * sizeof(RGBQUAD));

    // Allocate a BITMAPINFO structure.
    pBitmapInfo = (BITMAPINFO*)malloc(iSize + bmiHeader.biSizeImage);
    if (pBitmapInfo == NULL)
    {
        return result;
    }
    pBitmapInfo->bmiHeader = bmiHeader;

    // Decode the color table.
    for (i = 0; i < cColors; i++)
    {
        sscanf(pBitmap, "%8x", &pBitmapInfo->bmiColors[i]);
        pBitmap += (sizeof(RGBQUAD)  * 2);
    }

    // Decode the bitmap bits.
    if (bmiHeader.biSizeImage > 0)
    {
        pBitmapBits = (char*)pBitmapInfo + iSize;
        for (i = 0; i < bmiHeader.biSizeImage; i++)
        {
            sscanf(pBitmap, "%2x", &dwValue);
            pBitmapBits[i] = (unsigned char)dwValue;
            pBitmap += 2;
        }
    }

//    syslog (LOG_NOTICE, "--------- qseamless_process_SI: headerSize %d, height %d, bitCount %d, clrUsed %d, clrImportant %d, sizeImage %d, cColors %d",
//            bmiHeader.biSize, bmiHeader.biHeight, bmiHeader.biBitCount, bmiHeader.biClrUsed, bmiHeader.biClrImportant, bmiHeader.biSizeImage, cColors);

    result = convertBmpToRgba(pBitmapInfo, pBitmapBits, quadBuf, width, height);

    free(pBitmapInfo);

    return result;
}

void qseamless_process_SI(unsigned long id, unsigned long type, char *data)
{
    int chunk = 0;
    char format[] = "RGBA";

    char *pMask, *pColor, *pIcon = data;
    uint32 cbMask, cbColor;

    if (pIcon == NULL)
    {
        return;
    }

    if (sscanf(pIcon, "%8x", &cbMask) != 1)
    {
        return;
    }
    pMask = pIcon + 8;
    if (sscanf(pMask + cbMask, "%8x", &cbColor) != 1)
    {
        return;
    }
    pColor = pMask + cbMask + 8;

    int colorWidth, colorHeight, maskWidth, maskHeight;
    char colorBuf[32*32*4];
    char maskBuf[32*32*4];
    char rgbaBuf[32*32*4];
    if (!decodeBitmap(pColor, colorBuf, &colorWidth, &colorHeight))
    {
        return;
    }
    if (!decodeBitmap(pMask, maskBuf, &maskWidth, &maskHeight))
    {
        return;
    }
    if (colorWidth != maskWidth || colorHeight != maskHeight)
    {
        return;
    }

    int i;
    char *o, *m, *c;
    int size = colorWidth * colorHeight * 4;
    o = rgbaBuf;
    m = maskBuf;
    c = colorBuf;
    for (i = 0; i < size / 4; i++)
    {
        o[0] = c[2];
        o[1] = c[1];
        o[2] = c[0];

        o[3] = 0xff - ((int) (unsigned char) m[0] + (unsigned char) m[1] + (unsigned char) m[2]) / 3;

        o += 4;
        m += 4;
        c += 4;
    }

/*    char byte[3];    byte[2] = '\0';    bufLen = 0;    char iconBuf[4096];
    while (*data != '\0')    {        byte[0] = *data;        data++;
        if (*data == '\0')        {            return False;        }
        byte[1] = *data;        data++;        iconBuf[bufLen] = strtol(byte, NULL, 16);        bufLen++;    }*/

    ui_seamless_seticon(id, format, colorWidth, colorHeight, chunk, rgbaBuf, size);
}
//  SI (Set Icon) section end


